/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't deletee this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut www.vishia.org
 * @version 0.81d  (year-month-day)
 * list of changes:
 * 2007-01-01: JcHartmut www.vishia.org creation from older source
 *
 ****************************************************************************/

#include "ArraysJc.h"
#include "ExceptionJc.h"
#include "ReflectionJc.h"

int binarySearch_Object_ArraysJc(const void* a, const void* key, ComparatorJcREF c) //ComparatorJcRefp c)
{
  ObjectArrayJc* array = (ObjectArrayJc*) a;


	int low = 0;
	int high = array->length-1;

	while (low <= high)
  {
	  int mid = (low + high) >> 1;
    int cmp;
	  const void* elementInArray;
    /* java-sun:
    Object midVal = a[mid];
	  int cmp = c.compare(midVal, key);
    */
    elementInArray = get_i_ObjectArrayJc(array, mid);
    //Use VCALL_IFCJcpp because it is able to compile this source with C without ++

    cmp = compare_ComparatorJc(c, elementInArray, key);

	  if (cmp < 0)     { low = mid + 1; }
	  else if (cmp > 0){ high = mid - 1;}
	  else
		{ return mid; } // key found
	}
	return -(low + 1);  // key not found.
}


METHOD_C void rangeCheck_ArrayJc(int length, int fromIndex, int toIndex, StacktraceThreadContext_s* _threadContext)
{
  STACKTRC_TENTRY("rangeCheck_ArraysJc");

  if(fromIndex > toIndex || fromIndex < 0) { THROW(IllegalArgumentException, "", fromIndex); }
  if(toIndex > length-1)  { THROW(ArrayIndexOutOfBoundsException, "", toIndex); }

  STACKTRC_LEAVE;

}





METHOD_C int binarySearch_STRC_int_ii_ArraysJc(int32ARRAY const* a, int fromIndex, int toIndex, int32 key, StacktraceThreadContext_s* stacktraceThreadContext)
{ rangeCheck_ArrayJc(a->head.length, fromIndex, toIndex, stacktraceThreadContext);
  return binarySearch0_int_ii_ArraysJc(a, fromIndex, toIndex, key);
}





METHOD_C int binarySearch0_int_ii_ArraysJc(int32ARRAY const* a, int fromIndex, int toIndex, int32 key)
{ int low = fromIndex;
	int high = toIndex -1;
	while (low <= high)
  { int mid = (low + high) >> 1;
	  long midVal = a->data[mid];

	  if (midVal < key)
		{ low = mid + 1;
	  }
    else if (midVal > key)
		{ high = mid - 1;
	  }
    else
		{ return mid; // key found
    }
	}
	return -(low + 1);  // key not found.
}



/* old algorithm from me, tested but other behaviour on not founded keys:
  int ix, imin, imax;
  imax=array->length;            //Startwert nach letztem Eintrag
  imin=-1;                  //Startwert vor erstem Eintrag
  while( (imin +1) != imax)
  { const void* elementInArray;
    ix=(imin+imax)/2;        //immer dazwischen
    elementInArray = get_i_ObjectArrayJc(array, ix);
    //Use VCALL_IFCJcpp because it is able to compile this source with C without ++
    if( ( VCALL_IFCJcpp(compare, ComparatorJc, xxx, c)
            elementInArray, key
          _endVCALLJc
        ) > 0
      )
    { imax=ix;
    }            //Item in Tabelle ist groesser, d.h. davor suchen
    else
    {imin=ix;
    }       //Tabelleneintrag kleiner oder gleich, ab dort und dahinter suchen
  }
  ix=imin;     //fuer weiterarbeit


  return ix;
}
*/

