/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content: All declarations to use from user level to work with BlockHeapJc
 * for References of ObjectJc and derivated classes for C-language.
 * The references are used for garbage collection and virtual methods in C.
 *
 * @author JcHartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-15: JcHartmut creation from older sources
 *
 ****************************************************************************/
#ifndef __ObjectRefJc__h
#define __ObjectRefJc__h

/**NOTE: This file is also included in ObjectJc.h, but in this case the __ObjectJc__ is set.
 */
#ifndef __ObjectJc_h__
  #include "ObjectJc.h"
#endif

#ifndef __ReflectionJc_h__
//  #include "ReflectionJc.h"
#endif

/*@This file is included inside ObjectJc.h only if the appropriate defines
  , #if defined(mBackRef_ObjectJc) || defined(mIdxVtbl_ObjectJc)
  are setted in the file ,,CRuntimeJavalike_SysConventions.h,,.
  In this cases the enhanced references are used.

  This file should not be included directly from user.
*/


/** Assigns a reference between the Block of heap, which src is contained in, and refbase.
    The reference itself will be not stored here, it should be done outside because better
    pointer type checking. A back reference to refbase is stored in the block of the heap,
    and the Index of this back reference is stored in refbase.

    If src is not inside a block heap, a 0 value is setted to *refbase.

  @ param refbase the reference
  @ param src any location inside a block in BlockHeap
  @ return true if successfull, false if thrown
  */
METHOD_C bool setBackRefJc(ObjectRefValuesJc* refbase, void const* src);


/** Clears a reference between the Block of heap, which src is contained in, and refbase.
    The reference itself will be not stored here, it should be done outside because better
    pointer type checking. A back reference to refbase is stored in the block of the heap,
    and the Index of this back reference is stored in refbase.

    If src is not inside a block heap, a 0 value is setted to *refbase.

  @ param refbase the reference
  @ return true if successfull, false if thrown
  */
METHOD_C bool clearBackRefJc(ObjectRefValuesJc* refbase);



/** Assigns a reference between the Block of heap, which src is contained in, and refbase.
    The reference itself will be not stored here, it should be done outside because better
    pointer type checking. A back reference to refbase is stored in the block of the heap,
    and the Index of this back reference is stored in refbase.

    If src is not inside a block heap, a 0 value is setted to *refbase.

  @ param refbase the reference
  @ param src any location inside a block in BlockHeap
  @ return true if successfull, false if thrown
  */
METHOD_C bool setBackRefIdxJtblJc(ObjectRefValuesJc* refbase, void const* src, struct ClassJc_t const* reflection);


/**Allocates a block in the current Blockheap or initial with standard malloc.
 */
METHOD_C MemC alloc_sBlockHeapJc(int nrofBytes);


METHOD_C MemC alloc_s_BlockHeapJc(int size);

METHOD_C MemC alloc_BlockHeapJc(struct BlockHeapJc_t* ythis, int size);


METHOD_C MemC alloc_ii_BlockHeapJc(struct BlockHeapJc_t* ythis, int sizeObj, int noOfReferences);

/**activates the access of garbage collector if it is not always done.
 * If the reference is not stored in any enhanced reference using setBackRefJc
 * it is not testet by garbage collector until this time.
 * This method should be called on end of a statement block which allocs the mem.
 *
 * @param mem Memory location of the start of the heap block exactly like getted from alloc_...
 */
METHOD_C void activateGarbageCollectorAccess_BlockHeapJc(MemC mem);

METHOD_C void free_sBlockHeapJc(MemC);

METHOD_C void free_BlockHeapJc(struct BlockHeapJc_t* ythis, struct BlockHeapBlockJc_t* block);


/** Allocates the memory for the given type in current BlockHeap*/
#define alloc_T_sBlockHeapJc(TYPE) ((TYPE*)(alloc_BlockHeapJc(current_sBlockHeapJc(), sizeof(TYPE))))





/** The current BlockHeap. This reference may be dependent on the software component and thread. */
METHOD_C struct BlockHeapJc_t* current_sBlockHeapJc();



#endif //__ObjectRefJc__h
