/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content: implementation of the garbage collector.
 *
 * @author JcHartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-15: JcHartmut creation from older sources
 *
 ****************************************************************************/
//#include "CRuntimeJavalike.h"
//#include "ThreadJc.h"
#include "BlockHeapJc_internal.h"
#include "StringJc.h"
#include "LogMessageJc.h"

//#include <stdio.h>
//#include <stdlib.h>
#include <string.h>







void activateGarbageCollectorAccess_BlockHeapJc(MemC mem)
{
  BlockHeapJc* heap = null;
  BlockHeapBlockJc* block;

  block = deduceBlockHeapBlockFromObject(mem.address, &heap);
  if(heap != null)
  { block->typeOrMaxRef |= mConsideredInGarbage_Type_Object;
  }
  else
  { //do nothing, it may be alloced in startup time.
  }

}








static bool testBlock_GarbageCollectorJc(GarbageCollectorJc* ythis, BlockHeapBlockJc* block)
{ int maxBackref = block->typeOrMaxRef & mTypeOrMaxRef_BlockHeapJc;
  int idxBackref = 0;
  bool bInUse = false;
  STACKTRC_ENTRY("testBlock_GarbageCollectorJc");

  do
  { //test any back reference. If it is a reference from another block, mark it.
    //if it is a reference outside any blockheaps, the block is used.
    ObjectJcREF* ref = block->head.backRefs[idxBackref];  //the backRef contains a reference to the reference to this block.
    if(ref != null)
    { //look where the reference to this block is located:
      BlockHeapJc* retHeap;
      BlockHeapBlockJc* blockWithReference = deduceBlockHeapBlockFromObject(ref, &retHeap);
      if(  retHeap != null    //the reference is in any heap.
        && (blockWithReference->typeOrMaxRef & mConsideredInGarbage_Type_Object) //the block is to be tested in GC
        )
      { /** The reference to this block is located in another or this block.
         *  It is possible that this other block is referenced from any other or not.
         *  The blockWithReference builds a cluster with all the other blocks with references between it.
         *  If any block of the cluster have a reference outside, the blocks are used.
         *  But if no reference from outside is found, all of the blocks of the cluster are freed.
         */
        if( blockWithReference->nextBlock == null) //and the block isnot already observed by GC
        { blockWithReference->nextBlock = block->nextBlock;
          block->nextBlock = blockWithReference;
          { //report:
            if(ythis->idxReportBuffer >= sizeof(ythis->reportBuffer)-32)
            { sendMsg_LogMessageJc(ythis->log, 1001, ythis->reportBuffer, _threadContext);
              ythis->idxReportBuffer = sprintf(ythis->reportBuffer, "...GC ");
            }
            ythis->idxReportBuffer += sprintf(ythis->reportBuffer + ythis->idxReportBuffer, ", %p", blockWithReference);
          }
        }
        else
        { /**The block is already the queue and part of the cluster.
           * It will be tested later.
           */
        }
      }
      else
      { /**A reference outside a GC-controlled block indicates, that the block is referenced, in use.
         * Therefore all blocks of the cluster are used.
         */
        bInUse = true;
      }
    }
  }while(!bInUse && ++idxBackref < maxBackref);
  STACKTRC_LEAVE; return bInUse;
}






/**This method starts with one block and collects and tests all blocks, there have back references to the first block
 * or to a always tested block of the cluster. If no reference from outside is found to any block of the cluster,
 * the method returns false. All tested blocks are marked started with the first block with a chain of blocks,
 * using the nextBlock-association of the BlockHeapBlockJc-struct.
 * Therefore started from the first block the cluster is known from outside.
 *
 * @param block The first block to test.
 * @return true if a outside reference is found.
 */
static bool testBlockCluster_GarbageCollectorJc(GarbageCollectorJc* ythis, BlockHeapBlockJc* block)
{
  BlockHeapBlockJc* firstBlock = block;
  bool bInUse = false;
  STACKTRC_ENTRY("testBlockCluster_GarbageCollectorJc");
  block->nextBlock = kLastBlockInGarbageCtest_BlockHeapJc;  //mark it with !=null
  { //report:
    if(ythis->idxReportBuffer >0)
    { sendMsg_LogMessageJc(ythis->log, 1001, ythis->reportBuffer, _threadContext);
    }
    ythis->idxReportBuffer = sprintf(ythis->reportBuffer, "GC:test block=%p ref:", block);
  }
  do
  { /* test all blocks of a cluster which are referencing one to another,
     * pick up all this blocks and test wether a from outside is found to a block of this cluster
     */
    ythis->countStep +=1;
    bInUse = testBlock_GarbageCollectorJc(ythis, block);
    if(ythis->testMethod != null)
    { ythis->testMethod(ythis->countStep);
    }
    //either all references are tested, and no external reference is found, or a external reference is found.
    if(!bInUse)
    { /**no exterenal reference is found, but the cluster of blocks may increased, test the next:
       */
      block = block->nextBlock;
    }
  }while(  !bInUse                                        //break if any block is used, than the cluster is used.
        && block != kLastBlockInGarbageCtest_BlockHeapJc  //break if all blocks of the cluster are tested.
        && !theGarbageCollectorJc.bAbortTest              //break if any block in cluster is reused.
        );
  //
  if(theGarbageCollectorJc.bAbortTest)
  { if(ythis->idxReportBuffer >= sizeof(ythis->reportBuffer)-32)
    { sendMsg_LogMessageJc(ythis->log, 1001, ythis->reportBuffer, _threadContext);
      ythis->idxReportBuffer = sprintf(ythis->reportBuffer, "...GC ");
    }
    ythis->idxReportBuffer += sprintf(ythis->reportBuffer + ythis->idxReportBuffer, " -abort because reuse of any block");
    bInUse = true;
  }
  //
  if(bInUse)
  { /* At least one block of this cluster of blocks is used from outside,
     * Than all blocks of the cluster are used, because the others are used from blocks.
     * Or the test is aborted because a block of the cluster is resused in any way. The test have to repeat than.
     * Thad do not delete it.
     * But the queue of blocks should be disintegrated.
     */
    do
    { //remove the chain of tested blocks from the blocks to prevent disturbance of a later check.
      block = firstBlock;
      firstBlock = block->nextBlock;
      block->nextBlock = null;
    } while(firstBlock != kLastBlockInGarbageCtest_BlockHeapJc);
    theGarbageCollectorJc.bAbortTest = false;
  }
  else
  { //all blocks were tested, they are not referenced from outside.
    do
    { BlockHeapJc* blockHeap;
      block = firstBlock;      //process all blocks of the cluster.
      deduceBlockHeapBlockFromObject(block, &blockHeap);  //to get the heap control structure.
      firstBlock = block->nextBlock;
      memset(block, 0, blockHeap->bytesNormalBlock);
      block->typeOrMaxRef = kFree_Type_BlockHeapBlock;
      { //report
        if(ythis->idxReportBuffer >= sizeof(ythis->reportBuffer)-32)
        { sendMsg_LogMessageJc(ythis->log, 1001, ythis->reportBuffer, _threadContext);
          ythis->idxReportBuffer = sprintf(ythis->reportBuffer, "...GC ");
        }
        ythis->idxReportBuffer += sprintf(ythis->reportBuffer + ythis->idxReportBuffer, " => freed=%p", block);
      }
      blockHeap->lastFreeBlock->nextBlock = block;
      blockHeap->lastFreeBlock = block;
    } while(firstBlock != kLastBlockInGarbageCtest_BlockHeapJc);
  }/*while: test all references from outside to all blocks of this cluster*/
  //
  STACKTRC_LEAVE; return bInUse;
}









boolean garbageCollection__GarbageCollectorJc(GarbageCollectorJc* ythis, int logIdFreed, int logIdUsed)
{
  static BlockHeapJc* testedHeap = null;
  static int idxBlock = -1;        //because first increment, it starts with 0

  BlockHeapBlockJc* block;
  bool bInUse = false;

  STACKTRC_ENTRY("garbageCollection_BlockHeapJc");

  if(testedHeap == null)
  { testedHeap = theBlockHeapList;
    idxBlock = -1;
  }  //load on first run.

  do
  { //this loop searches the next used block.
    if(++idxBlock >= testedHeap->zBlock)
    { //evaluate the next heap
      idxBlock = 0;
      testedHeap = testedHeap->nextHeap;  //null at end
    }
    block = testedHeap == null ? null : (BlockHeapBlockJc*)( addOffset_MemAreaC(testedHeap->heapBegin, idxBlock * testedHeap->bytesNormalBlock));

  } while(  block != null          //break if all heaps are pass through.
         && ( (block->typeOrMaxRef & mTypeOrMaxRef_BlockHeapJc) == kFree_Type_BlockHeapBlock  //break if a used block is found.
            ||(block->typeOrMaxRef & mConsideredInGarbage_Type_Object) == 0  //loop if block is not considered by GC
            )
         );

  if(block != null)
  { //a used block is found.
    bInUse = testBlockCluster_GarbageCollectorJc(ythis, block);
    if(!bInUse)
    { //all blocks were tested, they are not referenced from outside.
      sendMsg_LogMessageJc(ythis->log, logIdFreed, ythis->reportBuffer, _threadContext);
    }/*while: test all references from outside to all blocks of this cluster*/
    else
    {
      sendMsg_LogMessageJc(ythis->log, logIdUsed, ythis->reportBuffer, _threadContext);
    }
  }/*if a used block is found. */
  STACKTRC_LEAVE;
  return !bInUse;  //true if a block is freed.
}



/**A static instance of the singleton garbage collector class. */
GarbageCollectorJc theGarbageCollectorJc = {0};


boolean garbageCollection_BlockHeapJc(struct LogMessageJc_t* log, int logIdFreed, int logIdUsed)
{
  /**A buffer to write reports running GC. */
  bool bFreed;
  theGarbageCollectorJc.log = log;
  theGarbageCollectorJc.idxReportBuffer = 0;
  bFreed = garbageCollection__GarbageCollectorJc(&theGarbageCollectorJc, logIdFreed, logIdUsed);
  return bFreed;
}


void setTestMethod(MT_int_Method_int testMethod)
{ theGarbageCollectorJc.testMethod = testMethod;
}


/**Reports the properties of one block after another.
 * This Method may be called in a while loop, but it is not reentrantable for the same BheapJc.
   It is a report method typically called in one thread. The reason is: The pointer to the
   last reported block is stored in the BheapJc-Structure. The next call is respected to
   this pointer.
 * @param buffer The Buffer to put the info into. The buffer will be cleared before.
 */
StringJc report_BheapJc(BlockHeapJc* ythis, StringBufferJc* buffer)
{ StringJc ret;
  int ii = 0;
  BlockHeapBlockJc* block = (BlockHeapBlockJc*)(ythis->heapBegin);
  while(ii < ythis->zBlock)
  { if( (block->typeOrMaxRef & mTypeOrMaxRef_BlockHeapJc) != kFree_Type_BlockHeapBlock)
    {

    }
  }

  ret.refbase = ii;
  //ret.ref = buffer->
  return ret;

}





