/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't deletee this Copyright/Copyleft inscription in this source file.
 *
 * @content implementation of routines for exception handling
 *
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut new: routine to test the consistence of stacktrace.
 * 2007-07-00: JcHartmut creation
 *
 ****************************************************************************/

#include "StringJc.h"
#include "ExceptionJc.h"

const char* exceptionTexts[33]=
{ "RuntimeException"  //0x0001
, "ClassCastException"   //0x0002
, ""   //0x0004
, ""   //0x0008
, "IndexOutOfBoundsException"   //0x0010
, "ArrayIndexOutOfBoundsException"   //0x0020
, "StringIndexOutOfBoundsException"   //0x0040
, ""   //0x0080
, "IllegalArgumentException"   //0x0100
, "NoSuchElementException"   //0x0200

, "IllegalStateException"   //0x0400
, ""   //0x0800
, ""   //0x1000
, ""   //0x2000
, ""   //0x4000
, ""   //0x8000
, "NoSuchFieldException"   //0x00010000
, ""   //0x00020000
, ""   //0x00040000
, ""   //0x00080000
, ""   //0x00100000
, ""   //0x00200000
, ""   //0x00400000
, ""   //0x00800000
, ""   //0x01000000
, ""   //0x02000000
, ""   //0x04000000
, ""   //0x08000000
, ""   //0x10000000
, ""   //0x20000000
, ""   //0x40000000
, ""   //0x80000000

, "commonException"   //0x0


};




void throw_sJc(int32 exceptionNr, StringJc msg, int value, StacktraceThreadContext_s* stacktraceThreadContext, int line)
{ //find stack level with try entry:
  if(stacktraceThreadContext != null)
  { StacktraceJc* stacktrace = stacktraceThreadContext->stacktrace;
    StacktraceElementJcARRAY* stacktraceEntriesInThreadContext = stacktraceThreadContext->stacktraceBuffer;
    int idxStacktraceEntries = 0;
    int idxStacktraceEntriesMax;
    idxStacktraceEntriesMax = stacktraceEntriesInThreadContext == null ? -1 : stacktraceEntriesInThreadContext->head.length;
    stacktrace->entry.line = line;
    while
    (  stacktrace != null             //false only if no try-level found, if the end of the stacktrace is reached
    && stacktrace->tryObject == null  //while try-level not reached
    )
    { if(idxStacktraceEntries < idxStacktraceEntriesMax)
      { //fill in infos
        stacktraceEntriesInThreadContext->data[idxStacktraceEntries] = stacktrace->entry; //NOTE: it is a memcpy.
        idxStacktraceEntries += 1;
      }
      stacktrace = stacktrace->previous;
    }
    stacktraceThreadContext->stacktrace = stacktrace;  //may be null if no TRYJc-level is found.
    stacktraceThreadContext->nrofEntriesStacktraceBuffer = idxStacktraceEntries;
    //
    //the stacktraceThreadContext->stacktraceBuffer is filled with the followed levels of Stacktrace,
    //the stacktrace refers the level of the TRY or it is null.
    //
    if(stacktrace != null)
    { //TRY-level is found:
      ExceptionJc* exception = &stacktrace->tryObject->exc;
      stacktrace->tryObject->exceptionNr = stacktrace->tryObject->exc.exceptionNr = exceptionNr;  //for longjmp
      exception->exceptionNr = exceptionNr;
      lightCopy_StringJc(&exception->exceptionMsg, msg);
      exception->exceptionValue = value;
      exception->backStacktrace = stacktrace;
      exception->nrofStacktraceEntries = idxStacktraceEntries;
      exception->stacktraceEntries = stacktraceEntriesInThreadContext;
      //throw exception, use the platform dependend variant.
      #if defined(__TRYCPPJc) //&& defined(__cplusplus)
       throw exceptionNr;
      #else
       longjmp(stacktrace->tryObject->longjmpBuffer, exceptionNr);
      #endif

    }
    else
    { //no TRYJc-level found,
      ExceptionJc exception;
      exception.exceptionNr = exceptionNr;
      lightCopy_StringJc(&exception.exceptionMsg, msg);
      exception.exceptionValue = value;
      exception.backStacktrace = stacktrace;  //it is null!
      exception.nrofStacktraceEntries = idxStacktraceEntries;
      exception.stacktraceEntries = stacktraceEntriesInThreadContext;

      uncatched_ExceptionJc(&exception, stacktraceThreadContext);
      while(true)
      { ObjectJc obj;
        wait_ObjectJc(&obj,100);  //STOP the execution.
      }
    }
  }
  else
  { //no TRYJc-level found,
    ExceptionJc exception;
    exception.exceptionNr = exceptionNr;
    lightCopy_StringJc(&exception.exceptionMsg, msg);
    exception.exceptionValue = value;
    exception.backStacktrace = null;
    exception.nrofStacktraceEntries = 0;
    exception.stacktraceEntries = 0;

    uncatched_ExceptionJc(&exception, stacktraceThreadContext);
    while(true)
    { ObjectJc obj;
      wait_ObjectJc(&obj,100);  //STOP the execution.
    }
  }
}



void throw_s0Jc(int32 exceptionNr, const char* msgP, int value, StacktraceThreadContext_s* stacktraceThreadContext, int line)
{ StringJc msg = s0_StringJc(msgP);
  throw_sJc(exceptionNr, msg, value, stacktraceThreadContext, line);
}


METHOD_C const char* getExceptionText(int32 exceptionNr)
{ int idxText = 0;
  const char* sText;
  if(exceptionNr != 0)
  { while((exceptionNr & 0x00000001) == 0){ exceptionNr >>=1; idxText +=1; }
  }
  else
  { idxText = 32;
  }
  sText = exceptionTexts[idxText];
  return sText;
}



METHOD_C void printStackTrace_ExceptionJc(ExceptionJc* ythis, FILE* out)
{ //DEF_stacktraceThreadContextJc
  int idxStacktraceEntries = 0;
  int nrofStacktraceEntriesMax;
  char sBuffer[100];
  StacktraceElementJcARRAY* stacktraceEntries = ythis->stacktraceEntries;
  StacktraceJc* stacktrace = ythis->backStacktrace;
  //ExceptionJc* exception = stacktrace->exception;
  const char* sException = getExceptionText(ythis->exceptionNr);

  copyToBuffer_StringJc(&ythis->exceptionMsg, sBuffer, sizeof(sBuffer));
  fprintf(out, "\n%s: %s: %i=0x%8.8X \n",sException, sBuffer, ythis->exceptionValue, ythis->exceptionValue);

  nrofStacktraceEntriesMax = stacktraceEntries == null ? -1 : ythis->nrofStacktraceEntries;
  while(idxStacktraceEntries < nrofStacktraceEntriesMax)
  { //the entries after try-level
    StacktraceElementJc* entry = &stacktraceEntries->data[idxStacktraceEntries++];
    fprintf(out, "  at %s (%s:%i)\n", entry->name, entry->source, entry->line);
  }

  while(stacktrace != null)
  { //the entries before try-level
    StacktraceElementJc* entry = &stacktrace->entry;
    fprintf(out, "  at %s (%s:%i)\n", entry->name, entry->source, entry->line);
    stacktrace = stacktrace->previous;
  }

}




/**Test the consistence of the stacktrace, useable if errors are searched*/
bool test_StacktraceJc(StacktraceJc* ythis)
{ boolean bCont = true;
  StacktraceJc* stacktrace, *stacktracePrev;
  stacktrace = ythis;
  while(bCont)
  { stacktracePrev = stacktrace->previous;
    if(stacktracePrev == null)
    { bCont = false;
    }
    else
    { if(stacktracePrev <= stacktrace)
      { bCont = false;
      }
      else if(stacktracePrev > stacktrace + 0x200000)
      { bCont = false;
      }
      else
      {
        stacktrace = stacktracePrev;
      }
    }
  }
  return stacktracePrev == null;
}





#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)

StacktraceJcpp::StacktraceJcpp(const char* sName, ThreadContextJc* stacktraceThreadContext)
{ if(stacktraceThreadContext == null) { stacktraceThreadContext = getCurrent_ThreadContextJc(); }
  this->stacktraceThreadContext = stacktraceThreadContext;
  entry.name = sName;
  tryObject = null;
  if(stacktraceThreadContext != null)
  { //if the stacktraceThreadContext is under construction while calling this routine,
    //it cannot be used.
    previous = stacktraceThreadContext->stacktrace;
    stacktraceThreadContext->stacktrace = this;
  }
}

#endif
