#ifndef __pkgC_RuntimeJavalike_ExceptionJc_h__
#define __pkgC_RuntimeJavalike_ExceptionJc_h__
/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content definition of user interface for exception handling
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut new: routine to test the consistence of stacktrace.
 * 2007-07-00: JcHartmut creation
 *
 ****************************************************************************/

#ifndef __ObjectJc_h__
  #include "ObjectJc.h"
#endif

//#include "ThreadContextJc.h"
//#include "ThreadJc.h"

typedef struct StacktraceThreadContext_t
{
  /**Pointer to the actual stacktrace entry.
   * This pointer is used and setted in any routine using the macro STACKthread_ENTRY and STACKthread_LEAVE.
   */
  struct StacktraceJc_t* stacktrace;

  /**actual nrofEntries i stacktraceBuffer. */
  int32 nrofEntriesStacktraceBuffer;

  /**Space for Stacktrace Buffer. */
  struct StacktraceElementJcARRAY_t* stacktraceBuffer;

  struct StringBufferJc_t* excMsg;
  /*NOTE: The element stacktraceBuffer have to be the last because some additional StackEntryJc may be added on end.*/

}StacktraceThreadContext_s;

METHOD_C StacktraceThreadContext_s* getCurrent_StacktraceThreadContext();

#define DEF_stacktraceThreadContextJc StacktraceThreadContext_s* _threadContext = getCurrent_StacktraceThreadContext()


#include <stdio.h>


/*@CLASS_C ExceptionJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/



#ifndef __TRYCPPJc
  #include <setjmp.h>
#endif


typedef enum ExceptionIdentsJc_t
{ ident_ExceptionJc                = 0x00000000
, ident_RuntimeExceptionJc         = 0x00000001
, ident_ClassCastExceptionJc       = 0x00000002
, ident_IndexOutOfBoundsExceptionJc=       0x00000010
, ident_ArrayIndexOutOfBoundsExceptionJc=  0x00000020
, ident_StringIndexOutOfBoundsExceptionJc= 0x00000040
, ident_IllegalArgumentExceptionJc = 0x00000100
, ident_NoSuchElementExceptionJc   = 0x00000200
, ident_IllegalStateExceptionJc    = 0x00000400

, ident_NoSuchFieldExceptionJc     = 0x00010000
}ExceptionIdentsJc;

typedef enum ExceptionMasksJc_t
{
  mask_ExceptionJc                 = 0xffffffff
, mask_RuntimeExceptionJc          = 0x0000ffff
, mask_ClassCastExceptionJc        = 0x00000002

, mask_IndexOutOfBoundsExceptionJc =      0x00000070
, mask_ArrayIndexOutOfBoundsExceptionJc=  0x00000020
, mask_StringIndexOutOfBoundsExceptionJc= 0x00000040
, mask_IllegalArgumentExceptionJc  = 0x000000100
, mask_NoSuchElementExceptionJc    = 0x000000200
, mask_IllegalStateExceptionJc    = 0x000000400

, mask_NoSuchFieldExceptionJc      = 0x00010000
}ExceptionMasksJc;




/**The Exception data contains all data of exception but references to the stacktrace.
 *
 */
typedef struct ExceptionJc_t
{
  /**Bit mask of the exception. There are a maximum of 32 Exception types. Every Exception is represented by one bit.
     See enum definition of ExceptionMasksJc.
   */
  int32 exceptionNr;

  /**The user message of the exception.
   */
  StringJc exceptionMsg;

  /**The user value of the exception.
   */
  int32 exceptionValue;

  /**The stacktrace backward to root as linked chain in the stack started from current stack level.
   * If this object is transfered to another levels or threads as the actual level because it is saved anywhere,
   * this reference should be null. Because the stack is not kept.
   */
  struct StacktraceJc_t* backStacktrace;

  /**Actual nr of valid entries in stacktraceEntries. Note: The array head contains the maximal size. */
  int nrofStacktraceEntries;

  /**Reference to the array of stacktrace entries from throw point.
   * In a originaly CATCHJc-block this reference refers to the array in the _threadContext.
   * But if this object is transfered to another levels or threads as the actual level because it is saved anywhere,
   * the list should be copied in heap and completed with the ,,backStacktrace,, informations.
   * Because the stacktraceEntries on _threadContext are not kept if other exceptions are thrown.
   * Use the method ,,safe_ExceptionJc(),, to do that.
   */
  struct StacktraceElementJcARRAY_t* stacktraceEntries;
  //struct{ const char* name; int line;} stacktraceBuffer[20];
}ExceptionJc;




#ifndef __TRYCPPJc
  #define NULL_ExceptionJc() { {0}, 0, NULL_StringJc(), 0 }
#else
  #define NULL_ExceptionJc() { 0, NULL_StringJc(), 0 }
#endif


/**Javalike: prints the Stacktrace at output stream. */
METHOD_C void printStackTrace_ExceptionJc(ExceptionJc* ythis, FILE* out);

/**Special: manifests the content of the stacktrace to a given structure.
 * It means, all information holded in the stack itself via previous pointers from each StacktraceJc-structure
 * or holded in the _threadContext is copied to the destination strutures.
 *
 * The element backStacktrace is set to null after this operation,
 *
 * @param dst a provided buffer for the exception or null. If null a new element is allocated in heap.
 * @param dst a provided buffer for the stacktrace or null. If null a new array with the designated length is allocated in heap.
 */
METHOD_C ExceptionJc* manifest_ExceptionJc(ExceptionJc* ythis, ExceptionJc* dst, struct StacktraceElementJcARRAY_t* dstStacktrace);

/**This routine is called in the THROW processing, if no TRY-level is found. The user should write this method.*/
METHOD_C void uncatched_ExceptionJc(ExceptionJc* ythis, StacktraceThreadContext_s* _threadContext);
//METHOD_C void uncatchedException(int32 exceptionNr, StringJcRef*  msg, int value, StacktraceThreadContext_s* _threadContext);



//#define null_ExceptionJc() {0}

/*@CLASS_C StacktraceElementJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


typedef struct StacktraceElementJc_t
{
  const char* name;
  const char* source;
  int line;
}StacktraceElementJc;





/*@CLASS_C TryObjectJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/






typedef struct TryObjectJc_t
{
/**The exception object here temporary in stack*/
  ExceptionJc exc;
  /**The exceptionNr is always contained in exc, but here doubled because it is set to 0 */
  int32 exceptionNr;
  #ifndef __TRYCPPJc
    jmp_buf longjmpBuffer;
  #endif

}TryObjectJc;




/*@CLASS_C StacktraceJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


typedef struct StacktraceJc_t
{ struct StacktraceJc_t* previous;
  StacktraceElementJc entry;

  /**Exception-reference if there is an exception, or null */
  TryObjectJc* tryObject;
}StacktraceJc;



//#define  THREADcontextP StacktraceThreadContext_s* _threadContext



/**This macro defines and initializes the stack variable ,,_threadContext,, and ,,stacktrace,,.
 *
 */

#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)
  /**C++-Variant: The StacktraceJcpp with its destructor is used, no STACKTRCE_LEAVE is necessary.
   * After the macro in the users call a semicolon should be written.
  */
  /* NOTE: The initialization with __FILE__ and __LINE__ must be a part of macro
   * because otherwise it is the fault file and line.
   */
  #define STACKTRC_ENTRY(NAME) DEF_stacktraceThreadContextJc; StacktraceJcpp stacktrace(NAME,_threadContext);\
  stacktrace.entry.source = __FILE__; stacktrace.entry.line = __LINE__
#else
  /**C-Variant: Use the macro ,,STACKTRC_LEAVE;,, at end of the block unconditionally!*/
  #define STACKTRC_ENTRY(NAME) \
    DEF_stacktraceThreadContextJc; \
    StacktraceJc stacktrace = {_threadContext->stacktrace, { NAME, __FILE__, __LINE__} , null}; \
    if(_threadContext != null) { _threadContext->stacktrace = &stacktrace; }
#endif

/**This macro defines and initializes the stack variables ,,stacktrace,,.
 * Use the macro ,,STACKTRC_LEAVE;,, at end of the block unconditionally!
 * After the macro in the users call a semicolon should be written.
 */
/* NOTE: The initialization with __FILE__ and __LINE__ must be a part of macro
 * because otherwise it is the fault file and line.
 */
#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)
  #define STACKTRC_TENTRY(NAME) StacktraceJcpp stacktrace(NAME,_threadContext);\
  stacktrace.entry.source = __FILE__; stacktrace.entry.line = __LINE__

#else
  /* NOTE: The variable stacktracePtr is only necessary because in C it should be a variable declaration.
   * The reason is: some more variable declarations may be followed after the STACKTRC_ENTRY() macro.
   * Its tricky. The optimizer will be remove it.
   */
  #define STACKTRC_TENTRY(NAME) \
    StacktraceJc stacktrace = {_threadContext->stacktrace, { NAME, __FILE__, __LINE__} , (TryObjectJc*)(null)}; \
    StacktraceJc* stacktracePtr = (_threadContext->stacktrace = &stacktrace)
#endif
/**This macro corrects the chained list of stacktrace, it sets the end of the previous stacktrace.
 * Use this macro unconditionally at end of a block using ,,STACKTRC_ENTRY(),, or ,,STACKTRC_XENTRY(),,.
 * Otherwise the chain of stacktrace elements is corrupted.
 */
#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)
  #define STACKTRC_LEAVE
#else
  #define STACKTRC_LEAVE _threadContext->stacktrace = stacktrace.previous
#endif


/**Test the consistence of the stacktrace, useable if errors are searched*/
METHOD_C bool test_StacktraceJc(StacktraceJc* ythis);



/**@deprecated. This macro is only present because compatibility with older sources. It is only useable in C++.
 */
#define STACKTRC_ROOT_ENTRY(NAME) STACKTRC_ENTRY(NAME);

/*@CLASS_CPP @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/



#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)

class StacktraceJcpp: public StacktraceJc
{
  private: StacktraceThreadContext_s* _threadContext;

  public: StacktraceJcpp(const char* sName, StacktraceThreadContext_s* _threadContext = null);

  public: ~StacktraceJcpp()
  { if(_threadContext != null) { _threadContext->stacktrace = previous;}
  }

};




#endif  /*__CPLUSPLUSJcpp*/


/*@DEFINE_C TRYJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/



/**throws an exception. This method is called directly in a THROW macro. It may be called immediately
 * without an _threadContext if an uncatchable exception occurs.
 * @param _threadContext if null than the uncatchedException-routine is called.
 */
METHOD_C void throw_sJc(int32 exceptionNr, StringJc msg, int value, StacktraceThreadContext_s* _threadContext, int line);



METHOD_C void throw_s0Jc(int32 exceptionNr, const char* msg, int value, StacktraceThreadContext_s* _threadContext, int line);







#define CALLINE (stacktrace.entry.line=__LINE__)

#ifdef __TRYCPPJc
  #define TRY \
  { TryObjectJc tryObject = {0}; \
    stacktrace.tryObject = &tryObject; \
    stacktrace.entry.line = __LINE__; \
    tryObject.exceptionNr = 0; \
    try
#else
  #define TRY \
  { TryObjectJc tryObject = {0}; \
    stacktrace.tryObject = &tryObject; \
    stacktrace.entry.line = __LINE__; \
    { tryObject.exceptionNr = setjmp(tryObject.longjmpBuffer); \
      if(tryObject.exceptionNr==0) \
      {
#endif

//#if defined(_JcPLUSPLUSJcpp) && defined(_Jcplusplus)
/**Write at end of a TRY-Block the followed macro: */
#ifdef __TRYCPPJc
  #define _TRY \
  catch(...) { tryObject.exc.backStacktrace = &stacktrace; tryObject.exc.nrofStacktraceEntries = _threadContext->nrofEntriesStacktraceBuffer; tryObject.exc.stacktraceEntries = _threadContext->stacktraceBuffer; stacktrace.tryObject = null; \
  if(false) {

#else
  #define _TRY tryObject.exc.backStacktrace = &stacktrace; tryObject.exc.nrofStacktraceEntries = _threadContext->nrofEntriesStacktraceBuffer; tryObject.exc.stacktraceEntries = _threadContext->stacktraceBuffer; stacktrace.tryObject = null;
#endif


#define CATCH(EXCEPTION, EXC_OBJ) \
  } else if((tryObject.exceptionNr & mask_##EXCEPTION##Jc)!= 0/* || tryObject.exceptionNr==ident_##EXCEPTION##Jc*/) \
  { ExceptionJc* EXC_OBJ = &tryObject.exc; tryObject.exceptionNr = 0;  /*do not check it a second time.*/






#define FINALLY \
  } /*close FINALLY, CATCHJc or TRYJc brace */\
  } /*close brace of whole catch block*/ \
  { { /*open to braces because _finishTRY has 2 closing braces.*/



/**Write on end of the whole TRY-CATCH-Block the followed macro:*/
#define END_TRY \
  } /*close FINALLY, CATCHJc or TRYJc brace */\
  } /*close brace of whole catch block*/ \
  if(tryObject.exceptionNr != 0) /*instead else of CATCHJc blocks, notice, FINALLYJc may be assigned after CATCHJc-Blocks. */\
  { /* delegate exception to previous level*/ \
    stacktrace.tryObject = null; /*Do not use the own longjmp!!! */ \
    throw_sJc(tryObject.exc.exceptionNr, tryObject.exc.exceptionMsg, tryObject.exc.exceptionValue, _threadContext, __LINE__); \
  } \
  } /*close brace from beginning TRYJc*/


/**Throws an exception.
 * @param EXCPETION ones of the defines in ExceptionIdentsJc, but without ident_ and Jc. It's the same like Exception class names in Java.
 *        example RuntimeException or IndexOutOfBoundsException
 * @param TEXT type StringJc. To get a StringJc from a string literal, write s0_StringJc("my text")
 * @param VAL a int value
 */
#define THROW(EXCEPTION, TEXT, VAL)  throw_s0Jc(ident_##EXCEPTION##Jc, TEXT, VAL, _threadContext, __LINE__)

#define THROW_s(EXCEPTION, TEXT, VAL)  throw_sJc(ident_##EXCEPTION##Jc, TEXT, VAL, _threadContext, __LINE__)


//old #define RETURNJc(VALUE) { stacktrace.line = -32767; returnJc = VALUE;}





/*@CLASS_C StacktraceBufferJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**Array of Stacktraceentries. */
typedef struct StacktraceElementJcARRAY_t
{ ObjectArrayJc head;
  /**The size of entries if this struct is staticly created. It is also the debug-vizible-size.*/
  #define STATICSIZE_StacktraceElementJcARRAY 20
  /**Some entries. The real number may be shortenly or greater, denpendend on the creation of the object.
   * The real size is stored in the head.
   */
  StacktraceElementJc data[STATICSIZE_StacktraceElementJcARRAY];
}StacktraceElementJcARRAY;

#define new_StacktraceElementJcARRAY(SIZE) ((StacktraceElementJcARRAY*)(new_ObjectArrayJc(SIZE, sizeof(StacktraceElementJc), null, 0)))

#endif //__pkgC_RuntimeJavalike_ExceptionJc_h__
