/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content definition of entries for container like java.util
 *
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/
#ifndef ListMapEntryJc_i__H_
#define ListMapEntryJc_i__H_

//#include "RuntimeHeapJc.h"
#include "ObjectJc.h"

/** The analogue in Java is TreeMap.Entry! See there.
*/






/** Definition of 1 Entry in the Mempool*/
typedef struct MapEntryJc_t
{
  /** The parent node of a TreeMap*/
  struct MapEntryJc_t* parent_Map;

  /** The left node of a TreeMap or the prev node of a LinkedList*/
  struct MapEntryJc_t* left_Map;

  /** The right node of a TreeMap or the next node of a LinkedList*/
  struct MapEntryJc_t* right_Map;

  /**Reference to the instance such as a LinkedList or TreeMap which uses this node.*/
  ObjectJc* xxxallocater;

  //int32 mark_Map;   //mark of the using see EColor

  /** Reference to the key object of a TreeMap.
      the reference contains a reference to the parent Object,
      this is the LinkedList, TreeMap and others,
      for backtrace references for the garbage collection
  */
  ObjectJcREF key_Map;

  /** Reference to the object of a Linked List, TreeMap or others.
      the reference contains a reference to the parent Object,
      this is the LinkedList, TreeMap and others,
      for backtrace references for the garbage collection
  */
  ObjectJcREF obj_Map;



} MapEntryJc;



typedef struct ListEntryJc_t
{
  /** The parent node of a TreeMap*/
  struct ListEntryJc_t* unused_parent_Map;

  /** The left node of a TreeMap or the prev node of a LinkedList*/
  struct ListEntryJc_t* previous;

  /** The right node of a TreeMap or the next node of a LinkedList*/
  struct ListEntryJc_t* next;

  /**Reference to the instance such as a LinkedList or TreeMap which uses this node.*/
  ObjectJc* xxxallocater;

  //int32 mark_Map;   //mark of the using see EColor

  /** Reference to the key object of a TreeMap.
      the reference contains a reference to the parent Object,
      this is the LinkedList, TreeMap and others,
      for backtrace references for the garbage collection
  */
  ObjectJcREF unused_key_Map;

  /** Reference to the object of a Linked List, TreeMap or others.
      the reference contains a reference to the parent Object,
      this is the LinkedList, TreeMap and others,
      for backtrace references for the garbage collection
  */
  ObjectJcREF element;



} ListEntryJc;




typedef struct FreeEntryJc_t
{
  /** The parent node of a TreeMap*/
  union ListMapEntryJc_t* nextFree;
  #define LAST_FreeEntryJc ((union ListMapEntryJc_t*)(-1))
  /**Reference to the instance such as a LinkedList or TreeMap which uses this node.*/
  ObjectJc* xxxallocater;



}FreeEntryJc;




typedef union ListMapEntryJc_t
{ ListEntryJc listEntry;
  MapEntryJc mapEntry;
  FreeEntryJc freeEntry;
}ListMapEntryJc;





typedef enum EColor { kBlack = 0, kRed = 0x1 }; //mark of a item for TreeMap
typedef enum { mBlackRed = 0x1 };        //bits in
typedef enum { kNoEntry = -1 };          //Index


METHOD_C ListMapEntryJc* searchFreeItem_MapEntry(ObjectJc* parent);  //throws runtimeException if no memory
    void releaseItem_MapEntry(ListMapEntryJc* ythis);



    /** skip to the requested item in a double chain.
        Skipping from top if the item is nearly on top, and from end it is nearly end.
        @param iList pos of item in the chain
        @param zList nr of items in the list.
        @param iFirst index in the memory pool of the first item
        @param iLast index in the memory pool of the last item
        @return the index in memPool of the found item
    */
METHOD_C ListMapEntryJc* skip_MapEntry(int iList, int zList, ListMapEntryJc* iFirst, ListMapEntryJc* iLast);

    //ListMapEntryJc* next_MapEntry(ListMapEntryJc* ythis) { return ythis->right_Map}
    #define next_MapEntry(ythis) (ythis->right_Map)
    #define prev_MapEntry(ythis) (ythis->left_Map)
    #define right_MapEntry(ythis) (ythis->right_Map)
    #define left_MapEntry(ythis) (ythis->left_Map)
    #define parent_MapEntry(ythis) (ythis->parent_Map)
    //bool isRed_MapEntry(ListMapEntryJc* ythis) { return(ythis->mark_Map & mBlackRed) == kRed}
    #define isRed_MapEntry(ythis) ((ythis->mark_Map & mBlackRed) == kRed)
    #define isBlack_MapEntry(ythis) ((ythis->mark_Map & mBlackRed) == kBlack)

    //void setLeft_MapEntry(ListMapEntryJc* ythis, ListMapEntryJc* left){ ythis->left_Map = left; }
    #define setLeft_MapEntry(ythis, left){ ythis->left_Map = left; }
    #define setRight_MapEntry(ythis, right){ ythis->right_Map = right; }
    #define setPrev_MapEntry(ythis, left){ ythis->left_Map = left; }
    #define setNext_MapEntry(ythis, right){ ythis->right_Map = right; }



    /** assign the obj in a new item and concat it with the item at ix.
        @param parentRef reference to the Object administered this Entry, this is the
             instance of a LinkedList, TreeMap and others.
        @param obj the Object to be refered
        @param iBefore  the index in the memory pool after the Position, or -1 if not after exists
        @param iAfter the index in the memory pool after the Position, or -1 if not after exists
        @return the index of the new item
    */
METHOD_C ListMapEntryJc* add_MapEntry(ObjectJc* parentRef, ObjectJcREF obj, ListMapEntryJc* iBefore, ListMapEntryJc* iAfter);

    /** search a new Entry and set the values, exception if no memory
        @param parentRef reference to the Object administered this Entry, this is the
               instance of a LinkedList, TreeMap and others.
        @param obj the Object to be referenced by this entry
    */
METHOD_C ListMapEntryJc* xxnew_MapEntry(ObjectJc* parentRef, ObjectJcREF obj, ObjectJcREF key, ListMapEntryJc* parent);

/** search a new Entry and set the values, exception if no memory
    @xxxparam parentRef reference to the Object administered this Entry, this is the
           instance of a LinkedList, TreeMap and others.
    @param obj the Object to be referenced by this entry
*/
METHOD_C ListMapEntryJc* new_ListMapEntryJc(ListMapEntryJc** emptyEntries, struct BheapJc_t* blockHeap); //, ListMapEntryJc* parent, ListMapEntryJc* iBefore, ListMapEntryJc* iAfter);


    /** get the Object in memPool.
      @param ix index in memPool
      @return Object
    */

METHOD_C ObjectJcREF setObj_MapEntry(ListMapEntryJc* ythis, ObjectJcREF obj);

    #define    get_MapEntry(ythis) ( ythis->obj_Map )
    #define getObj_MapEntry(ythis) ( ythis->obj_Map )
    #define getKey_MapEntry(ythis) ( ythis->key_Map )
    #define setKey_MapEntry(ythis, key) { ythis->key_Map = *key; }






#endif
