/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content Implementation of ObjectJc
 *
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut adap:The elements idSemaphore and idWaitingThread are combined to idSyncHandles.
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/
#include <stdlib.h>     //malloc
#include <string.h>     //memset
#include "ObjectJc.h"  //:NOTE: after stdlib.h because malloc maybe defined.
#include "StringJc.h"
#include "ReflectionJc.h"
#include "ExceptionJc.h"
//#include "os_wrapper.h"

/********************************************************************************************/
//Object


ObjectJc* ctor_ObjectJc(MemC mem) //ObjectJc* ythis)
{ ObjectJc* ythis = (ObjectJc*)mem.address;
  ythis->ownAddress = ythis;
  ythis->offsetToHeapBlock = offsetBlock_MemC(mem);  //NOTE 0 if no offset is given.
  ythis->idSyncHandles = kNoSyncHandles_ObjectJc;
  //
  //NOTE: the both next values cannot be setted from focus ObjectJc,
  //      they should be setted outside the ctor behind them from focus of the derivated class.
  //      use setReflection_ObjectJc(..) therefore.
  ythis->objectIdentSize = 0;
  ythis->reflectionClass = null;
  return ythis;
}


void setReflection_ObjectJc(ObjectJc* ythis, struct ClassJc_t const* reflection,  int32 typeInstanceSizeIdent)
{ ythis->reflectionClass = reflection;
  ythis->objectIdentSize = typeInstanceSizeIdent;
}




struct ClassJc_t const* getClass_ObjectJc(ObjectJc const* ythis)
{ return ythis->reflectionClass;//->theClass;
}


ObjectJc* clone_Object_F(ObjectJc const* ythis, MemC buffer)
{
  return null;
}

bool equals_Object_F(ObjectJc const* ythis, ObjectJc const* cmp)
{
  return false;
}

void finalize_Object_F(ObjectJc* ythis)
{
}

int32 hashCode_Object_F(ObjectJc const* ythis)
{
  return 0;
}



int getIdentInfo_ObjectJc(ObjectJc const* ythis)
{ int32 identsize = ythis->objectIdentSize;
  if(identsize & mIsLargeSize_objectIdentSize_ObjectJc) return (identsize >>24) & 0x1f;
  else if(identsize & mSizeBits_objectIdentSize_ObjectJc == kIsSmallSize_objectIdentSize_ObjectJc) return (identsize >>16) & 0x0fff;
  else return (identsize >>20) & 0x0ff;
}



StringJc toString_Object_F(ObjectJc* ythis)
{ //StringBuffer* ss = StringBuffer::new_();
  //ss->append("Object at @").append(Integer::toHexString((int)(this)));
  StringJc ss = CONST_StringJc("No information"); //only constant
  return ss;   //return by value, that is 2 words (8 Byte).
}


//typedef StringJc (*Method_toString_Object)(ObjectJc_rp);





/********************************************************************************************/
//Object__Array


void* get_i_ObjectArrayJc(ObjectArrayJc const* ythis, int idx)
{
  if(idx < 0 || idx > ythis->length){ return null; } //IndexOutOfBoundaryException
  else
  {
    int offset = idx * ythis->sizeElement;
    if(true) //ythis->mode == kDirect_ObjectArrayJc)
    {
      return ((MemUnit*)(ythis + 1)) + offset;
    }
    else
    { return 0;
    }
  }
}


int size_Object__ArrayJc(const ObjectArrayJc* ythis)
{
  return ythis->length;
}


/** Gets the number of bytes of the whole array calculated from the informations inside the head.
  */
METHOD_C int getNrOfBytes_ObjectArrayJc(ObjectArrayJc const* ythis)
{ return sizeof(ObjectArrayJc) + ythis->length * ythis->sizeElement;
}





METHOD_C ObjectArrayJc* ctor_ObjectArrayJc(ObjectArrayJc* ythis, int nSize, int nBytesPerElement, ClassJc const* reflection, int32 typeInstanceIdent)
{ //ObjectArrayJc* arrayHead = (ObjectArrayJc*)(ythis);  //the universal type of ARRAYJc
  if(ythis != null)
  { MemC mem;
    mem.address = (MemAreaC*)ythis;   setSizeAndOffset_MemC(mem, sizeof(ObjectArrayJc), 0);
    ctor_ObjectJc(mem); //&ythis->object);
    { int32 sizeArray = nSize * nBytesPerElement + sizeof(ObjectArrayJc);
      if( (typeInstanceIdent & mSizeBits_objectIdentSize_ObjectJc) == kIsSmallSize_objectIdentSize_ObjectJc)
      { if(sizeArray > mSizeSmall_objectIdentSize_ObjectJc)
        { STACKTRC_ENTRY("ctor_ObjectArrayJc");
          THROW(IndexOutOfBoundsException, "too large", sizeArray);
          STACKTRC_LEAVE;
        }
        typeInstanceIdent &= ~mSizeSmall_objectIdentSize_ObjectJc;
      }
      else if( (typeInstanceIdent & mSizeBits_objectIdentSize_ObjectJc) == kIsMediumSize_objectIdentSize_ObjectJc)
      { if(sizeArray > mSizeMedium_objectIdentSize_ObjectJc)
        { STACKTRC_ENTRY("ctor_ObjectArrayJc");
          THROW(IndexOutOfBoundsException, "too large", sizeArray);
          STACKTRC_LEAVE;
        }
        typeInstanceIdent &= ~mSizeMedium_objectIdentSize_ObjectJc;
      }
      else if(typeInstanceIdent & mIsLargeSize_objectIdentSize_ObjectJc)
      { if(sizeArray > mSizeLarge_objectIdentSize_ObjectJc)
        { STACKTRC_ENTRY("ctor_ObjectArrayJc");
          THROW(IndexOutOfBoundsException, "too large", sizeArray);
          STACKTRC_LEAVE;
        }
        typeInstanceIdent &= ~mSizeLarge_objectIdentSize_ObjectJc;
      }
      else
      { STACKTRC_ENTRY("ctor_ObjectArrayJc");
        THROW(IndexOutOfBoundsException, "undefined size", typeInstanceIdent);
        STACKTRC_LEAVE;
      }
      typeInstanceIdent |= sizeArray;
      //it is always 1-dimension.
      typeInstanceIdent = (typeInstanceIdent & ~mArray_objectIdentSize_ObjectJc) | kArray_objectIdentSize_ObjectJc;
    }
    setReflection_ObjectJc(&ythis->object, reflection, typeInstanceIdent);

    ythis->length = nSize;
    ythis->sizeElement = nBytesPerElement;
    ythis->mode = 0; //kDirect_ObjectArrayJc;
    { MemUnit* data = (MemUnit*)(ythis) + sizeof(ObjectArrayJc);  //The data are assigned always immediately after the head. This is the law.
      memset(data, 0, nSize * nBytesPerElement);
    }
  }
  return ythis;
}


METHOD_C ObjectArrayJc* new_ObjectArrayJc(int SIZE, int BYTESELEMENT, struct ClassJc_t const* REFLECTION, int32 typeInstanceIdent)
{ int nrOfBytes = sizeof(ObjectArrayJc) + SIZE * BYTESELEMENT;
  ObjectArrayJc* address;
  //STACKTRC_ENTRY("new_ObjectArrayJc");
  address = (ObjectArrayJc*)(malloc(nrOfBytes));
  //if(address != null){ memset(address, 0, nrOfBytes); }
  ctor_ObjectArrayJc(address, SIZE, BYTESELEMENT, REFLECTION, typeInstanceIdent);
  //STACKTRC_LEAVE;
  return address;
  //return &address->object;

}









Int32ARRAY* ctor_Int32ARRAY(Int32ARRAY* ythis, int nrOfElements)
{ if(ythis != null)
  { ythis->head.length = nrOfElements;
    ythis->head.sizeElement = sizeof(int32);
    ythis->head.mode = 0; //kDirect_ObjectArrayJc;
    memset(ythis->data, 0, nrOfElements * ythis->head.sizeElement);
  }
  return ythis;
}





/** This method body is inside ObjectJc.c because simple applications (ExceptionJc.c)
  * call only this method from string, without other dependencis.
  */
METHOD_C StringJc s0_StringJc(const char* src)
{ StringJc ret;
  ret.refbase = strlen(src); // | mIsConstant_StringJc;
  ret.ref = src;
  //ret.ref.ref = src;
  return ret;
}







/* ******************************************************************************************/
/* MemC
*/

MemC alloc0_MemC(int size)
{ MemC mem;
  setSizeAndOffset_MemC(mem, size, 0);
  mem.address = (MemAreaC*)malloc(size);
  if(mem.address)
  { memset(mem.address, 0, size);
  }
  return mem;
}
