#ifndef __ReflectionJc_h__
#define __ReflectionJc_h__
/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content Interface for Reflection
 *
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut rename: MT_void_Method_void instead VoidMethodVoid etc.
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/

#include "ObjectJc.h"
//#include "StringJc.h"


/*@CLASS_C FieldJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/** A field is one entry in the reflection information about attributes or references of any type.
  * In opposite to orignal-Java a FieldJc is not based on Object,
  * but the Field[]-type, in Jc named "FieldJcArray" is based on ObjectJc like every ARRAYJc.
  * It means at ex you cannot call toString_ObjectJc(myField.object), it results in a compiler syntax error.
  * The reason if the FieldJc-type do not based on ObjectJc is: preserve of expenditure, it is not necessary.
  * A field may describe:
  * * a simle scalar attribute of type such as int or long, the type field references to ,,ReflectionintJc,, and so on.
  * * a reference to any other type, the instance may be derived also, the type field references to ,,ReflectionReferenceType,,.
  * * a embedded other type. That is a possibility only in C(++), not possible in original Java.
  * : but the Java2C translator produces such constructs because optimization, and a hand write code
  * : can use this because optimization. The type field references to ;;ReflectionEmbeddedType,,,
  * : but the modifier-bit mEmbedded_Modifier_reflectJc is set in bitModifiers.
  */

/*
typedef enum ScalarTypes_ClassJc_t
{ int64_TypeJc = 1, int32_TypeJc, int16_TypeJc, int8_TypeJc
, uint64_TypeJc, uint32_TypeJc, uint16_TypeJc, uint8_TypeJc
, float_TypeJc, double_TypeJc
, char8_TypeJc, char8Array_TypeJc
, char16_TypeJc, char16Array_TypeJc
, void_TypeJc
, ObjectJc_TypeJc , ObjectArrayJc_TypeJc
, StringJc_TypeJc
, ClassJc_TypeJc
, reflection__ObjectRefValuesJc
}ScalarTypes_ClassJc;
*/

typedef enum ScalarTypes_TypeJc_t
{
  void_TypeJc       = 0x01,
  int64_TypeJc      = 0x02,
  uint64_TypeJc     = 0x03,
  int32_TypeJc      = 0x04,
  uint32_TypeJc     = 0x05,
  int16_TypeJc      = 0x06,
  uint16_TypeJc     = 0x07,
  int8_TypeJc       = 0x08,
  uint8_TypeJc      = 0x09,
  int_TypeJc        = 0x0a,
  uint_TypeJc       = 0x0b,
  float_TypeJc      = 0x0c,
  double_TypeJc     = 0x0d,
  char_TypeJc       = 0x0e,
  bool_TypeJc       = 0x0f,
  afterlastPrimitive_TypeJc       = 0x18
}ScalarTypes_TypeJc;










/**The next defines are necessary because the automatic code generation generates such types if void * or void* or char* is written.*/
#define REFLECTION_void_Pointer       ((struct ClassJc_t const*) void_TypeJc )
#define REFLECTION_voidPointer       ((struct ClassJc_t const*) void_TypeJc )
#define REFLECTION_charPointer       ((struct ClassJc_t const*) void_TypeJc )
#define REFLECTION_char_Pointer       ((struct ClassJc_t const*) void_TypeJc )

/**Reflection of standard types. */
#define REFLECTION_void               ((struct ClassJc_t const*) void_TypeJc )
#define REFLECTION_int64              ((struct ClassJc_t const*) int64_TypeJc )
#define REFLECTION_uint64             ((struct ClassJc_t const*) uint64_TypeJc)
#define REFLECTION_int32              ((struct ClassJc_t const*) int32_TypeJc )
#define REFLECTION_uint32             ((struct ClassJc_t const*) uint32_TypeJc)
#define REFLECTION_int16              ((struct ClassJc_t const*) int16_TypeJc )
#define REFLECTION_uint16             ((struct ClassJc_t const*) uint16_TypeJc)
#define REFLECTION_int8               ((struct ClassJc_t const*) int8_TypeJc  )
#define REFLECTION_uint8              ((struct ClassJc_t const*) uint8_TypeJc )
#define REFLECTION_int                ((struct ClassJc_t const*) int_TypeJc   )
#define REFLECTION_uint               ((struct ClassJc_t const*) uint_TypeJc  )
#define REFLECTION_long               ((struct ClassJc_t const*) uint32_TypeJc)
#define REFLECTION_float              ((struct ClassJc_t const*) float_TypeJc  )
#define REFLECTION_float32            ((struct ClassJc_t const*) float_TypeJc  )
#define REFLECTION_double             ((struct ClassJc_t const*) double_TypeJc )
#define REFLECTION_char               ((struct ClassJc_t const*) char_TypeJc  )
#define REFLECTION_bool               ((struct ClassJc_t const*) bool_TypeJc  )
#define REFLECTION_boolean            ((struct ClassJc_t const*) bool_TypeJc )
#define REFLECTION_BOOL               ((struct ClassJc_t const*) bool_TypeJc )

#define REFLECTION_s32                ((struct ClassJc_t const*) int32_TypeJc )
#define REFLECTION_u32                ((struct ClassJc_t const*) uint32_TypeJc)
#define REFLECTION_s16                ((struct ClassJc_t const*) int16_TypeJc )
#define REFLECTION_u16                ((struct ClassJc_t const*) uint16_TypeJc)
#define REFLECTION_s08                ((struct ClassJc_t const*) int8_TypeJc  )
#define REFLECTION_u08                ((struct ClassJc_t const*) uint8_TypeJc )

/*
#define REFLECTION_bool               ((struct ClassJc_t const*) 0x0f )
#define REFLECTION_bool               ((struct ClassJc_t const*) 0x0f )
#define REFLECTION_bool               ((struct ClassJc_t const*) 0x0f )
#define REFLECTION_bool               ((struct ClassJc_t const*) 0x0f )
*/

#define REFLECTION_ObjectifcBaseJcpp ((struct ClassJc_t const*) 0x18 )

//#define REFLECTION_ObjectJc          ((struct ClassJc_t const*) 0x19 )
//#define REFLECTION_ObjectJcpp        ((struct ClassJc_t const*) 0x1a )
#define REFLECTION_Object_ArrayJc    ((struct ClassJc_t const*) 0x1b )
#define REFLECTION_StringJc          ((struct ClassJc_t const*) 0x1d )
//#define REFLECTION_ClassJc           ((struct ClassJc_t const*) 0x20 )


#define REFLECTION_ObjectJc          ((struct ClassJc_t const*) 0x41 )
#define REFLECTION_ObjectJcpp        ((struct ClassJc_t const*) 0x42 )
#define REFLECTION_ObjectRefValuesJc ((struct ClassJc_t const*) 0x44 )
#define REFLECTION_ClassJc           ((struct ClassJc_t const*) 0x45 )
#define REFLECTION_ClassJc_t         ((struct ClassJc_t const*) 0x45 )

#define kREFLECTION_LastConstant      0x50

typedef struct  FieldJc_t
{ //ObjectJc super;

  /** The symbolic name of the field*/
  //StringJcRef  name;
  char name[kLengthNAME_FIELD_ReflectionJc];

  /**Nr of elements of an static array*/
  int16 nrofArrayElements;

  /** The class object representing the type of the field.
   * It may be a class object also for primitive types
   * or for primitive Type there is a simple constant value, see REFLECTION_type above.
   */
  struct ClassJc_t const* type;

  /**The modifiers of the Field. The Bits are defined in Modifier_reflectJc.
   * The concept follows Java, see java.lang.reflect.Field.getModifier or java.lang.reflect.Modifier,
   * but it is enhanced in respect to C/C++ and UML requirements.
   */
  int32     bitModifiers;  //Bits of Modifier_reflectJc.

  /** position of the field inside a object of the type of the declaring class.
    * The position is the offset from the ObjectJc base class to the field, not the offset from the total base.
    * Because it is possible that the object is known via interface reference. A interface reference
    * pointered the position of the pointer to the virtual table of the interface,
    * a call of obj->toObjectJc() is necessary to get the base for this offset.
    */
  int16     position;

  /**Offset in a pointered class to the ObjectifcBaseJcpp class structure if such base interface exists.
     This value is 0 if it is not a pointer or if it is a pointer of not Object-derivated structures.
   */
  int16     offsetToObjectifcBase;
  /** The class object representing the class or interface that declares the field represented by this Field object. */
  struct ClassJc_t const* declaringClass;
}FieldJc;



#define TYPESIZEOF_FieldJc (kIsSmallSize_typeSizeIdent_ObjectJc + 0x0FF60000 + sizeof(FieldJc))
#define OBJTYPE_FieldJc (kIsSmallSize_objectIdentSize_ObjectJc + 0x0FF50000)



//#define const_FieldJc(SNAME, NAME, TYPE, MODI) { /*CONST_ObjectJc(TYPESIZEOF_FieldJc, null), */const_String(SNAME), TYPE, MODI, OFFSET_FieldJc(NAME), &DECLARING_CLASS_FieldJc.reflection }
#define const_FieldJc(SNAME, NAME, TYPE, MODI) { SNAME, 0, TYPE, MODI, OFFSET_FieldJc(NAME), 0, &DECLARING_CLASS_FieldJc.clazz }

//#define CONST_ARRAYFIELDJc(SNAME, NAME, TYPE, MODI) { SNAME, TYPE, MODI, ( (int)&( ((CLASS_CURRENTJc*)(0x1000))->data->NAME ) - (int)&( ((CLASS_CURRENTJc*)(0x1000))->data )), TYPE, &BASE_REFLECTION_CURRENTJc.CLASS_CURRENTJc.clazz }
/** Defines the constant value of a field instance.
  * The followed defines have to be defined before:
  * *REFLECTION_CURRENTJc
  */
#define CONST_ARRAYFIELDJc(SNAME, NAME, TYPE, MODI) { SNAME, 0, TYPE, MODI, ( (int)&( ((CLASS_CURRENTJc*)(0x1000))->data->NAME ) - (int)&( ((CLASS_CURRENTJc*)(0x1000))->data )), 0, REFLECTION_CURRENTJc }

/**Defines a field as constant structure.
 * @param SNAME name of field element as string literal.
 * @param NAME identifier name of the field element in the C-structure.
 * @param TYPE type of the name, with Reflection_NAME the associated classJc is designated.
 * @param MODI Mofifier, using values of enum type Modifier_reflectJc_t.
 */
//#define CONST_FIELDJc(SNAME, NAME, TYPE, MODI) { SNAME, TYPE, MODI,(int)(&( ((CLASS_CURRENTJc*)(0x1000))->NAME )) - (int)((ObjectJc*)((CLASS_CURRENTJc*)(0x1000))), REFLECTION_CURRENTJc }
#define CONST_FIELDJc(SNAME, OFFSET, TYPE, MODI) { SNAME, 0, TYPE, MODI, OFFSET, 0, REFLECTION_CURRENTJc }

/**Defines a field as constant empty structure.
 */
#define NULL_FIELDJc() { "", 0, null, 0, 0, null }

/** Constant Definition of a FieldJc-instance inside a whole reflection definition.
  * @param CLASS Name of the Type wherein this field is defined. The Value is used  to calculate the offset of the element
  *              and to build the pointer to the declaring class in Form reflection##CLASS.clazz.
  * @param SNAME String given name of the field. It should be identical with NAME, but with quotion marks.
  * @param NAME  identifier of the element inside the Type CLASS.
  * @param TYPE  The type of the the element, the reflection class is build with reflection##TYPE
  * @param MODI  Modifier, see CLASS_C ModifierJc.
  */
#define CONST_FieldJc(CLASS, SNAME, NAME, TYPE, MODI) { SNAME, 0, REFLECTION_##TYPE, MODI, (int)&( ((CLASS*)(0x1000))->NAME ) - 0x1000, 0 }
//#define CONST_FieldJc(CLASS, SNAME, NAME, TYPE, MODI) { SNAME, 0, &reflection##TYPE.clazz, MODI, (int)&( ((CLASS*)(0x1000))->NAME ) - 0x1000, &reflection##CLASS.clazz }


#define const_ObjectJcpp_FieldJc(SNAME, NAME, TYPE) const_FieldJc(SNAME, NAME, TYPE, mObjectJcpp_FieldJc)
#define const_ObjectJc_FieldJc  (SNAME, NAME, TYPE) const_FieldJc(SNAME, NAME, TYPE, mObjectJc_FieldJc)




/**Macro for a constant head of FieldJc array useable to define the constants for reflection. New since 2007.09
 */
#define CONST_ARRAYHEAD_FieldJc(NAME) CONST_ObjectArrayJc(FieldJc, ARRAYLEN(REFLECTION_IMAGE.attributes_##NAME.data), 0, null, &REFLECTION_IMAGE.attributes_##NAME)

/**Macro for a constant head of FieldJc array useable to define the constants for reflection. New since 2007.09
 */
#define CONST_ARRAYHEAD_z_FieldJc(SIZE, OWNADDRESS) CONST_ObjectArrayJc(FieldJc, SIZE, 0, null, OWNADDRESS)

#define getName_FieldJc(THIS) ((THIS)->name)

#define getType_FieldJc(THIS) ((THIS)->type)

#define getDeclaringClass_FieldJc(THIS) ((THIS)->declaringClass)

#define getModifiers_FieldJc(THIS) ((THIS)->bitModifiers)



#define getStaticArraySize_FieldJc(THIS) ((THIS)->nrofArrayElements)

/**Gets the absolute adress of the element represented by this field inside obj. It is assumed that the obj represented that object,
   which contains really this field. Otherwise the behavior is undefined.
   @param obj the Object. It shouln't derivated from ObjectJc, it may be any struct or class,
              but the type should matched to the ClassJc description getted with getDeclaringClass_FieldJc(ythis).
   @param idx Index if the FieldJc describes, that it's an array like element. Otherwise the idx is not used.
*/
METHOD_C void* getMemoryAddress_FieldJc(const FieldJc* ythis, void const* obj);


/**Gets the address of the referenced element.
 * @param address The address of the reference, it may be the address of a container element.
 * @param idx the index
 * @throws IndexOutOfBoundsException if the idx is fault.
 */
METHOD_C void* getRefAddr_FieldJc(const FieldJc* ythis, void const* address, int idx);

/**Gets the absolute address of the indexed element of a UML-specific container element known as UML_LinkedList.
 * The algorithm should be specified in the users area (extra file.c) because it is UML-Framework-specific.
 * @param address The address of the container element.
 * @param idx the index
 * @throws IndexOutOfBoundsException if the idx is fault.
 */
METHOD_C void* getRefAddrUML_LinkedList_FieldJc(void const* address, int idx);

/**Gets the absolute address of the indexed element of a UML-specific container element known as UML_ArrayList.
 * The algorithm should be specified in the users area (extra file.c) because it is UML-Framework-specific.
 * @param address The address of the container element.
 * @param idx the index
 * @throws IndexOutOfBoundsException if the idx is fault.
 */
METHOD_C void* getRefAddrUML_ArrayList_FieldJc(void const* address, int idx);

/**Gets the absolute address of the indexed element of a UML-specific container element known as UML_Map.
 * The algorithm should be specified in the users area (extra file.c) because it is UML-Framework-specific.
 * @param address The address of the container element.
 * @param idx the index
 * @throws IndexOutOfBoundsException if the idx is fault.
 */
METHOD_C void* getRefAddrUML_Map_FieldJc(void const* address, int idx);


/**Gets the object represented by the Field ythis from an given object.
 * If it is not a complex datatype but an simple type (int, float and others),
 * this method returns null. This is in opposite to Java, java wrappes the primitive type.
 * If it is a C++ class it may be possible, that the field contains
 * the pointer to any baseclass or interface inside the instance. This method returns the pointer
 * to the baseclass ObjectJc of the instance.
 * @param obj The Object. In opposite to Java it may not based on Object.
 *     The declaring class of the FieldJc must match to the type of Object, that is the condition.
 *     If the Class description of the declaring class of the Field don't request, that obj is based on ObjectJc,
 *     the Field describes at example a simple structure. But the getField-method isn't able to test it.
 * @param idx Index, used if the Field describes an ObjectArrayJc or an container or UML-container.
 *     This is an additional argument in opposite to Java. The reason is: In Java an array or container everytime
 *     is accessible outside the reflections, because the containers such as LinkedList have own reflections
 *     and own access methods, and the arrays have the own Element length.
 *     But in C there is the possibility to describe elements of arrays or containers with this Field information,
 *     but only the address of the container or ObjectArrayJc-base is given. The get-Method allows the access to
 *     this elements. If no idx is necessary, the idx is ignored.
 */
METHOD_C void* get_FieldJc(FieldJc const* ythis, void* obj, int idx);


#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)
  inline void* get_FieldJc(FieldJc const* ythis, ObjectifcBaseJcpp* obj, int idx)
  { return get_FieldJc(ythis, obj->toObjectJc(), idx);
  }
#endif




/**Gets the Object and the describing class of the Object from given FieldJc and given object.
 * If the object is non based on ObjectJc, it hasn't own reflection infos inside it,
 * but the reflection info to the pointer to it is known as a Class type, returned by getType_FieldJc.
 * In this case this information is supplied.
 * The class info always matches to the returned obj.
 * If the field describes a primitive type, it returns null and sets obj to null.
 *
 * @param obj src obj as input to which the FieldJc matches. Because reflections are also
 *            useable in C for non-ObjectJc-inherited instances, it is a void*-reference.
 * @param retObj Reference of type void* for the getted obj as output. It may be null if this output is not necessary.
 *
 * @param idx Index, used if the Field describes an ObjectArrayJc or an container or UML-container.
 *     This is an additional argument in opposite to Java. The reason is: In Java an array or container everytime
 *     is accessible outside the reflections, because the containers such as LinkedList have own reflections
 *     and own access methods, and the arrays have the own Element length.
 *     But in C there is the possibility to describe elements of arrays or containers with this Field information,
 *     but only the address of the container or ObjectArrayJc-base is given. The get-Method allows the access to
 *     this elements. If no idx is necessary, the idx is ignored.
 */
METHOD_C struct ClassJc_t const* getClassAndObj_FieldJc(FieldJc const* ythis, void* obj, int idx, void** retObj);



/**Searches an object with given start object and path. This is a static method in context of ClassJc.
 * The returned object may be based on ObjectJc or not. In C/C++ systems, especially in user oriented data structures,
 * the inheritance of ObjectJc is not taken for granted. In this cases the describing ClassJc information will be kept as getType_FieldJc()
 * in the Field information of this element, not in the implementing instance.
 *
 * @param sPath The path to the object. At example ,,"myObj.theElement[20].x",,.
 * @param startObj The object from which the path starts, it have to be based on ObjectJc.
 * @param field Reference to a FieldJc reference, it will be set as return with the last found Field information.
 * @param idx Reference to a int, it will be set as return with the last found index information in path,
 *            or set to -1 if no index is given at last position.
 * @return The founded object matching to the returned field. The Object is the last but one part of the VariablePath,
 *         but the returned field is the last part of the Variable path. The returned idx is >=0 if the last part
 *         contains an index information.
 */
METHOD_C void* searchObject_ClassJc(StringJc sPath, ObjectJc* startObj, FieldJc const** field, int* idx);



/**Gets the actual length of the array if the field describes a reference of an array or collection,
 * @param obj The Object matching to the Field
 * @return the actual length, it may be 0.
 * @throws NoSuchFieldException if it describes not an array or collection.
 */
METHOD_C int getArraylength_FieldJc(FieldJc const* ythis, void const* obj);

METHOD_C int getArraylengthUML_LinkedList_FieldJc(void const* address);


METHOD_C int getArraylengthUML_ArrayList_FieldJc(void const* address);


METHOD_C int getArraylengthUML_Map_FieldJc(void const* address);


#define setFloat_Field(YTHIS, OBJ, VALUE) (*(float*)(getMemoryAddress_FieldJc(YTHIS, (OBJ)->toObjectJc(), STACKTRC)) = VALUE)

#define getFloat_Field(YTHIS, OBJ) (*(float*)(getMemoryAddress_FieldJc(YTHIS, (OBJ)->toObjectJc(), STACKTRC)))

/** Returns the reference to the field inside the given object. This method is not javalike.
  * Because the type of the field is not known in this context, the reference is of type void*.
  * The user should cast it outside to the type of the field. The casting is only admissible
  * to the exactly type of the field.
  * If it is a primitive type, the reference may be casted to the reference to the primitive type,
  * such as (int*). If the reference is a complex type, especially a interface in C++, the cast
  * is only admissible to the interface type, not to a supposed derivated type. The casting to
  * a derivated type in C++ is only admissible with static_cast<type>, because the compiler
  * should adjust the reference.
  */
METHOD_C void* getReference_FieldJc(FieldJc const* ythis, ObjectJc* obj);

/** Sets a reference at the desired field in the given object obj. The field should be
  * a reference type of a scalar or not object-based instance. In other cases (if it is a scalar direct
  * or a object base reference), the method throws IllegalArgumentException.
  * Use set_FieldJc to assign a reference to a object-based instance. The method
  * don't check anything. This method isn't exist in java, because java don't know references.to
  * not object base instances.
  * @param obj The object to set the value inside.
  * @param value The reference to the destination instance to set in the object's field.
  * @exception IllegalArgumentException if the methode doesn't match to the arguments.
  */
METHOD_C void setReference_FieldJc(FieldJc const* ythis, ObjectJc* obj, void* value);

METHOD_C int getInt_FieldJc(const FieldJc* ythis, void* obj);

METHOD_C void setInt_FieldJc(const FieldJc* ythis, void* obj, int val);

METHOD_C float getFloat_FieldJc(const FieldJc* ythis, void* obj);

METHOD_C double getDouble_FieldJc(const FieldJc* ythis, void* obj);

METHOD_C bool  getBool_FieldJc(const FieldJc* ythis, void* obj);

METHOD_C void setFloat_FieldJc(const FieldJc* ythis, void* obj, float val);

METHOD_C int64 getInt64_FieldJc(const FieldJc* ythis, void* obj);


//void setFloat_FieldJc(const FieldJc* ythis, ObjectJc* obj, float value);



/** Field[] consists of ObjectArrayJc and some FieldJc elements directly after them.
*/
typedef struct FieldJcArray_t
{ /** Base data of every array */
  //ObjectArrayJc array;
  ObjectArrayJc head;
  /** For debugging, 10 Elements are assumed. The real number of values is stored in array.len*/
  FieldJc data[10];
}FieldJcArray;


#define FieldARRAY FieldJcArray
//METHOD_C const FieldJc* get_FieldARRAYc(const FieldARRAY* ythis, int idx);






#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)

class  FieldJcpp: public FieldJc
{
  public: ObjectJcpp* get(ObjectJcpp* obj, int idx) const
  { return static_cast<ObjectJcpp*>(get_FieldJc(this, obj->toObjectJc(), idx));
  }

  public: ClassJcpp const* getType() const;

  public: void* getMemoryAddress(ObjectJcpp* obj) const
  { return getMemoryAddress_FieldJc(this, obj->toObjectJc());
  }

  public: int getInt(ObjectJc* obj) const
  { return getInt_FieldJc(this, obj);
  }

  public: void setInt(ObjectJcpp* obj, int value) const
  { setInt_FieldJc(this, obj->toObjectJc(), value);
  }

  public: float getFloat(ObjectJc* obj) const
  { return getFloat_FieldJc(this, obj);
  }

  public: bool  getBool_FieldJc(const FieldJc* ythis, void* obj)
  { return getBool_FieldJc(this, obj);
  }

  public: void setFloat(ObjectJcpp* obj, float value) const
  { setFloat_FieldJc(this, obj->toObjectJc(), value);
  }

  public: void* getReference(ObjectJcpp* obj) const
  { getReference_FieldJc(this, obj->toObjectJc());
  }

  public: void setReference(ObjectJcpp* obj, void* value) const
  { setReference_FieldJc(this, obj->toObjectJc(), value);
  }

  public: int getModifiers() const { return getModifiers_FieldJc(this); }

};

#endif //defined(__CPLUSPLUSJcpp) && defined(__cplusplus)






/*@CLASS_C MethodJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/




#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)
  typedef void(* Void_Method)(void);   //normaler Funktionszeiger

  typedef void(ObjectJc::*Object__VoidMethod)();  //Klassenfunktionszeiger ???
  typedef int(ObjectJc::*Object__IntMethod)();  //Klassenfunktionszeiger ???
  typedef int(ObjectJc::*Object__IntMethod1Int)(int);  //Klassenfunktionszeiger ???
  typedef int(ObjectJc::*Object__IntMethod2Int)(int, int);  //Klassenfunktionszeiger ???
  typedef int(ObjectJc::*Object__IntMethod1Float)(float);  //Klassenfunktionszeiger ???
  typedef int(ObjectJc::*Object__IntMethod2Float)(float, float);  //Klassenfunktionszeiger ???
  typedef float(ObjectJc::*Object__FloatMethod1Int)(int);  //Klassenfunktionszeiger ???
  typedef float(ObjectJc::*Object__FloatMethod2Int)(int, int);  //Klassenfunktionszeiger ???
  typedef float(ObjectJc::*Object__FloatMethod1Float)(float);  //Klassenfunktionszeiger ???
  typedef float(ObjectJc::*Object__FloatMethod2Float)(float, float);  //Klassenfunktionszeiger ???
#else
  typedef void (*Object__VoidMethod)();
#endif


typedef void (*Method_void_Object)(ObjectJc*);
typedef void (*Method_void_Object1Int)(ObjectJc*, int);
typedef void (*Method_void_Object2Int)(ObjectJc*, int, int);
typedef void (*Method_void_Object3Int)(ObjectJc*, int, int, int);
typedef void (*Method_void_Object1Float)(ObjectJc*, float);
typedef int (*Method_int_Object)(ObjectJc*);


/**Type of the Method */
typedef enum Type_MethodJc_t
{ kNothing_Method, kInt_void, kInt_Int, kInt_2Int, kInt_3Int, kInt_Float, kInt_2Float
, kFloat_void, kFloat_Int, kFloat_2Int, kFloat_3Int, kFloat_Float, kFloat_2Float
, kVoid_void, kVoid_Int, kVoid_2Int, kVoid_3Int, kVoid_Float, kVoid_2Float
}Type_MethodJc;


typedef struct  MethodJc_t
{ //ObjectJc super;
  //StringJcRef  name;
  char name[kLengthNAME_CLASS_ReflectionJc];

  /**Modifier see java class Modifier*/
  int32     bModifiers;
  /**adress of the method*/
  //void* adress;
  Object__VoidMethod adress;
  //Void_Method* adress;
  /** Type of the method */
  Type_MethodJc eType;
}MethodJc;


METHOD_C const StringJc getName_Method(const MethodJc* ythis);



/**
Invokes the underlying method represented by this Method object,
on the specified object with the specified parameters.
Individual parameters are automatically unwrapped to match primitive formal parameters,
and both primitive and reference parameters are subject to method invocation conversions
as necessary. <br/>

If the underlying method is static, then the specified obj argument is ignored. It may be null. <br/>

If the number of formal parameters required by the underlying method is 0,
the supplied args array may be of length 0 or null. <br/>

If the underlying method is an instance method, it is invoked using dynamic method lookup.<br/>

@param obj the object the underlying method is invoked from.
           that the obj matches to the MethodJc ythis. It means, the MethodJc ythis
           is getted from a object of exactly this type or from the obj itself.
           This is tested by original Java - set- or get- operations.
           But it is not tested here to optimize calculation time. If it is a part of a
           Java2C-translation, the test should be done on Java-level, if it is a directly
           programmed in C or C++ part, the user should be programmed carefully, how it is
           ordinary in C or C++.<br/>
           The pointer is accepted as an void*, from there the user does not need to convert
           it to the common ObjectJc.<br/>
@param args the arguments used for the method call. This Arguments are given as int32-types.
           If the method has differing parameters, they are converted inside the invoke.
           The Description in MethodJc knows the type. The user also must know the type.
           The parameter must stored in the binary form used inside, it means, a float value
           must be stored as its integer image, getted by <code>*(int32*)(&floatVaribale)</code>.<br/>
Returns:
the result of dispatching the method represented by this object on obj with parameters args
*/
METHOD_C int32 invoke_v_MethodJc(const MethodJc* ythis, void* obj, Int32ARRAY* params);


//#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)
  int32 invoke_MethodJc(const MethodJc* ythis, ObjectJc* obj, Int32ARRAY* params);
//#endif

#define CONST_ARRAYHEAD_MethodJc(NAME) CONST_ObjectArrayJc(MethodJc, ARRAYLEN(REFLECTION_IMAGE.methods_##NAME.data), 0, null, &REFLECTION_IMAGE.methods_##NAME)

#define CONST_MethodJc(CLASS, SNAME, NAME, TYPE, MODI) { SNAME, 0, null, TYPE }


/** Method[] consists of ObjectArrayJc and some MethodJc elements directly after them.
*/


typedef struct MethodJcARRAY_t
{ /** Base data of every array */
  ObjectArrayJc head;
  /** For debugging, 10 Elements are assumed. The real number of values is stored in array.len*/
  MethodJc fields[10];
}MethodJcARRAY;

METHOD_C const MethodJc* get_MethodJcARRAY(const MethodJcARRAY* ythis, int idx);

/**Converts a Array of type MethodJcARRAY with any desired size to the given MethodJcARRAY-Type.
   The conversion may test the content or not.
*/
METHOD_C MethodJcARRAY const* ptrConstCasting_MethodJcARRAY(ObjectArrayJc const* ptr);



/*@CLASS_C ClassJc ************************************************************************/




typedef struct  ClassJc_t
{ ObjectJc super;

  /** The typename. If it is a Class represents a primitive type, the name is such as "int", "float", "boolean".*/
  char name[kLengthNAME_METHOD_ReflectionJc];

  /** position of the base structure/class ObjectJc inside this type, it is the base address for the offset of fields.*/
  int32 posObjectBase;

  /**sizeof the type*/
  int32 nSize;

  FieldJcArray const* attributes;

  MethodJcARRAY const* methods;

  /** The superclass, ObjectJc if no other superclass.*/
  struct Class_idxVtblJc_a_t const* superClasses;
  //struct ClassJc_t const* superClass;

  /** Array of interfaces to this class.*/
  struct Class_idxVtblJc_a_t const* interfaces;
  //struct{ ObjectArrayJc base; ClassJc* interfaces; };

  /** Some bits determines the kind of the Class, see Modifier_reflectJc*/
  int32 modifiers;

  /**Pointer to jump table for dynamic calls (virtual methods).
   * This is a typed struct, starting with Vtbl_ObjectJc.
   */
  MT_clone_ObjectJc** jtbl;
  //Method_int_Object* jtbl;

}ClassJc;


#define OBJTYPE_ClassJc (kIsSmallSize_objectIdentSize_ObjectJc + 0x0ff80000)
#define OBJTYPE_ReflectionImageJc (mIsLargeSize_objectIdentSize_ObjectJc + 0x1e000000)
#define OBJTYPE_ReflectionImageBaseAddressJc (kIsSmallSize_objectIdentSize_ObjectJc + 0x0ff70000)

/** Constant Definition of a ClassJc-instance usefull for simple plain old data
  * without special defined methods. A superclass ObjectJc is not specified.
  * It doesn#t mean that the data can't base on ObjectJc, but the access is not able
  * via this reflection definition. It may or may not based on any superclass.
  * Use the macro CONST_xxx_ClassJc instead if you have complexer structures.
  *
  * It is been designated for a whole reflection constant definition block
  * with the following rules:
  * *The name of the constant whole reflection instance should be ,,reflectionNAME,,.
  * *The ''virtual table'' is denoted with ,,vtbl,,.
  * *The attribute-block is denoted with ,,fields,,.
  * *A superclass or any interfaces are not present.
  * *Methods are not present.
  *
  */
#define CONST_dataonly_ClassJc(SNAME, NAME) \
{ CONST_ObjectJc(sizeof(reflection##NAME), , &reflection##NAME.clazz , null) \
, SNAME\
, 0     \
, sizeof(NAME) \
, (FieldJcArray const*)(&reflection##NAME.fields) \
, null, null, null, 0 \
}

/** Adaequate complex reflection for C++-classes
*/

#define CONST_Cpp_ClassJc(SNAME, NAME, OWNADDRESS)\
{ CONST_ObjectJc(sizeof(reflection##NAME), OWNADDRESS, null)   \
, SNAME /*CONST_StringJc(SNAME)*/                      \
, 0  \
, sizeof(NAME)                              \
, (FieldJcArray const*)&reflection##NAME.fields \
, (MethodJcARRAY const*)(&reflection##NAME.methods)\
, null  \
, null  \
, mObjectifcBaseJcpp_Modifier_reflectJc   \
}




/**Macro for constant value of the ObjectJc on head of any reflectionImage-structure.
 * The macro REFLECTION_IMAGE should be defined with the identifier of the reflection image.
 * Therefore all information to build the constant value is given, no additionaly arguments are needed.
 */
#define CONST_ObjectJc_REFLECTION_IMAGE() CONST_ObjectJc(OBJTYPE_ReflectionImageJc + sizeof(REFLECTION_IMAGE), &(REFLECTION_IMAGE), null)


/**Macro for constant value of a ClassJc inside a ReflectionImage-structure.
 * The macro REFLECTION_IMAGE should be defined with the identifier of the reflection image.
 */
#define CONST_ClassJc(SNAME, NAME)\
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(REFLECTION_IMAGE.reflect_##NAME), &REFLECTION_IMAGE.reflect_##NAME, null)   \
, SNAME                       \
, 0  /**Position of ObjectJc in the Type should be 0, if only C struct should be used.*/ \
, sizeof(NAME)                              \
, (FieldJcArray const*)&REFLECTION_IMAGE.attributes_##NAME \
, (MethodJcARRAY const*)(&REFLECTION_IMAGE.methods_##NAME)\
, null  \
, null  \
, mObjectJc_Modifier_reflectJc   \
}



/**Macro for constant value of a only-data-ClassJc inside a ReflectionImage-structure.
 * The macro REFLECTION_IMAGE should be defined with the identifier of the reflection image.
 */
#define CONST_dataImg_ClassJc(SNAME, NAME)\
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(REFLECTION_IMAGE.reflect_##NAME), &REFLECTION_IMAGE.reflect_##NAME, null)   \
, SNAME /*CONST_StringJc(SNAME)*/                      \
, 0  /**Position of ObjectJc in the Type should be 0, if only C struct should be used.*/ \
, sizeof(NAME)                              \
, (FieldJcArray const*)&REFLECTION_IMAGE.attributes_##NAME \
, null  \
, null  \
, null  \
, mObjectJc_Modifier_reflectJc   \
}


/**Macro for constant value of a ClassJc inside a ReflectionImage-structure.
 * The macro REFLECTION_IMAGE should be defined with the identifier of the reflection image.
 */
#define CONST_data_ClassJc(SNAME, NAME)\
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(ClassJc), &reflect##NAME, null)   \
, SNAME                       \
, 0  /**Position of ObjectJc in the Type should be 0, if only C struct should be used.*/ \
, sizeof(NAME)                              \
, &attribs##NAME \
, null  \
, null  \
, null  \
, 0   \
}



/** Methods from Class */
METHOD_C FieldJcArray const* getDeclaredFields_ClassJc(ClassJc const* ythis);//sss

METHOD_C FieldJc const* getDeclaredField_ClassJc(ClassJc const* ythis, StringJc* sName);

METHOD_C MethodJcARRAY const* getDeclaredMethods_ClassJc(ClassJc const* ythis);

METHOD_C MethodJc const* getDeclaredMethod_ClassJc(ClassJc const* ythis, char const* sName);


/**gets the Name of the class. It is a static method, because the argument class may be a constant for simple types.
 * @param class Either the class object or a constant to design simple types, see REFLECTION_xxx.
 */
METHOD_C const char* getName_ClassJc(ClassJc const* clazz);

METHOD_C ClassJc const* getSuperclass_ClassJc(ClassJc const* ythis);

/**gets the modifiers for this class or interface, encoded in an integer.
 * They should be decoded using the methods of class ModifierJc.
 */
METHOD_C int32 getModifiers_ClassJc(ClassJc const* ythis);


/** returns true if the type is a primitive Type, like java Class::isPrimitive()
  */
METHOD_C bool isPrimitive_ClassJc(ClassJc const* ythis);

/** returns true if the type is a reference Type. Not javalike.
  */
#define isReference_ClassJc(THIS) ((THIS->modifiers & mReference_Modifier_reflectJc)!=0)

/** returns true if the type is a instance of a C-struct, based on ObjectJc.
  * This method is defined outside java, in java all classes based only on Object.
  */
#define isObjectJc_ClassJc(THIS) ((THIS->modifiers & mObjectJc_Modifier_reflectJc)!=0)

/** returns true if the type is a instance of a C++-class, based on ObjectifcBaseJcpp.
  * It may be also a reference to a interface. The reference type should based on ObjectifcBaseJcpp.
  * This method is defined outside java, in java all classes based only on Object.
  */
#define isObjectifcBaseJcpp_ClassJc(THIS) ((THIS->modifiers & mObjectifcBaseJcpp_Modifier_reflectJc)!=0)







/*@CLASS_C ModifierJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/** Modifiers to specify the kind of field, see class java.lang.reflect.Modifier
*/

typedef enum  Modifier_reflectJc_t
{  /**
   * The <code>int</code> value representing the <code>public</code>
   * modifier.
   */
  mPUBLIC_Modifier_reflectJc           = 0x00000001

  /**
   * The <code>int</code> value representing the <code>private</code>
   * modifier.
   */
, mPRIVATE_Modifier_reflectJc          = 0x00000002

  /**
   * The <code>int</code> value representing the <code>protected</code>
   * modifier.
   */
, mPROTECTED_Modifier_reflectJc        = 0x00000004

  /**
   * The <code>int</code> value representing the <code>static</code>
   * modifier.
   */
, mSTATIC_Modifier_reflectJc           = 0x00000008

  /**
   * The <code>int</code> value representing the <code>final</code>
   * modifier.
   */
, mFINAL_Modifier_reflectJc            = 0x00000010

  /**
   * The <code>int</code> value representing the <code>synchronized</code>
   * modifier.
   */
, mSYNCHRONIZED_Modifier_reflectJc     = 0x00000020

  /**
   * The <code>int</code> value representing the <code>volatile</code>
   * modifier.
   */
, mVOLATILE_Modifier_reflectJc         = 0x00000040

  /**
   * The <code>int</code> value representing the <code>transient</code>
   * modifier.
   */
, mTRANSIENT_Modifier_reflectJc        = 0x00000080

  /**
   * The <code>int</code> value representing the <code>native</code>
   * modifier.
   */
, mNATIVE_Modifier_reflectJc           = 0x00000100

  /**
   * The <code>int</code> value representing the <code>interface</code>
   * modifier.
   */
, mINTERFACE_Modifier_reflectJc        = 0x00000200

  /**
   * The <code>int</code> value representing the <code>abstract</code>
   * modifier.
   */
, mABSTRACT_Modifier_reflectJc         = 0x00000400

  /**
   * The <code>int</code> value representing the <code>strictfp</code>
   * modifier.
   */
, mSTRICT_Modifier_reflectJc           = 0x00000800

    /**outside java definition:
     * If not 0, than it is a primitive type.
     * The value at this bits represents the nr of Bits of the primitive type in the target system.
     */
, mPrimitiv_Modifier_reflectJc         = 0x000f0000

    /**outside java definition:
     * Position of this Bits.
     */
, kBitPrimitiv_Modifier_reflectJc =  16

    /**outside java definition:
     * The <code>int</code> value representing the isUML_List()-Property.
     * It is set if a Reference is a multiple aggregation realized with a list container
     * using the ListJc interface.
     */
, mListJc_Modifier_reflectJc          =  0x00100000

    /**outside java definition:
     * The <code>int</code> value representing the isUML_List()-Property.
     * It is set if a Reference is a multiple aggregation realized with a array container
     * using the ObjectArrayJc structure.
     */
, mObjectArrayJc_Modifier_reflectJc   =  0x00200000

    /**outside java definition:
     * The <code>int</code> value representing the isUML_List()-Property.
     * It is set if a Reference is a multiple aggregation realized with a list container
     * using the MapJc interface.
     */
, mMapJc_Modifier_reflectJc           =  0x00400000

    /**outside java definition:
     * The <code>int</code> value representing the isStaticArray-Property.
     * It is set if multiple elements are after another imediately,
     * The number of elements is hold in the FieldJc.nrofArrayElements
     */
, mStaticArray_Modifier_reflectJc           =  0x00800000

    /** outside java definition:
     * The <code>int</code> value representing the isEmbedded()-Property of fields,
     * It is set if a attribute represented by the field in reflection represents
     * a embedded structured (not a reference to data).
     * It is the same bit like mPRIMITIVE, because it means not concurrency things,
     * there are comparable: in class (primitive type) or in field (embedded data).
     */
, mEmbedded_Modifier_reflectJc         =  0x01000000

    /** outside java definition:
     * The <code>int</code> value representing the isReference()-Property.
     * A reference is a reference to a primitive type or structured type,
     * not inherited from ObjectJc.
     */
, mReference_Modifier_reflectJc        =  0x02000000

    /** outside java definition:
     * The <code>int</code> value representing the isObjectJc()-Property.
     * It is set if the Type contains ObjectJc at its first position,
     * especially for C-struct using CRuntimeJavalike-concepts. If it is set,
     * a C-pointer cast is allowed to keep the ObjectJc-pointer.
     * The bit is not set if the type is derived from ObjectJc,
     * but don't contain it at first position
     * It is not set if
     */
, mObjectJc_Modifier_reflectJc        =  0x04000000

    /**outside java definition:
     * The <code>int</code> value representing the isObjectifc()-Property.
     * It is set if the Type is derived from ObjectifcBaseJcpp.
     */
, mObjectifcBaseJcpp_Modifier_reflectJc  =  0x08000000

    /**outside java definition:
     * The <code>int</code> value representing the isUML_List()-Property.
     * It is set if a Reference is a multiple aggregation realized with a list container.
     * The realisation of the list may be specific for a UML framework, therefore the access to the elements
     * is made with the user definable UML_ListJc methods.
     */
, mUML_LinkedList_Modifier_reflectJc   =  0x10000000

    /**outside java definition:
     * The <code>int</code> value representing the isUML_Array()-Property.
     * It is set if a Reference is a multiple aggregation realized with a list container.
     * The realisation of the list may be user specific for a UML framework, therefore the access to the elements
     * is made with the user definable UML_ArrayJc methods.
     */
, mUML_ArrayList_Modifier_reflectJc    =  0x20000000

    /**outside java definition:
     * The <code>int</code> value representing the isUML_Map()-Property.
     * It is set if a Reference is a multiple aggregation realized with a list container.
     * The realisation of the list may be user specific for a UML framework, therefore the access to the elements
     * is made with the user definable UML_MapJc methods.
     */
, mUML_Map_Modifier_reflectJc          =  0x40000000


} Modifier_reflectJc;


#define isReference_ModifierJc(VALUE) ( (VALUE & mReference_Modifier_reflectJc) != 0)

#define isCollection_ModifierJc(VALUE) ( (VALUE & (mListJc_Modifier_reflectJc | mObjectArrayJc_Modifier_reflectJc  | mMapJc_Modifier_reflectJc | mStaticArray_Modifier_reflectJc | mUML_LinkedList_Modifier_reflectJc | mUML_ArrayList_Modifier_reflectJc| mUML_Map_Modifier_reflectJc)) != 0 )

#define isStaticArray_ModifierJc(VALUE) ( (VALUE &  mStaticArray_Modifier_reflectJc ) != 0 )





/*@DEFINE_C @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/




/** Macro to define a users type reflection with the exactly length of the table of virtual methods
  * used in the header file of a users type.
  * See ReflectionVtblJc. It creates a type named Reflection__TYPE and a external reference
  * to the static reflection variable named reflection__TYPE.
  * @param TYPE The users type.
  * @param nrofVIRTFN really number of the virtual methods, it must match with the
  *        definition of the reflection of this type.
  */
//#define TYPEDEF_REFLECTIONJc(TYPE, nrofVIRTFN) typedef struct Reflection__##TYPE##_t{ ClassJc reflection; Void_MethodVoid vTbl[nrofVIRTFN];} Reflection__##TYPE; extern const Reflection__##TYPE reflection__##TYPE;
#define TYPEDEF_REFLECTIONJc(TYPE) typedef struct Reflection__##TYPE##_t{ ClassJc clazz; Vtbl_##TYPE vtbl;} Reflection__##TYPE; extern const Reflection__##TYPE reflection__##TYPE;

/** Macro to define a users type reflection of a class type,
  * used in the header file of a users type.
  * See ReflectionVtblJc. It creates a type named Reflection__TYPE and a external reference
  * to the static reflection variable named reflection__TYPE.
  * @param TYPE The users type.
  * @param NROFVCALL number of virtual methods of the type, may be 0 if the type is only used in C++.
  */
#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)
  #define TYPEDEF_REFLECTIONJcpp(TYPE) typedef struct Reflection__##TYPE##_t{ ClassJc clazz; } Reflection__##TYPE; extern const Reflection__##TYPE reflection__##TYPE;
#else
  #define TYPEDEF_REFLECTIONJcpp(TYPE) TYPEDEF_REFLECTIONJc(TYPE)
#endif


/** Typedef of a reflection structure inclusive pointer to dynamic linked methods.
  * It is used particularly for C-structure-like classes to realize dynamic linked methods
  * in C. In a C++-environment it is necessary if Plain Old Data (POD) are used
  * in conjunction with the using of reflection and overloading of some ObjectJc
  * methods, at example toString().<br>
  * The number of elements in vMethods are depended on the number of virtual methods.
  * The number here is only a standard value to support a debugging.
  * You see the first ones methods, or, also able fault pointers. See macro TYPEDEF_REFLECTIONJc.<br>
  * In a poor C++ environment the element vMethods is not existent, the debugger shows
  * fault pointers. See macro TYPEDEF_REFLECTIONJcpp.
*/
typedef struct ReflectionVtblJc_t
{
  ClassJc clazz;
  MT_void_Method_void vMethods[10];
}ReflectionVtblJc;


//TYPEDEF_REFLECTIONJc(ObjectJc)





/** This type describes one entry of a interface or superclass.
    In regard of virtual methods an index inside the table of virtual methods is stored here.
*/
typedef struct Class_idxVtblJc_t
{
  /** The reflection definition*/
  ClassJc const* clazz;
  /** Index of the virtual table inside its parent.*/
  int idxVtbl;
}Class_idxVtblJc;



/** This type describes a array of Class_idxVtblJc*/
typedef struct Class_idxVtblJc_a_t
{ /** Base data of every array */
  ObjectArrayJc head;
  /** For debugging, 10 Elements are assumed. The real number of values is stored in array.len*/
  Class_idxVtblJc data[10];
}Class_idxVtblJc_a;






/** Class[] consists of ObjectArrayJc and some ClassJc elements directly after them.
*/
typedef struct ClassARRAYp_t
{ /** Base data of every array */
  ObjectArrayJc head;
  /** For debugging, 10 Elements are assumed. The real number of values is stored in array.len*/
  ClassJc data[10];
}ClassARRAYp;

METHOD_C const ClassJc* get_ClassARRAYpc(const ClassARRAYp* ythis, int idx);






#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)


class  ClassJcpp: public ClassJc
{
  public: FieldJcpp const* getDeclaredField(StringJc* src) const
  {
    return static_cast<FieldJcpp const*>(getDeclaredField_ClassJc(this, src));
  }

};

#endif //defined(__CPLUSPLUSJcpp) && defined(__cplusplus)



/*@INLINE ********************************************************************************************/

#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)

/* Problem with CHeader.sbnf
inline ClassJcpp const* FieldJcpp::getType() const
{ return static_cast<ClassJcpp const*>(getType_FieldJc(this));
}
*/


#endif //defined(__CPLUSPLUSJcpp) && defined(__cplusplus)



/*@DEFINE_C ********************************************************************************************/


/** External definitions of language standard types.
    Note: The types with postfix Jc are the same as in Java (int is 32 bit, long is 64 bit), the transformations from special c/c++-Types are made with define.
*/

#define reflection__uint64Jc reflection__longJc
#define reflection__int64Jc reflection__longJc
#define reflection__uint32Jc reflection__intJc
#define reflection__int32Jc reflection__intJc
#define reflection__uint16Jc reflection__shortJc
#define reflection__int16Jc reflection__shortJc
#define reflection__uint8Jc reflection__byteJc
#define reflection__int8Jc reflection__byteJc
//extern const struct Reflectionint_t{ ClassJc clazz; } reflectionint;

#define reflection__uintJc reflection__intJc
#define reflection__boolJc reflection__booleanJc
//extern ClassJc reflection__booleanJc;

#define reflection__float32Jc reflection__floatJc
//extern const struct Reflectionfloat_t{ ClassJc clazz; } reflectionfloat;

extern const ClassJc reflection__longJc;
extern const ClassJc reflection__intJc;
extern const ClassJc reflection__shortJc;
extern const ClassJc reflection__byteJc;
extern const ClassJc reflection__booleanJc;
extern const ClassJc reflection__floatJc;
extern const ClassJc reflection__doubleJc;
extern const ClassJc reflection__charJc;

/** Reflection_void is a reflection definition for a void pointer.
 */
extern const ClassJc reflection__voidJc;
extern const ClassJc reflection__ObjectifcBaseJcpp;

#define reflection__ObjectArrayJc reflection__ObjectJc
#define reflection__StringJc null
#define reflection__ClassJc reflection__ObjectJc
#define reflection__ObjectJcpp reflection__ObjectJc

/*@CLASS_C Reflection__ObjectJc @@@@@@@@*/

typedef struct Reflection__ObjectJc_t
{ ClassJc clazz; //x
  MT_void_Method_void vtbl[5];
} Reflection__ObjectJc;

extern const Reflection__ObjectJc reflection__ObjectJc;


#if 0
/*
extern const struct ReflectionFieldJc_t
{ ClassJc clazz;
  Vtbl_ObjectJc vtbl;
  DEFINE_ARRAYJc(FieldJc, 2) fields;
}reflectionFieldJc;
*/
#endif



#endif //__ReflectionJc_h__
