/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content Implementation of String processing
 *
 * @author Jchartmut www.vishia.org
 * @version 0.81
 * list of changes:
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/

//#include "CRuntimeJavalike.h"
#include <stdlib.h>
#include "ObjectJc.h"
#ifdef USE_RUNTIMEHEAPJc
  #include "RuntimeHeapJc.h"
#endif

#include "StringBufferJc.h"
#include "ExceptionJc.h"
#include "StringJc.h"
#include <string.h>
#include <stdio.h>  //using sprintf
//#include "RuntimeHeapJc.h"



/**
  Strings are necessary in control applications for
  <ul><li>Assignment of warning and error messages with some data,<li>
      <li>some other small things,</li>
  </ul>
  but not to process large string problems. Thats while the maximum number of chars
  may be limited to less than 32000, typical there are 250 or 1000 chars.<br/>
  Strings may be:
  <ul><li>contant string literals, written in form "fix string",</li>
      <li>parts of a given string (from method String.substring)</li>
      <li>Strings in a buffer, managed by StringbufferJc.</li>
  </ul>
  In all this cases the string is identified with the pointer to the chars and the
  actual length. The C-convention: identification of a string by a zero-terminated
  array of chars, is not the matching solution, because only constant strings
  or the whole string in a buffer may be identified in this manner.<br>
  Because a string may be also a part of a content of StringBufferJc, the followed rule
  is found:
  <ul><li>If the string is a constant string, the pointer to the array of chars is stored
          in <code>value</code> and the number of chars is stored in <code>count</code>.
          The <code>offset</code> is set to <code>-1</code>.</li>
      <li>If the string is a part or the whole content of the buffer in StringBufferJc,
          The <code>value</code> pointered the start of buffer in the StringBufferJc.
          The adress of the StringBufferJc-Structur is ascertainable by subtraction of the
          difference between the Buffer inside a StringBufferJc and the StringBufferJc.
          The <code>offset</code>-Attribute is used to pointer the position inside buffer.
          </li>
  </ul>
  A StringBufferJc must known if a StringJc is referenced to it, by two causes:
  <ul><li>For garbage collection: A string should be handled like a reference.</li>
      <li>If a StringJc references the StringBufferJc and a change of the StringBufferJc
          will proceed, the StringBufferJc should copied in a new memory area and than modified,
          to held the content for pointering strings constant.</li>
  </ul>
  This requested knowledge is realized in StringBufferJc by using the array of back-references,
  see RuntimeHeapJc, and by query a flag in the StringBufferJc-attributes.
*/





#if 0
void setref_String(StringJc* ref, StringJc src)
{ STACKTRC_NAME("setref_String")
  if((ref->refbase & mIsConstant_StringJc) != mIsConstant_StringJc)
  { clearBackRef_ObjectJc(&ref->refbase);
  }
  *ref = src;  //copy the content, no backreference is necessary
}
#endif






StringBufferJc* ctor_s0i_StringBufferJc(StringBufferJc* ythis, char* buffer, int size)
{ if(ythis != null)
  { int count;
    MemC mem = {sizeof(ObjectJc), (MemAreaC*)ythis};
    ctor_ObjectJc(mem);
    if(size < 0)
    { size = -size;
    }
    ythis->size = -size;  //negative to sign it is a refernce.
    count = strlen(buffer);  //from the given content
    if(count > size)
    { count = size; }        //but never greater than given size. The content may be a part of 0-terminate C-String.
    ythis->count= count;
    ythis->value.buffer = buffer;
  }
  return ythis;
}






void clone_String(ObjectJc* ythis)
{
}

void finalize_String(ObjectJc* ythis)
{
}

int32 hashCode_String(ObjectJc* ythis)
{
  return 0;
}



StringJc toString_String(StringJc* ythis)
{ return *ythis;
}




const char* getConstChar_StringJc(const StringJc* ythis)
{ //int count;
  //const char* sRet = getCharsAndCount_StringJc(ythis, &count);
  const char* sRet = ythis->ref;
  return sRet;
}



const char* toChar_StringJc(const StringJc* ref)
{ //int count;
  //const char* sRet = getCharsAndCount_StringJc(ref, &count);
  const char* sRet = ref->ref;
  return sRet;
}


bool equals_StringJc(StringJc const* ythis, StringJc const* cmp)
{ int countThis = ythis->refbase & mLength__StringJc;
  int countCmp = cmp->refbase & mLength__StringJc;
  if(countThis == mLength__StringJc) countThis = strlen(ythis->ref);
  if(countCmp == mLength__StringJc) countCmp = strlen(cmp->ref);
  if(countCmp != countThis) return false;
  else return (memcmp(ythis->ref, cmp->ref, countThis) == 0);
}

#if 0
bool equals_StringJc(StringJc const* ythis, StringJc const* cmp)
{ STACKTRC_ENTRY("equals_StringJc")
  int countThis;
  const char* sThis;
  int countCmp;
  const char* sCmp;
  sThis = getCharsAndCount_StringJc(ythis, &countThis);
  sCmp  = getCharsAndCount_StringJc(cmp  , &countCmp );
  if(countCmp != countThis) STACKTRC_LEAVE; return( false;
  else STACKTRC_LEAVE; return( (memcmp(sThis, sCmp, countThis) == 0);
}
#endif






static StringBufferJc* transsubstantiateToStringBuffer_String(StringJc* ythis, StacktraceThreadContext_s* _threadContext)
{ StringBufferJc* buffer;
  ObjectRefValuesJc refbase = ythis->refbase;
  STACKTRC_TENTRY("transsubstantiateToStringBuffer_StringJc");
  if(refbase & mBuffered__StringJc)
  { //it is directly the begin of the StringBufferJc.
    buffer = (StringBufferJc*)((MemUnit*)(ythis->ref) - offsetChars_StringBufferJc);
  }
  else
  { int length = refbase & mLength__StringJc;
    const char* str = ythis->ref;
    if(length == mLength__StringJc)
    { length = strlen(str);
    }
    { buffer = new_StringBufferJc(-1);  //allocate with standard size
      if(length > buffer->size) THROW(IndexOutOfBoundsException,"string length", length);
      memcpy(buffer->value.direct, str, length);
      buffer->count = length;
      setFromBuffer_StringJc(ythis, buffer);
    }
  }
  STACKTRC_LEAVE; return( buffer);
}





METHOD_C void setLength_StringBufferJc(StringBufferJc* ythis, int newLength)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  STACKTRC_ENTRY("setLength_StringBufferJc");
  if(newLength < 0)
  { THROW(IndexOutOfBoundsException, "", newLength);
  }
  else if(newLength < ythis->count)
  { memset(buffer + newLength, 0, ythis->count - newLength);
    ythis->count = newLength;
  }
  else if(newLength <= size)
  { memset(buffer + ythis->count, 0, newLength - ythis->count);
    ythis->count = newLength;
  }
  else
  { //the newLength increases the String with \0-chars
    THROW(IndexOutOfBoundsException, "unrecognize", newLength);
  }
  STACKTRC_LEAVE;
}





void add_s0_StringJc(StringJc* ythis, const char* addP)
{ const StringJc add = s0_StringJc(addP);
  add_s_StringJc(ythis, add);
}


void add_s_StringJc(StringJc* ythis, StringJc add)
{ StringBufferJc* buffer;
  STACKTRC_ENTRY("add_s_StringJc");

  buffer = transsubstantiateToStringBuffer_String(ythis, _threadContext);
  append_s_StringBufferJc(buffer, add);
  ythis->refbase = ythis->refbase & ~mLength__StringJc | buffer->count;
  STACKTRC_LEAVE;
}


void add_i_StringJc(StringJc* ythis, int add)
{ StringBufferJc* buffer;
  STACKTRC_ENTRY("add_i_StringJc");

  buffer = transsubstantiateToStringBuffer_String(ythis, _threadContext);
  append_i_StringBufferJc(buffer, add);
  ythis->refbase = ythis->refbase & ~mLength__StringJc | buffer->count;
  STACKTRC_LEAVE;
}













StringBufferJc* append_s_StringBufferJc(StringBufferJc* ythis, StringJc add)
{ int lengthAdd = add.refbase & mLength__StringJc;
  int nChars;
  char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  STACKTRC_ENTRY("append_s_StringBufferJc");

  if(lengthAdd == mLength__StringJc){ lengthAdd = strlen(add.ref); }
  nChars = ythis->count + lengthAdd;
  if(nChars > size)
  { //StringJc msg = s0_StringJc("StringBuffer to many chars");
    THROW(RuntimeException, "StringBuffer to many chars", nChars);
  }
  memcpy(buffer + ythis->count, add.ref, lengthAdd);
  ythis->count = nChars;

  STACKTRC_LEAVE;
  return( ythis);
}

StringBufferJc* append_c_StringBufferJc(StringBufferJc* ythis, char add)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  int nChars = ythis->count + 1;
  STACKTRC_ENTRY("append_c_StringBufferJc");
  if(nChars > size)
  { //StringJc msg = const_String("StringBuffer to many chars");
    THROW(RuntimeException, "StringBuffer to many chars", nChars);
  }
  buffer[ythis->count] = add;
  ythis->count = nChars;

  STACKTRC_LEAVE;
  return( ythis);
}



/**Inserts a given char* with known length. This mehtod is static, because the same official using is insert_s_StringBufferJc. */
static StringBufferJc* insert_s0i_StringBufferJc(StringBufferJc* ythis, int offset, const char* add, int nAdd)
{ int nChars;
  char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  STACKTRC_ENTRY("insert_s0i_StringBufferJc");
  nChars = ythis->count + nAdd;
  if(nChars > size)
  { //StringJc msg = const_String("StringBuffer to many chars");
    THROW(RuntimeException, "StringBuffer to many chars", nChars);
  }
  { int nRest = offset < ythis->count;
    if(nRest > 0) memmove(buffer + offset + nAdd, buffer + offset, nRest);
  }
  memcpy(buffer + offset, add, nAdd);
  ythis->count = nChars;

  STACKTRC_LEAVE; return( ythis);
}






/** found an algorithm unusing division, because some divisions may be expensive in time
  * at some processors.
*/

StringBufferJc* insert_ii_StringBufferJc(StringBufferJc* ythis, int offset, int32 value, int radix)
{ char buffer[33];    //max 32 bits and negativ sign for "-10000000000000000000000000000000"
  char cc;
  /**Array of values to test the position in digit. Fill it with 10, 100 etc if radix = 10;*/
  int testValues[32]; //max 32

  int idxTestValues = 0;
  int nChars = 0;
  int test = radix;
  StringBufferJc* ret;
  STACKTRC_ENTRY("insert_ii_StringBufferJc");
  if(value < 0)
  { value = -value; //may be -0x80000000
    buffer[nChars++] = '-';
  }
  //
  //fill in digigts to detect positions in digit
  while(value >= test)
  { testValues[idxTestValues++] = test;
    test *= radix;
  }
  //
  //test input digit
  while(--idxTestValues >=0)
  { cc = '0';
    test =testValues[idxTestValues];
    while(value >= test)
    { cc += 1;
      value -= test;
    }
    if(cc > '9')
    { cc += ('a' - '9' -1);
    }
    buffer[nChars++] = cc;
  }
  cc = (char)('0' + value); //last digit
  if(cc > '9')
  { cc += ('a' - '9' -1);
  }
  buffer[nChars++] = cc;
  ret = insert_s0i_StringBufferJc(ythis, offset, buffer, nChars);
  STACKTRC_LEAVE; return( ret);
}



StringBufferJc* append_i_StringBufferJc(StringBufferJc* ythis, int value)
{ insert_ii_StringBufferJc(ythis, ythis->count, value, 10);
  return ythis;
}

StringBufferJc* insert_f_StringBufferJc(StringBufferJc* ythis, int index, float value)
{ char buffer[20];
  int nChars;
  nChars = sprintf(buffer, "%f", value);
  return insert_s0i_StringBufferJc(ythis, index, buffer, nChars);
}





METHOD_C StringBufferJc* delete_StringBufferJc(StringBufferJc* ythis, int start, int end)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  if(end >= start && end <= size && start >=0)
  { int nrofBytesFromEndOfDeleteAreaToRealyEnd = ythis->count - end;
    int offset = end - start;
    if(offset > 0 && nrofBytesFromEndOfDeleteAreaToRealyEnd >0)
    { memmove(buffer + start, buffer + start + offset, nrofBytesFromEndOfDeleteAreaToRealyEnd);
      memset(buffer + start + nrofBytesFromEndOfDeleteAreaToRealyEnd, 0, offset);  //store 0 at rest to clean it.
    }
  }
  else
  { STACKTRC_ENTRY("insert_s0i_StringBufferJc");
    THROW(StringIndexOutOfBoundsException, "", 0);
    STACKTRC_LEAVE;
  }
  return ythis;
}


METHOD_C void cleanToSize_StringBufferJc(StringBufferJc* ythis)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  if(ythis->count >= 0 && ythis->count <= size)
  { memset(buffer + ythis->count, 0, size - ythis->count);  //store 0 at rest to clean it.
  }
  else
  { STACKTRC_ENTRY("insert_s0i_StringBufferJc");
    THROW(RuntimeException, "corrupt content, count=", ythis->count);
    STACKTRC_LEAVE;
  }
}
