/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content Implementation of String processing
 *
 * @author Jchartmut www.vishia.org
 * @version 0.81
 * list of changes:
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/
#include "StringJc.h"
#include "ExceptionJc.h"

#include <string.h>

#ifdef mBackRef_ObjectJc
  /** The RuntimeHeapJc.h are only needed, if back references are to be stored.*/
//xxxTODO  #include "RuntimeHeapJc.h"
#else
  #define setBackref_BheapJc(REF, SRC_P) /*it is unused*/
#endif


/*@CLASS_C_IMPLEMENT StringJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/



const struct{ ObjectArrayJc head; FieldJc data[2]; } fields_StringBufferJc
=
{ CONST_ObjectArrayJc(FieldJc, 2, 0, null, &fields_StringBufferJc)
, { CONST_FieldJc(StringBufferJc, "count", count, int, 0)
  , CONST_FieldJc(StringBufferJc, "count", count, int, 0)
} };

const ReflectionStringBufferJc reflectionStringBufferJc
=
{ { CONST_ObjectJc(0, &reflectionStringBufferJc, null)
  , "StringBufferJc"
  , 0 //posObjBase
  , sizeof(StringBufferJc)
  , (FieldJcArray const*) &fields_StringBufferJc
  , (MethodJcARRAY const*) null
  , (Class_idxVtblJc_a*) null  //superClasses
  , (Class_idxVtblJc_a*) null  //interfaces
  ,0 //modifiers
  }
, { null
  }
};


/** Some methods from StringJc are defined in ObjectJc.h, because there are simple methods
  * not using RuntimeHeap, Garbage Collection and others. This simple methods are implemented here.
  * That are all methods which do not copy strings, only compare, convert to char* etc.
  */



/**reads the nrofChars and the reference into local variable. Static subroutine for inner organisation:*/
char const* getCharsAndCount_StringJc(const StringJc* ythis, int* count)
{ char const* chars = ythis->ref;
  int nChars = ythis->refbase & mLength__StringJc;
  if(nChars == mLength__StringJc) { nChars = strlen(chars); }

  *count = nChars;
  return( chars);
}


void setFromBuffer_StringJc(StringJc* ythis, struct StringBufferJc_t* buffer)
{ STACKTRC_ENTRY("setFromBuffer_StringJc");
  clear_StringJc(ythis);
  ythis->ref = (buffer->size < 0 ? buffer->value.buffer : buffer->value.direct);
  setBackRefJc(&ythis->refbase, buffer);
  ythis->refbase |= buffer->count & mLength__StringJc;
  ythis->refbase |= mBuffered__StringJc;
  STACKTRC_LEAVE;
}







void set_StringJc(StringJc* ythis, StringJc src)
{ STACKTRC_ENTRY("set_StringJc");
  clear_StringJc(ythis);

  #ifdef mBackRef_ObjectJc
    //if( (src.refbase & mBackRef_ObjectJc) >0)
    { ythis->ref = src.ref;
      setBackRefJc(&ythis->refbase, src.ref);
      ythis->refbase |= src.refbase & ~mBackRef_ObjectJc;  //copy nrof Chars and mBuffered__StringJc
    }
    //else{ *ythis = src; } //copy the content
  #else
    *ythis = src; //copy the content
  #endif
  STACKTRC_LEAVE;
}

#if 0
void set_StringJc(StringJc* ythis, StringJcRef const* src)
{ STACKTRC_ENTRY("set_String")
  ObjectRefValuesJc refbase = src->refbase;

  if( (refbase & mIsConstant_StringJc) == mIsConstant_StringJc)
  { clear_StringJc(ythis);
    *ythis = *src;  //copy the content of constant reference.
  }
  else if(src->ref.substring == null)
  { //src is a null reference.
    clear_StringJc(ythis);
  }
  else if(src->ref.substring->negOffset < 0)
  { //src is a Substring
  }
  else
  { //src is a StringBuffer
    setFromBuffer_StringJc(ythis, src->ref.buffer);
  }
}
#endif





void clear_StringJc(StringJc* ythis)
{ STACKTRC_ENTRY("clear_StringJc");

  #ifdef mBackRef_ObjectJc
    if( (ythis->refbase & mBackRef_ObjectJc) >0)
    { clearBackRefJc(&ythis->refbase);
    }
    else{ ythis->refbase = 0; ythis->ref = null; } //copy the content
  #else
    ythis->refbase = 0;
    ythis->ref = null;
  #endif
  STACKTRC_LEAVE;
}

#if 0
void clear_StringJc(StringJc* ythis)
{ STACKTRC_NAME("setref_String")
  ObjectRefValuesJc refbase = ythis->refbase;
  if(  (refbase & mIsConstant_StringJc) != mIsConstant_StringJc
    && ythis->ref.buffer != null
    )
  {
    clearBackRef_ObjectJc(&ythis->refbase);
  }
  ythis->refbase = 0;
  ythis->ref.ref = null;
}
#endif




/** Gets the nr of chars and the pointer to the chars from a StringJcRef.
 * A StringJcRef may be a reference directly to a const char*, in this case
 * the offsetBackPointer contains a negative value, it is the count in negative image.<br/>
 * A StringJcRef may also be a reference directly to a StringBufferJc. This is the typical case
 * by concated strings. In this case
 */

#if 0
const char* getCharsAndCount_StringJc(const StringJcRef* ythis, int* count)
{ STACKTRC_ENTRY("toChar_String")
  ObjectRefValuesJc refbase = ythis->refbase;
  const char* chars;
  if((refbase & mZeroTerminated_StringJc) == mZeroTerminated_StringJc)
  { chars = ythis->ref.chars;
    *count = strlen(chars);
  }
  else if( (refbase & mIsConstant_StringJc) == mIsConstant_StringJc)
  { //it is a constant string
    *count = refbase & mLength__StringJc;
    chars = ythis->ref.chars;
  }
  else
  { //it is either a directly referenced StringBufferJc or a SubstringJc
    int offset;
    SubstringJc* substring = ythis->ref.substring;
    *count = substring->count;
    offset = - substring->negOffset;
    if(offset >= 0)
    { chars = substring->stringBuffer->(buffer->size < 0 ? buffer->value.buffer : buffer->value.direct) + offset;
    }
    else
    { //it is a referenced StringBufferJc itself:
      StringBufferJc* buffer = (StringBufferJc*)(substring);
      //TRICKY the lengthAdd is at same position.
      chars = buffer->(buffer->size < 0 ? buffer->value.buffer : buffer->value.direct);
    }
  }
  STACKTRC_LEAVE; return( chars);
}
#endif



METHOD_C int copyToBuffer_StringJc(const StringJc* ythis, char* buffer, int maxSizeBuffer)
{ int nChars = ythis->refbase & mLength__StringJc;
  STACKTRC_ENTRY("copyToBuffer_StringJc");
  if(nChars == mLength__StringJc) { nChars = strlen(ythis->ref); }
  if(nChars >= maxSizeBuffer){ nChars = maxSizeBuffer -1; }
  strncpy(buffer, ythis->ref, nChars);
  buffer[nChars] = 0;
  STACKTRC_LEAVE; return( nChars);
}




bool equals_s0_StringJc(const StringJc* ythis, const char* sCmp)
{ int countThis = ythis->refbase & mLength__StringJc;
  if(countThis == mLength__StringJc){ countThis = strlen(ythis->ref); }
  if((int)(strlen(sCmp)) != countThis)
  { return(false);
  }
  else
  { bool bRet = memcmp(ythis->ref, sCmp, countThis) == 0;
    return(bRet);
  }
}


bool startsWith_s0_StringJc(const StringJc* ythis, const char* sCmp)
{ int countThis = ythis->refbase & mLength__StringJc;
  int len = strlen(sCmp);
  if(countThis == mLength__StringJc){ countThis = strlen(ythis->ref); }
  if(len > countThis)
  { return(false);
  }
  else
  { bool bRet = memcmp(ythis->ref, sCmp, len) == 0;
    return(bRet);
  }
}



/** Realisation of substring: If the parent string is a constant string,
  * the substring will be also a simple constant string directly pointered a part of parent.
  * If the String is inside a StringBuffer, may be a SubstringJc of StringBufferJc or StringBufferJc itself,
  * the problem is more complexly. In this case inside the parent string it should be registered
  * the using of the Buffer. It may be a RuntimeHeapBlockJc with garbage collection.
  */
METHOD_C StringJc substring_StringJc(StringJc* ythis, int beginIndex, int endIndex)
{ StringJc ret = {0};
  const char* chars = ythis->ref;
  int count = ythis->refbase & mLength__StringJc;
  STACKTRC_ENTRY("substring_StringJc");

  if(count == mLength__StringJc) { count = strlen(chars);}

  if(endIndex < 0) {endIndex = count; }

  if(endIndex > count){ THROW(IndexOutOfBoundsException, "endIndex", endIndex); }
  else if(beginIndex <0 || beginIndex > endIndex){ THROW(IndexOutOfBoundsException, "beginIndex", beginIndex); }
  else
  {
    ret.ref = chars + beginIndex;
    ret.refbase = (endIndex - beginIndex);
  }

  STACKTRC_LEAVE; return( ret);
}


#if 0
METHOD_C StringJc substring_StringJc(StringJc* ythis, int beginIndex, int endIndex)
{ STACKTRC_ENTRY("substring_StringJc")
  StringJc ret;
  int count;
  if( (ythis->refbase & mIsConstant_StringJc) == mIsConstant_StringJc)
  { //it is a constant string
    const char* chars = ythis->ref.ref;
    if((ythis->refbase & mZeroTerminated_StringJc) == mZeroTerminated_StringJc)
    {     count = strlen(chars);}
    else{ count = ythis->refbase & mLength__StringJc; }

    if(endIndex < 0) {endIndex = count; }

    if(endIndex > count){ THROWJc(IndexOutOfBoundsExceptionJc, "endIndex", endIndex); }
    else if(beginIndex <0 || beginIndex > endIndex){ THROWJc(IndexOutOfBoundsExceptionJc, "beginIndex", beginIndex); }
    else
    { ret.ref.chars = chars + beginIndex;
      ret.refbase = (endIndex - beginIndex) | mIsConstant_StringJc;
    }
  }
  else
  { THROWJc(RuntimeExceptionJc, "substring of not constant Strings not ready yet.",0);
  }
  STACKTRC_LEAVE; return( ret;
}
#endif



METHOD_C int indexOf_i_StringJc(StringJc* ythis, int ch, int fromIndex)
{ const char* chars = ythis->ref;
  int nChars = ythis->refbase & mLength__StringJc;
  STACKTRC_ENTRY("StringJc.indexOf(int,int)");
  if(nChars == mLength__StringJc) { nChars = strlen(ythis->ref); }
  //const char* chars = getCharsAndCount_StringJc(ythis, &nChars);
  if(fromIndex <0)
  { STACKTRC_LEAVE; return( -1);
  }
  else
  { int ii;
    for(ii = fromIndex; ii < nChars; ii++)
    { if(chars[ii] == ch) { STACKTRC_LEAVE; return( ii); }
    }
    STACKTRC_LEAVE; return( -1);  //not found STACKTRC_LEAVE; return(s in for loop if found!
  }
}



METHOD_C int indexOf_s_i_StringJc(StringJc* ythis, StringJc str, int fromIndex)
{ int nChars;
  STACKTRC_ENTRY("StringJc.indexOf(int,int)");
  { const char* chars = getCharsAndCount_StringJc(ythis, &nChars);
    int nCharsSearch;
    const char* charsSearch = getCharsAndCount_StringJc(&str, &nCharsSearch);
    if(fromIndex <0)
    { STACKTRC_LEAVE; return( -1);
    }
    else
    { int ii;
      char cc = charsSearch[0];
      for(ii = fromIndex; ii < (nChars - nCharsSearch); ii++)
      { if(chars[ii] == cc)
        { //first char found, test the rest chars
          if(strncmp(charsSearch, chars+ii, nCharsSearch) == 0)
          { STACKTRC_LEAVE; return( ii);
          }
        }
      }
    }
    STACKTRC_LEAVE; return( -1);  //not found, note: return ii in for-loop if found.
  }
}


METHOD_C int indexOf_s0_i_StringJc(StringJc* ythis, const char* src, int fromIndex)
{ StringJc str1;
  init_s0_StringJc(&str1, src);
  return indexOf_s_i_StringJc(ythis, str1, fromIndex);

}





METHOD_C int length_StringJc(StringJc* ythis)
{ int nChars = ythis->refbase & mLength__StringJc;
  if(nChars == mLength__StringJc) { nChars = strlen(ythis->ref); }
  return nChars;
}



METHOD_C StringJc toString_StringBufferJc(StringBufferJc const* src)
{
  StringJc ret = NULL_StringJc();
  ret.refbase = src->count;        //it may be 0
  ret.ref = (src->size < 0 ? src->value.buffer : src->value.direct);
  return ret;
}


void ctor_StringBufferJc(StringBufferJc* ythis, int size)
{ MemC mem;
  mem.address = (MemAreaC*)ythis;
  setSizeAndOffset_MemC(mem, size, 0);
  ctor_ObjectJc(mem);
  ythis->count = 0;
  if(size < 0)
  { size = -size;
  }
  ythis->size = size;
  memset(ythis->value.direct, 0, ythis->size);
}



StringBufferJc* new_StringBufferJc(int size)
{ StringBufferJc* buffer;
  STACKTRC_ENTRY("new_StringBufferJc");
  if(size <=0)
  { //no size is given
    #ifdef USE_BLOCKHEAPJc
      if(threadContext->blockheapStd!=null)
      { size = threadContext->blockheapStd->bytesNormalBlock - 10 * 8; //sizeof(ObjectJcRef);
      }
      else
      { //initial mode:
        size = 240;
      }
    #else
      size =240;
    #endif
  }
  #if USE_BLOCKHEAPJc
    //If Blockheap is available, get the buffer from it.
  #elif USE_MALLOC_FREE
    //If no Blockheap is available, and the system is arranged to use malloc and delete,
    //the user is responisble for free the buffer!
    buffer = (StringBufferJc*)((alloc_MemC(size)).address);
    ctor_StringBufferJc(buffer, size);
  #else
    //From where to get the buffer? If a Garbage Collector concept is established, it is the best way.
    //Therefore the buffer is created in BlockHeap and deleted by Garbage Collector if no references to it are exist.
    //But without a Garbage Collection? If the user should look after it, it is unappreciative
    //because it is a negligibly subject. Therefore it may be soluted errorneous.
    //A better way is to release the user from this decision.
    //A StringBufferJc are available in the threadContext as thread local resource. It can be used temporary
    //for this aim. But the user must not store this reference in a non predictable way.It should be used only locally.
    //if there are conflicts, the user gets to know from this because the content of buffer is changed.
    //Normally the content is assembled in a buffer and than transported to any other location, especially to console via printf.
    buffer = _threadContext->excMsg;
    setLength_StringBufferJc(buffer, 0);
  #endif
  STACKTRC_LEAVE; return( buffer);

}




/**************************************************************************************************/
/* some conversions numeric from/to String */

static int stringToInt__(const char* src, int size)
{ int val = 0;
  bool bNegativ;
  STACKTRC_ENTRY("stringToInt__");
  if(*src == '-'){ src+=1; size -=1; bNegativ = true; }
  else bNegativ = false;
  while(size > 0 && *src >='0' && *src <='9')
  { val = 10* val + (*src - '0');
    src+=1;
    size -=1;
  }
  if(bNegativ){ val = -val; }
  STACKTRC_LEAVE; return( val);
}



int parseInt_IntegerJc(StringJc src)
{ return stringToInt__(src.ref, src.refbase & mLength__StringJc);
}





