/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content Implementation of String processing
 *
 * @author Jchartmut www.vishia.org
 * @version 0.81
 * list of changes:
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/
#ifndef __StringJc_h__
#define __StringJc_h__

#include "ObjectJc.h"
#include "ReflectionJc.h"
#include <string.h>

/*@CLASS_C StringJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

  /** The definition of the StringJc-structure is done in ObjectJc.h,
    * because it is necessary as output of toString_ObjectJc.
    */
  /**Constant StringJc references initialize data to a string with known length.
   */
  #define CONST_s0_i_StringJc(TEXT, LEN) { LEN , TEXT}

  /**Constant StringJc references initialize data to a string with known length.
   */
  #define CONST_i_StringJc(TEXT, LEN) { LEN , TEXT}

  /** constant StringJc references a zero-terminated string. The length of the string
      is not stored inside StringJc, the length bits are setted to there maximum
      to detect this constellation.
    */
  #define CONST_s0_StringJc(TEXT) { mLength__StringJc, TEXT}

  /**Constant StringJc references initialize data to a zero-terminated string.
   * The length of the string
   * is not stored inside StringJc, the length bits are setted to there maximum
   * to detect this constellation.
   */
  #define CONST_StringJc(TEXT) { mLength__StringJc, TEXT}

  /**A pointer to StringJc structure containing null-values.
   * NOTE: a pointer is necessary because a const problem occurs otherwise.
   */
  extern StringJc* const null_StringJc;

  /** Initialize a StringJc instance with the reference to the given constant zeroterminated string.
    * The up till now value inside YTHIS is not respected. This method is usefull to initialize new instances
    * espacially inside the stack.
    */
  #define init_s0_StringJc(YTHIS, src) init_s0i_StringJc(YTHIS, src, strlen(src))

  /** Initialize a StringJc instance with the reference to the given constant zeroterminated string.
    * The up till now value inside YTHIS is not respected. This method is usefull to initialize new instances
    * espacially inside the stack.
    */
  #define ctor_s0i_StringJc(YTHIS, src, nrofChars) { (YTHIS)->ref = src; (YTHIS)->refbase = nrofChars; }
  #define init_s0i_StringJc(YTHIS, src, nrofChars) { (YTHIS)->ref = src; (YTHIS)->refbase = nrofChars; }
  //#define init_s0i_StringJc(YTHIS, src, nrofChars) { (YTHIS)->ref.chars = src; (YTHIS)->refbase = mIsConstant_StringJc | nrofChars; }


  /** Initialize a StringJc instance with an other given StringJc.
    * The up till now value inside YTHIS is not respected. This method is usefull to initialize new instances
    * espacially inside the stack.
    */
  #define init_s_StringJc(YTHIS, src) { YTHIS->refbase = src.refbase;  YTHIS->ref = src.ref;  }


  /** Assigns a StringBufferJc to the String, usefull to work without heap
   */

  METHOD_C void assignBuffer_StringJc(StringJc* ythis, struct StringBufferJc_t* buffer);







  /* Gets the nr of chars and the pointer to the chars from a StringJc.
   * A StringJc may be a reference directly to a const char*, in this case
   * the offsetBackPointer contains a negative value, it is the count in negative image.<br/>
   * A StringJc may also be a reference directly to a StringBufferJc. This is the typical case
   * by concated strings. In this case
   */
  METHOD_C const char* getCharsAndCount_StringJc(const StringJc* ythis, int* count);

  /** Copy the content to a buffer of the user.*/
  METHOD_C int copyToBuffer_StringJc(const StringJc* ythis, char* buffer, int maxSizeBuffer);

  /** Compares this string to the specified object. The result is true if
    * and only if the argument is not null and is a String object that represents
    * the same sequence of characters as this object.
    * Modified from Java, see String.equals(Object)
    */
  METHOD_C bool equals_s0_StringJc(const StringJc* ythis, const char* sCmp);

  /** Compares this string to the specified object. The result is true if
    * and only if the argument is not null and is a String object that represents
    * the same sequence of characters as this object.
    * Modified from Java, see String.equals(Object)
    */
  METHOD_C bool equals_StringJc(const StringJc* ythis, const StringJc* sCmp);


  /** Tests if this string starts with the specified prefix.
    * Modified from Java, see String.startsWith(String)
    */
  METHOD_C bool startsWith_s0_StringJc(const StringJc* ythis, const char* sCmp);

  /** Tests if this string starts with the specified prefix.
    * Original Java, see String.startsWith(String)
    */
  METHOD_C bool startsWith_StringJc(const StringJc* ythis, const StringJc* sCmp);


  /** Clears the reference to a string. If the string is a StringBuffer or Substring,
    * and RuntimeHeap with garbage collection is used, the back reference to ythis
    * will be released.
    */
  METHOD_C void clear_StringJc(StringJc* ythis);





  /** Set this as reference to a Stringbuffer.
    * If Runtimeheap with garbage collection is used, the back referenced to ythis will be
    * registered in buffer. If ythis was a reference before, the reference will be
    * released. If the reference is already a reference to the same buffer,
    * nothing will be done.
    * @param ythis the String to be set.
    * @param buffer the buffer to be referenced.
    */
  METHOD_C void setFromBuffer_StringJc(StringJc* ythis, struct StringBufferJc_t* buffer);


  /** Returns the length
   */
  METHOD_C int length_StringJc(StringJc* ythis);


  /** Returns a new string that is a substring of this string. The substring begins at the specified beginIndex and extends to the character at index endIndex - 1.
    * Thus the length of the substring is endIndex-beginIndex.
    * Same as in Java, see String.substring(int, int)
    * @param startIndex the beginning index, inclusive.
    * @param endIndex the ending index, exclusive. If -1, than the endIndex is the length of string, like call in Java without endIndex.
    * @throws IndexOutOfBoundsException - if the beginIndex is negative,
    * or endIndex is larger than the length of this String object, or beginIndex is larger than endIndex.
    */
  METHOD_C StringJc substring_StringJc(StringJc* ythis, int beginIndex, int endIndex);

  /** Returns the index within this string of the first occurrence
      of the specified character. If a character with value ch occurs in
      the character sequence represented by this String object,
      then the index of the first such occurrence is returned --
      that is, the smallest value k such that:

      , this.charAt(k) == ch

      is true. If no such character occurs in this string, then -1 is returned.

      Same as in Java.

      @param ch - a character.
      @return the index of the first occurrence of the character in the character sequence represented by this object, or -1 if the character does not occur.
    */
  #define indexOf_StringJc(YTHIS, CH)  indexOf_i_StringJc(YTHIS, CH, 0)

  /** Returns the index within this string of the first occurrence
      of the specified character, starting the search at the specified index.

      If a character with value ch occurs in the character sequence
      represented by this String object at an index no smaller than fromIndex,
      then the index of the first such occurrence is returned
      --that is, the smallest value k such that:

      , (this.charAt(k) == ch) && (k >= fromIndex)

      is true. If no such character occurs in this string at or after position fromIndex, then -1 is returned.

      There is no restriction on the value of fromIndex.
      If it is negative, it has the same effect as if it were zero:
      this entire string may be searched. If it is greater than the length of this string,
      it has the same effect as if it were equal to the length of this string: -1 is returned.
      Same as in Java.

      @param ch - a character.
      @param fromIndex the index to start the search from.
      @return the index of the first occurrence of the character in the character sequence represented by this object, or -1 if the character does not occur.
    */
  METHOD_C int indexOf_i_StringJc(StringJc* ythis, int ch, int fromIndex);



  METHOD_C int indexOf_s_i_StringJc(StringJc* ythis, StringJc str, int fromIndex);


  METHOD_C int indexOf_s0_i_StringJc(StringJc* ythis, const char* str , int fromIndex);


  #define indexOf_s0_StringJc(YTHIS, CH)  indexOf_s0_i_StringJc(YTHIS, CH, 0)


  #define indexOf_s_StringJc(YTHIS, CH)  indexOf_s_i_StringJc(YTHIS, CH, 0)







METHOD_C void add_s0_StringJc(StringJc* ythis, const char*);

METHOD_C void add_s_StringJc(StringJc* ythis, StringJc);

METHOD_C void add_i_StringJc(StringJc* ythis, int);

METHOD_C const char* toChar_StringJc(const StringJc* ythis);

/**sets the reference, java: String str = src;
*/
METHOD_C void setref_String(StringJc* ref, StringJc src);

/**sets the reference, java: String str = src;
*/
METHOD_C void setref_b_String(StringJc* ref, struct StringBufferJc_t* src);


//void String_Integer(int value, int radix, StringBufferJc* dst);

/*@CLASS_CPP StringJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)

/**The C++-Variante. It is comfortable to use C++ because some methods and operators are argument-sensitive (overloaded),
 * and the code is better readable.
 * This class defines only methods, and the methods are not virutal. It followes the java convention: The class java.lang.String is final.
 * Thats why a applicable side action exists: The memory layout of StringJcpp is identical with the memory layout of StringJc.
 * So conversions may be able without any problems. Using static_cast<StringJc*>
 */
class StringJcpp: public StringJc
{
  public: StringJcpp(){ refbase = 0; ref = null; }

  public: StringJcpp(const char* src)
  { init_s0i_StringJc(this, src, strlen(src));
  }

  public: StringJcpp(const char* src, int size)
  { ctor_s0i_StringJc(this, src, size);
  }

  public: StringJcpp(const StringJc src)
  {
    init_s_StringJc(this, src);
  }

  public: ~StringJcpp(){ } //  CLEARREFJc(*static_cast<StringJc*>(this)); }


  public: StringJcpp& operator=(StringJcpp src)
  {
    set_StringJc(this, src); return *this;
  }

  public: StringJcpp& operator=(StringJc src)
  {
    set_StringJc(this, src); return *this;
  }

  public: StringJcpp& operator=(const char* src)
  {
    clear_StringJc(this); init_s0_StringJc(this, src); return *this;
  }

  public: StringJcpp& operator=(int src)  //src may be only 0
  {
    clear_StringJc(this); return *this;
  }

  //public: StringJcpp operator+=(StringJcpp& add){ add_StringJc(this, &add, null); return *this; }

  public: StringJcpp& operator+(const char* add)
  { add_s0_StringJc(this, add); return *this;
  }

  public: StringJcpp& operator+(const int add)
  { add_i_StringJc(this, add); return *this;
  }

  public: StringJcpp& operator+=(const char* add)
  { add_s0_StringJc(this, add); return *this;
  }

  public: StringJcpp& operator+=(const StringJcpp& add)
  { add_s_StringJc(this, add); return *this;
  }

  public: StringJcpp& operator+=(int add)
  { add_i_StringJc(this, add); return *this;
  }


  public: bool equals(const char* sCmp)
  { return equals_s0_StringJc(this, sCmp);
  }

  public: bool equals(const StringJc* sCmp)
  { return equals_StringJc(this, sCmp);
  }

  public: int length()
  { return length_StringJc(this);
  }

  public: bool startsWith(const char* sCmp)
  { return startsWith_s0_StringJc(this, sCmp);
  }

  public: bool startsWith(const StringJc* sCmp)
  { return startsWith_StringJc(this, sCmp);
  }

  public: int indexOf(char cc)
  { return indexOf_StringJc(this, cc);
  }

  public: int indexOf(char cc, int fromIndex)
  { return indexOf_i_StringJc(this, cc, fromIndex);
  }

  public: int indexOf(const char* str)
  { return indexOf_s_i_StringJc(this, s0_StringJc(str),0);
  }


  public: StringJc substring(int beginIndex, int endIndex)
  { StringJc ret = substring_StringJc(this, beginIndex, endIndex);
    return ret;
  }

  public: StringJc substring(int beginIndex)
  { StringJc ret = substring_StringJc(this, beginIndex, length_StringJc(this));
    return ret;
  }
};

#endif //__CPLUSPLUSJcpp




/*@CLASS_C StringBufferJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


//#define staticSize_StringBufferJc 80

/** A StringBuffer contains the String directly after a head structure.
 */
typedef struct StringBufferJc_t
{
  /** It is an Object.*/
  ObjectJc object;

  /** actual nr of characters*/
  int16 count;

  /** size of the Buffer. If this number is negative, a buffer is referenced. */
  int16 size;

  union value_t
  {
    /** The buffer may be disposed directly in this structure. The length here given
    is a help value for debugging, the really length is maxNumberOfChars.
    */
    char direct[4];
    #define offsetChars_StringBufferJc (sizeof(ObjectJc) + 2* sizeof(int16))  //sizeof(count) + sizeof(maxNumberOfChars)

    /**Pointer to the referenced buffer.
     * If this is choosen, the maxNumberOfChars is negative.
    */
    char* buffer;
  }value;
}StringBufferJc;


typedef struct xxStringBufferJc_t
{
  /** It is an Object.*/
  ObjectJc object;
  //StringBufferJcD data;
}xxStringBufferJc;

REFTYPEDEFJc(StringBufferJc);

#define staticSize_StringBufferJc (sizeof(((StringBufferJc*)(0x1000))->value.direct))


/**Initializes the Stringbuffer to reference the given buffer with content with the given size.
 * This method should be used if an external char[] is to be used.
 * The nrof chars in the StringBuffer is determined by the content of the buffer
 * with the 0-terminated C-like method.
 * The char[] can also be a part of a char[].
 * @param buffer A char[...]-buffer with content. write 0 at start if it should be empty.
 * @param size the nrof bytes for buffer.
 */
METHOD_C StringBufferJc* ctor_s0i_StringBufferJc(StringBufferJc* ythis, char* buffer, int size);

/**inner Routine*/
//METHOD_C void ctor_s0i_StringBufferJcD(StringBufferJcD* ythis, char* buffer, int size);

/**Sets the length of the character sequence. It's like Java.
 * The sequence is changed to a new character sequence whose length is specified by the argument.
 * For every nonnegative index k less than newLength, the character at index k in the new character sequence
 * is the same as the character at index k in the old sequence if k is less than the length of the old character sequence;
 * otherwise, it is the null character '\u0000'.
 * In other words, if the newLength argument is less than the current length, the length is changed to the specified length.
 *
 * If the newLength argument is greater than or equal to the current length, sufficient null characters ('\u0000')
 * are appended so that length becomes the newLength argument.
 *
 * The newLength argument must be greater than or equal to 0.
 *
 * @param newLength The new length
 * @throws IndexOutOfBoundsException - if the newLength argument is negative.
 * @see length()
 */
METHOD_C void setLength_StringBufferJc(StringBufferJc* ythis, int newLenhth);

/** java:StringBuffer() */
METHOD_C StringBufferJc* new_StringBufferJc(int size);

/**This macro defines a C-constant (with {..}) for initializing a StringBuffer instance with its static size.
 * @param OBJP Pointer to the instance itself. It is used to store the OWNADDRESS for the ObjectJc-part.
 */
#define CONST_StringBufferJc(OBJP) { CONST_ObjectJc(sizeof(StringBufferJc), &(OBJP)->object, null), 0, staticSize_StringBufferJc, {0,0} }


/**This macro definesx a C-constant (with {..}) for initializing a StringBuffer instance with an additional size.
 * This macro is usefull if a StringBuffer with a followed char array is defined to increase the size of string in buffer.
 * The definition of such an construct is possible inside a struct definition with followed sample:
 *
 * ,,struct { ... ; StringBufferJc buffer; char bufferIncreasingCharArray[120]; ... } myData =
 *
 * The ,,bufferIncreasingCharArray,, have to be assembled immediately after the ,,buffer,,. The initializer of this part may be written in form of:
 *
 * ,,{ ... , CONST_addSize_StringBufferJc(&myData.buffer, 120), { '\0'}, ... },,
 *
 * The using of a following char array to increase the buffer is opportune to get a larger StringBuffer, because the compiler always assigns the memory
 * of the followed char array immediately after the previous StringBufferJc, and the own char array of the StringBufferJc is assigned as the last
 * element there. The problem of additional bytes because 4-Byte-allignment or such is not present, because a StringBufferJc have a size of multiple 4 bytes.
 * But the user should define the size of the char array as multiple of 4 to prevent alignment problems also.
 *
 * Note: Outside a struct the definition a definition of elements one after another don't may assigned one after another in memory.
 * It differents from compiler versions or other conditions such as segmentation sizes of physical memory!
 *
 * @param OBJP Pointer to the instance itself. It is used to store the OWNADDRESS for the ObjectJc-part.
 * @param ADDSIZE The additional size of the StringBuffer, defined by siezof the followed char array.
 */
#define CONST_addSize_StringBufferJc(OBJP, ADDSIZE) { CONST_ObjectJc(sizeof(StringBufferJc) + (ADDSIZE), &(OBJP)->object, null), 0, staticSize_StringBufferJc + ADDSIZE, {0,0} }
//#define DEF_StringBufferJc(NAME, SIZE) struct{ StringBufferJc buffer; char add[SIZE - staticSize_StringBufferJc]; } NAME = { { CONST_ObjectJc(0, &sBuffer.buffer.object, null), 0, SIZE, {0} } , {0}}

/**Initializes the Stringbuffer*/
METHOD_C void ctor_StringBufferJc(StringBufferJc* ythis, int size);

/** java:StringBuffer.append(String) */
METHOD_C StringBufferJc* append_s_StringBufferJc(StringBufferJc* ythis, StringJc add);

/** java:StringBuffer.append(String) */
#define append_s0_StringBufferJc(YTHIS, ADD) append_s_StringBufferJc(YTHIS, s0_StringJc(ADD));

/** java:StringBuffer.append(char) */
METHOD_C StringBufferJc* append_c_StringBufferJc(StringBufferJc* ythis, char add);

/** java:StringBuffer.append(int) */
METHOD_C StringBufferJc* append_i_StringBufferJc(StringBufferJc* ythis, int add);

/** java:StringBuffer.append(float) */
#define append_f_StringBuffer(YTHIS, value) insert_f_StringBufferJc(YTHIS,0, value)

/**Java: insert(int offset, int i)
 * Inserts the string representation of the second int argument into this sequence.
 * The second argument is converted to a string as if by the method String.valueOf,
 * and the characters of that string are then inserted into this sequence at the indicated offset.
 *
 * The offset argument must be greater than or equal to 0, and less than or equal to the length of this sequence.
 * @param offset - the offset.
 * @param i - an int.
 * @return a reference to this object.
 * throws StringIndexOutOfBoundsException - if the offset is invalid.
 * @see String.valueOf(int), insert(int, java.lang.String), length()
 */
METHOD_C StringBufferJc* insert_ii_StringBufferJc(StringBufferJc* ythis, int offset, int32 value, int radix);

/** java:StringBuffer.insert(float) */
METHOD_C StringBufferJc* insert_f_StringBufferJc(StringBufferJc* ythis, int index, float value);




/**Removes the characters in a substring of this sequence. It is original from Java.
 * The substring begins at the specified start and extends to the character
 * at index end - 1 or to the end of the sequence if no such character exists.
 * If start is equal to end, no changes are made.
 *
 * @params start The beginning index, inclusive.
 * @params end The ending index, exclusive.
 * @return This object.
 * @throws StringIndexOutOfBoundsException if start is negative, greater than length(),
 *           or greater than end.
 *
 */
//METHOD_C StringBufferJc* delete_StringBufferJcD(StringBufferJcD* ythis, int start, int end);
METHOD_C StringBufferJc* delete_StringBufferJc(StringBufferJc* ythis, int start, int end);


/** Copies the string-reference into a reference of a StringBuffer.
  * Either, the given reference references a string in a StringBuffer, only the content
  * of the reference will be copied. Or the string is a direct String or substring.
  * Than a new StringBuffer-instance will be created.
*/
METHOD_C StringBufferJc* toStringBuffer_String(StringJc const* src);

METHOD_C StringJc toString_StringBufferJc(StringBufferJc const* src);

/**cleans the Rest of content in the buffer after the valid chars.
 * This method is enhanced in comparision to Java. It consideres a usage of StringBufferJc for string fields in structures.
 * The content in the structur's fields should be cleanly. The naming of the method is contrastable to the Java method trimToSize().
 */
METHOD_C void cleanToSize_StringBufferJc(StringBufferJc* ythis);

/**Definition of the reflection access. */
typedef struct ReflectionStringBufferJc_t
{
  ClassJc clazz;
  MT_void_Method_void vMethods[10];
}ReflectionStringBufferJc;
extern const ReflectionStringBufferJc reflectionStringBufferJc;
#define REFLECTION_StringBufferJc &reflectionStringBufferJc



#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)

/**The C++-Variante. It is comfortable to use C++ because some methods and operators are argument-sensitive (overloaded),
 * and the code is better readable.
 * This class defines only methods, and the methods are not virutal. It followes the java convention: The class java.lang.String is final.
 * Thats why a applicable side action exists: The memory layout of StringJcpp is identical with the memory layout of StringJc.
 * So conversions may be able without any problems. Using static_cast<StringJc*>
 */
class StringBufferJcpp: public ObjectifcBaseJcpp, public StringBufferJc
{
  public: StringBufferJcpp(char* buffer, int len){ ctor_s0i_StringBufferJc(this, buffer, len); }

  public: StringBufferJc* append(StringJc str){ return append_s_StringBufferJc(this, str); }

  public: void setLength(int newLength){ setLength_StringBufferJc(this, newLength); }

  public: StringBufferJc* delete_(int start, int end){ return delete_StringBufferJc(this, start, end); }

  public: void cleanToSize(){ cleanToSize_StringBufferJc(this); }

  public: ObjectJc* toObjectJc(){ return &this->object; }
  public: ObjectJcpp* toObjectJcpp(){ return null; }
};
#endif //__CPLUSPLUSJcpp

/*@CLASS_C IntegerJc@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**Parses the string argument as a signed decimal integer. The characters in the string must all be decimal digits,
 * except that the first character may be an ASCII minus sign '-' ('\u002D') to indicate a negative value.
 * The resulting integer value is returned, exactly as if the argument and the radix 10 were given as arguments
 * to the parseInt(java.lang.String, int) method.
 * @param s String containing the int representation to be parsed
 * @return the integer value represented by the argument in decimal.
 * @throws NumberFormatException - if the string does not contain a parsable integer.
 * @javaLike
 */
METHOD_C int parseInt_IntegerJc(StringJc src);


#endif //__StringJc_h__
