/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content This file defines only the struct ObjectJc_t and the base interface ObjectifcBaseJcpp.
 * It is the minimal base for some struct or interface
 * which should be designable without knowledge of the CRuntimeJavalike concept.
 * This Header should be included while using the CRuntimeJavalike concept
 * but a surrogating other header which define these two elements in a other way may be used otherwise.
 *
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut chg:The elements idSemaphore and idWaitingThread are combined to idSyncHandles.
 *                       New element offsetToHeapBlock, but not in use yet.
 * 2007-10-00: JcHartmut creation, dissolved from ObjectJc.h
 *
 ****************************************************************************/
#ifndef __objectBaseC__h__
#define __objectBaseC__h__




/*@CLASS_C ObjectJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

  /**Object is the superclass of all superclasses. In C-like manner it is a struct
   * at begin of any class-like struct.
   */
  typedef struct  ObjectJc_t
  {

    /**Info about object identification and the size of the data.
     * There are three informations in several bits:
     *
     * *bit 31,30: Indication wether it is a array (type ObjectArrayJc, max. 3-dimensional).
     *
     * *bit 29,28: Kind of the size information.
     *
     * *bit 28 to 24 or 27 to 20 or to 16: 5, 8 or 12 bits to indicate the object (instance).
     *
     * *bit 23 or 19 or 15 to 0: 24, 20 or 16 bits for size of the instance.
     *
     * The size may be in range 0 to 65k for ''small'' Objects or to 1 MByte or 16 MByte for
     * ''medium'' or ''large'' Objects. Bits 29,28 indicates this kind of designation.
     *
     * *If bit 29 is 1, it is a large object with up to 16 MByte, the sizeof-Bits are
     * Bit 23..0. There are 5 Bits to differ user Objects, bit 28 to 24. The user may have
     * upto 31 large Objects (instances) in one data focus or maybe in the whole software.
     *
     * *If bit 29, 28 is 01, it is a medium object with up to 1 MByte, the sizeof-Bits are
     * Bit 19..0. There are 8 Bits to differ user Objects, bit 27 to 20. The user may have
     * upto 255 medium Objects (instances) in one data focus or maybe in the whole software.
     *
     * *If bit 29, 28 is 00, it is a small object with up to 64 kByte, the sizeof-Bits are
     * Bit 15..0. There are 12 Bits to differ user Objects, bit 27 to 16. The user may have
     * upto 4095 medium Objects (instances) in one data focus or maybe in the whole software.
     *
     * The ObjectIdent-Bits may be 0, if no identification of the Object is used.
     * The user should take the responsibility for this bits.
     * The instance idents should be defined in user space.
     * it may be non-ambigous for a whole user project or not,
     * but it should be non-ambigous for every data set. It helps to identify data
     * in a plain old data (POD) structure.
     * The user can define this bits in a special definition as constants. This constants
     * should contain the kind-of-size bits 29 and 28.
     * The ObjectIdent-Bits are not contained in reflection information, this information is only getable here.
     *
     * The sizeof-Bits may be 0, if no sizeof information is necessarry. But this bits
     * are usefull if some different objects are places in a memory space after another, and
     * a memory evaluation may be processed. The memory evaluation can detect the type of the
     * objects via Reflection. but it need not use this information. The sizeof information
     * helps to skip from one Object to the followed.
     */
    int32 objectIdentSize;


      /** The bits defines an ObjectArrayJc. If this bits are 0, it is not an ObjectArrayJc.*/
      #define mArray_objectIdentSize_ObjectJc      0xc0000000

      /** If the Bits with mask mArray_objectIdentSize_ObjectJc are equal, it is a 1-dimensional ObjectArrayJc.*/
      #define kArray_objectIdentSize_ObjectJc      0x40000000

      /** If the Bits with mask mArray_objectIdentSize_ObjectJc are equal, it is a 2-dimensional ObjectArrayJc.*/
      #define kArray2_objectIdentSize_ObjectJc      0x80000000

      /** If the Bits with mask mArray_objectIdentSize_ObjectJc are equal, it is a 3-dimensional ObjectArrayJc.*/
      #define kArray3_objectIdentSize_ObjectJc      0xc0000000

      /** The bits defines which mask is used to get the size.*/
      #define mSizeBits_objectIdentSize_ObjectJc   0x30000000

      /** If this value compared with ,,objectIdentSize & mSizeBits_objectIdentSize_ObjectJc,, is true,
        * the size is stored in the bits of ,,mSizeSmall_objectIdentSize_ObjectJc...
        * The bits mask with ,,mTypeMedium_objectIdentSize_ObjectJc,, are used for type info.
        */
      #define kIsSmallSize_objectIdentSize_ObjectJc  0x00000000

      /** If this value compared with ,,objectIdentSize & mSizeBits_objectIdentSize_ObjectJc,, is true,
        * the size is stored in the bits of ,,mSizeMedium_objectIdentSize_ObjectJc...
        * The bits mask with ,,mTypeMedium_objectIdentSize_ObjectJc,, are used for type info.
        */
      #define kIsMediumSize_objectIdentSize_ObjectJc 0x10000000

      /** If this bit is set, the size is stored in the bits of ,,mSizeLarge_objectIdentSize_ObjectJc,,.
        * The bits mask with ,,mTypeLarge_objectIdentSize_ObjectJc,, are used for type info.
        */
      #define mIsLargeSize_objectIdentSize_ObjectJc  0x20000000

      /** Size bits for small objects, small means up to 64 kByte. There are 14 bits
        * useable for 16383 small types.
        */
      #define mSizeSmall_objectIdentSize_ObjectJc  0x0000ffff

      /** This bits are used to store the type or instance info for small objects.
        */
      #define mTypeSmall_objectIdentSize_ObjectJc  0x0fff0000

      /** Size bits for medium objects, medium means up to 1 MByte. There are 10 bits
        * useable for 1023 medium types.
        */
      #define mSizeMedium_objectIdentSize_ObjectJc 0x000fffff

      /** This bits are used to store the type or size info for medium objects.
        */
      #define mTypeMedium_objectIdentSize_ObjectJc 0x0ff00000

      /** Size bits for large objects up to 16 MByte.
        */
      #define mSizeLarge_objectIdentSize_ObjectJc  0x00ffffff

      /** This bits are used to store the type or instance info for large objects.
          There are only 5 bits useable for 31 large types.
        */
      #define mTypeLarge_objectIdentSize_ObjectJc  0x1f000000

  /** The own address of the instance. This information is usefull if the data of the instance
    * are transfered into another memory space (at example via file transfer).
    * All pointers inside this instance may be relativ build with this information.
    */
    /**The own address of the instance saved in the instance itself. This value may be important
     * if the data are transmitted in an other system (evaluation of data)
     * and some references between data are present. The evaluator may be able to found
     * the correct association between the instances by testing this value.
     */
    void const* ownAddress;

    /** Offset to the beginn of block, if this Object is part of a BlockHeapBlockJc, or 0 elsewhere.
     *  The offset is a negativ number.
     */
    int16 offsetToHeapBlock;

    /** identification number for wait/notify and synchronized.
     *  Because most of ObjectJc are not used for synchronized or wait/notify,
     *  the necessary handles for that are stored in a separate struct located in an separate memory space
     *  defined in os_wrapper.c.
     *  Here only the 2 bytes for a id are provided.
     */
    int16 idSyncHandles;
    #define kNoSyncHandles_ObjectJc -1


    /** The info about this type of the object. The Java-like reflection-concept is used.
    */
    struct ClassJc_t const* reflectionClass;


  }ObjectJc;




#if defined(__CPLUSGEN) && defined(__cplusplus)



  /** Superclass for class Object and also for all interface classes.
      This abstract base class defines a method to convert this to Object*
      especially necessary for interface classes.<br/>
      The methods of this class should be implemented in any class
      implementing any interface.
  */
  class  ObjectifcBaseJcpp
  {
    /** converts to the ObjectJcpp - pointer, superclass of all instances..
    A type casting to (ObjectJcpp*) applied to a pointer of interface type
    forces the using of the ObjectJcpp* - casting operator of the instance-type,
    mediate from the virtual table of the interface.
    Note that a casting to super or derived classes in C++ modifies the this-pointer
    with small adress modification to the right virtual table.
    The default implementation supplies no Reflection or ObjectJc-support.
    */
    public: virtual class ObjectJcpp* toObjectJcpp(){ return null; }

    /** converts to the ObjectJc- pointer, superclass of all instances..
    A type casting to (ObjectJc*) applied to a pointer of interface type
    forces the using of the ObjectJcpp* - casting operator of the instance-type,
    mediate from the virtual table of the interface.
    Note that a casting to super or derived classes in C++ modifies the this-pointer
    with small adress modification to the right virtual table.
    The default implementation supplies no Reflection or ObjectJc-support.
    */
    public: virtual ObjectJc* toObjectJc(){ return null; }

    //public: operator ObjectJcpp*(){ return toObjectJc(); }
    //public: ObjectJcpp*(){ return operator ObjectJcpp*(); }

    //public: virtual ClassJc_t const* getClass() const =0;

    /** finalize may overwritten to close something, called by garbage collector.*/
    //protected: virtual void finalize(){}

    /*Note: if the virtual mehtods of class are defined here, there should also defined
     *in the users class. It is not sufficing to define it in ObjectJcpp, because some Interfaces
     *also based on this class. The C++-compiler may not assign the methods from one baseclass (ObjectJcpp)
     *to another Baseclass (the interface).
     *To access getClass() and the others from ObjectJcpp you must use toObjectJc()
     */
    //public: virtual struct ClassJc_t* getClass()=0;

  };

#endif //defined(__CPLUSPLUSJcpp) && defined(__cplusplus)







#endif //__objectBaseC__h__
