/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content This file contains routine declarations to call the operation system level.
 *
 * @author Jchartmut www.vishia.org
 * @version 0.81
 * list of changes:
 * 2007-10-00: JcHartmutwww.vishia.de creation
 *
 ****************************************************************************/
#ifndef __os_wrapper_h__
#define __os_wrapper_h__


#include "simpleC.h"

/**gets the ThreadContext of the current Thread.
 * @return The pointer and size of the thread context without type information.
 *         The user is responsible for test of size and type conversion.
 *         Normally the given size setted with os_setThreadContextSettings should be returned.
 *         If the os_setThreadContextSettings-routine is not called before, the routine returns {0, null}.
 */
METHOD_C MemC os_getCurrentThreadContext();

/**The thread routine should have the followed prototype.
   @param data Pointer to data. Mostly it is a class structure.
   @return The exit value of the thread. The operation system can protocoll or show the exit value in a special kind.
 */
typedef void OS_ThreadContextInitRoutine(MemC threadContextRaw);

/**sets the size and method to create a ThreadContext.
 * A ThreadContext is a struct managed in the users space which is thread-specific.
 * A user can get always the pointer to the context of the current thread, but never to another thread.
 * But it is possible that threads are not created with the os_createThread-routine but with other os calls.
 * If the thread has not a ThreadContext, the named routine is called to initilize it.
 * This settings are valid for the whole executable.
 */
METHOD_C void os_setThreadContextSettings(OS_ThreadContextInitRoutine* initRoutine, int sizeofThreadContext);







/**The thread routine should have the followed prototype.
   @param data Pointer to data. Mostly it is a class structure.
   @return The exit value of the thread. The operation system can protocoll or show the exit value in a special kind.
 */
typedef int OS_ThreadRoutine(void* data);

/**creates a thread. The thread routine will be started immediately.
 * The thread exists as long as the routine processes. If the routine returns, the thread will be deregistred.
 * It is not planning that a thread is registered in the os level but it is not started yet.
 * If the thread routine should be wait for starting event, use the os_wait / os_notify mechanism.
 * @param handle Pointer to a pointer-type variable holding the os-specifc handle for the thread.
 * @param routine The start routine.
 * @param data Pointer to data for the routine.
 * @param name Name of the Thread. It is imaginable, that the os will register threads with a name.
 * @param prio The priority of the thread. Values from 1 (lowest) to 255 (highest) are supported.
               If the operation system doesnot support this number of levels,
               it is admissible that some priorities are mapped to the same level.
               The application should not depends on the execution order,
               exclusively the hard realtime should be controled by fine priority attention.
 * @param stacksize The size of the stack. If it is 0, a system specific default size is used.
 */
int os_createThread(struct OS_HandleThread_t const** handle, OS_ThreadRoutine routine, void* data, const char* name, int prio, int stacksize);





/**creates a object for wait-notify Object. The inner type of such an Object is os-dependent.
 * Here only a forward declared struct pointer is knwon.
 * @return 0 if no error, or an error code.
 */
int os_createWaitNotifyObject(struct OS_HandleWaitNotify_t const** waitObject);


/**removes a object for wait-notify.
 */
int os_removeWaitNotifyObject(struct OS_HandleWaitNotify_t const* idxWaitObject);





/** Waits for a notification.
 */

int os_wait(struct OS_HandleWaitNotify_t const* waitObject, uint32 milliseconds);


/** Notifies all waiting thread to continue.
 */
int os_notifyAll(struct OS_HandleWaitNotify_t const* waitObject);


/** Notifies only one waiting thread to continue.
 */
int os_notify(struct OS_HandleWaitNotify_t const* waitObject);


/**A simple sleep.
 * @param millisecond It must be a positive value.
 * @return 0 if the given time is up. return < 0 if any error occurs.
 */
int os_sleep(int32 milliseconds);










METHOD_C MemC os_allocMemC(int size);

METHOD_C int os_freeMemC(MemC mem);





/**Liefert den Zhlerstand eines CPU-weit verfgbaren Zhlers in Auflsung von 1 Millisekunden, umlaufend in 48 Tagen.
   Der Zhler ist nicht an eine absolute Zeit gebunden, zhlt aber gleichmig.
*/
int32 os_milliTime();

/**Liefert den Zhlerstand eines CPU-weit verfgbaren Zhlers in Auflsung von 1 Millisekunden, umlaufend in ca.1 Stunde.
   Der Zhler ist nicht an eine absolute Zeit gebunden, zhlt aber gleichmig.
   Es ist nicht garantiert, dass in allen Umgebungen die tatschliche Auflsung der Zhlerstnde 1 Mikrosekunden betrgt.
*/
int32 os_microTime();


#endif //__os_wrapper_h__
