/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @content This file contains basicly definitions and declarations using in C
 * independent of the CRuntimeJavalike concept.
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut some changes in definitions to treat with memory:
 *                       * MemAreaC is shown with 32-bit-integer in debugger, not with char.
 *                       * The macros addOffset_MemAreaC and offset_MemAreaC work with any type of pointer.
 *                       * The element size_ of MemC should not be directly used outside, use the macro size_MemC
 *                         and setSize_MemC instead. size_ stores also an offset for treating in BlockHeap.
 * 2007-10-00: JcHartmut creation
 *
 ****************************************************************************/
#ifndef __simpleC_h__
#define __simpleC_h__
/**This file contains some usefull definitions for simpe C programming.

*/
#include "complctr.h"

//prevent multiple include if os_mem.h is included before
#ifndef __MemC_h__
#define __MemC_h__  //prevent multiple include if os_mem.h is included behind

/*@DEFINE_C specialDefines @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/** This type represents a pointer to a void name(void) - method.
  * It is needed on some locations in software.
  */
typedef void(* MT_void_Method_void)(void);

/** This type represents a pointer to a int name(int) - method.
  * It is needed on some locations in software.
  */
typedef int(* MT_int_Method_int)(int);


#define SIMPLE_CAST(Type, value) ((Type)(value))

#if defined(__cplusplus) && defined(__CPLUSGEN)
  #define STATIC_CAST(type, value) (static_cast<type>(value))

#else //not __CPLUSGEN

  #define STATIC_CAST(Type, value) ((Type)(value))
#endif


/**Calculates on compile time the number of elements of given array instance.
 * @param ARRAY a instance of an array type.
 */
#define ARRAYLEN(ARRAY) (sizeof(ARRAY)/sizeof(ARRAY[0]))





/*@CLASS_C Int32Array256 @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/** This structure is useable if raw data should be mapped, it may typical used as pointer type.
  * The size of the array is a free choiced number, appropriate to show the data in debugger.
*/
typedef struct Int32Array256_t
{ int32 data[256];
}Int32Array256;





/*@CLASS_C MemAreaC @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/** This structure is useable if raw data should be mapped, it should only used as pointer type.
  * The size of the array is a free choiced number, appropriate to show the data in debugger.
*/
typedef struct MemAreaC_t
{ /**This is only a struct to see the data in debugger. Do not use it directly.*/
  int32 __[20];
} MemAreaC;


//typedef MemUnit MemAreaxxx[1000];

/**returns a pointer to MemAreaC with OFFSET in memory.
 * @param BASE a pointer from type MemAreaC*. The type will be tested by compiling.
 * @param OFFSET a address-offset (sizeof-Type)
 * @return pointer from type MemAreaC* at memory location BASE + OFFSET
 */
#define addOffset_MemAreaC(BASE, OFFSET) ( (MemAreaC*) (((MemUnit*)(BASE)) + (OFFSET)) )

/**offset from BASE to BASE2.
 * @param BASE a pointer from type MemAreaC*. The type will be tested by compiling.
 * @param BASE2 a pointer from type MemAreaC*. The type will be tested by compiling.
 * @return number of memory address locations from BASE to BASE2
 */
#define offset_MemAreaC(BASE, BASE2) ((MemUnit*)(BASE2) - (MemUnit*)(BASE));




/*@CLASS_C MemC @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/**This class provides basic working with memory allocation.
 * The Problem in C is: a Pointer to memory is not interpretable. It is a void* mostly.
 * How many bytes are there, it is unknown.
 *
 * The struct MemJc contains both:
 * * The pointer to the data as pointer to memory layout.
 * * The size of data in memory.
 * But there is no type info.
 */
typedef struct MemC_t
{ int32 size_;
  MemAreaC* address;
}MemC;



#ifdef  __cplusplus
extern "C" {
#endif


/**gets the size, max is limited (64 kByte) if there is a offset to the BlockHeapBlock is given. */
#define size_MemC(THIS) ( (THIS).size_ & 0x80000000 ? (THIS).size_ & 0xffff : (THIS).size_)

/**gets the offset to block, it is negative or 0 if no offset to BlockHeapBlockJc is given. max is 0x400 Byte blocksize. */
#define offsetBlock_MemC(THIS) ((THIS).size_ & 0x80000000 ? (int16)((THIS).size_ >>16) : 0)


/**Sets the size and offset.
 * The offset should be either 0 (if the MemC contains not an BlockHeapBlockJc-location) or it should be a negativ number.
 * If the offset is 0, the size is only limited to 31 or 15 bits (15 bit for simple 16-bit-controller).
 * If the offset is negativ, the SIZE must not exceed the number of associated bits.
 * But it is not checked here, because this macro is not used in user space,
 * but typicall used only at implementation layer of the os system adapter.
 */
#define setSizeAndOffset_MemC(THIS, SIZE, OFFSET) ((THIS).size_ = (SIZE) + ((OFFSET)<<16))


METHOD_C MemC alloc_MemC(int size);

METHOD_C MemC init0_MemC(MemC mem);

METHOD_C int free_MemC(MemC mem);

/**Builds a MemC struct and returns it per value.
 * This is a helper method to supply a MemC argument in a method call.
 */
METHOD_C MemC build_MemC(void* address, int size);

#ifdef  __cplusplus
} //extern "C"
#endif


/**Allocates the amount of bytes in heap,but do not initialize it. */
#define MALLOC(TYPE) ((TYPE*)malloc(sizeof(TYPE)))

#endif //__MemC_h__


/**allocates and initializes with 0.
 */
MemC alloc0_MemC(int size);


#endif  //__simpleC_h__
