package org.vishia.exampleJava2C.emulationEnv;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import org.vishia.byteData.ByteDataAccess;
import org.vishia.exampleJava2C.java4c.MainController;
import org.vishia.exampleJava2C.java4c.ProcessMessageIdents;
import org.vishia.util.StringFormatter;


public class MainEmulation implements iRequireMainController
{
  /**Way in microMeter */
  double way1, way2;
  
  double disturbance1 = 1.0;
  
  /**Voltage as output from controller to the both movement motors. */
  short voltage1, voltage2;
  
  /**The continuously time. */
  int time =0;
  
  /**The sampling interval in microsecond. */
  int samplingInverval = 100;
  
  final MainController theController;
  
  final LogMessageJc msg;
  
  
  
  /**An instance of interface WaySensor implemented as immediate class definition
   * to access the value way1 from MainEmulation.
   */
  WaySensor waySensor1 = new WaySensor()
  {
    
    public int getWay()
    { return (int)(way1);
      
    }
  };
  
  
  /**A second instance of interface WaySensor implemented as immediate class definition
   * to access the value way2 from MainEmulation.
   */
  WaySensor waySensor2 = new WaySensor()
  {
    public int getWay()
    { return (int)(way2);
      
    }
  };
  
  
  /**An instance of interface WayActuator implemented as immediate class definition
   * to affect the value way1 from MainEmulation.
   */
  WayActuator wayActuator1 = new WayActuator()
  {
    public void setMotorVoltage(short voltage)
    { voltage1 = voltage; 
    }
  };
  
  
  
  /**An instance of interface WayActuator implemented as immediate class definition
   * to affect the value way1 from MainEmulation.
   */
  WayActuator wayActuator2 = new WayActuator()
  {
    public void setMotorVoltage(short voltage)
    { voltage2 = voltage; 
    }
  };
  
  
  private MainEmulation() 
  throws IOException
  { writeValues = new WriteValues();
    msg = new ProcessMessageImpl();
    theController = new MainController(this, msg, null);
    way1 = 1000000;  //start at 1 m
    way2 = 1000000;
  }
  
  
  void step()
  { time +=1;
    if(time == 551)
      stop();
    /* the amplification of the actuator-motor is disturbed, it is between 0.4 and 1.4
     * it causes an exception 
     * because the software MainController.java has a difficult error to demonstrate this:
     * If the output from PID-Controller is greater than 20000, it is limited,
     * because the actuator has a range from -10000 to 10000. But if the limitation occurs,
     * the integral part of PID-ctrl should be fixed. This isn't implemented. So the integral
     * has an overflow and an exception is thrown. 
     * If the amplification is more less, the control deviation is reached a programmed limit,
     * so the PID-controller is switched off and no exception occurs.
     * If the amplification is greater, the actuator value is no limited and no exception occurs.
     * It is only a small range.
     */    
    disturbance1 = 0.9 + 0.5*Math.sin( ((double)(time % 1000))/1000 * 2* Math.PI);
    //10 V moves 100 mm per second.
    way1 += 100.0 * ((double)(voltage1))/samplingInverval * disturbance1;
  }



  public WayActuator requireWay1Actuator()
  { return wayActuator1;
  }



  public WaySensor requireWay1Sensor()
  { return waySensor1;
  }

  public WayActuator requireWay2Actuator()
  { return wayActuator2;
  }



  public WaySensor requireWay2Sensor()
  { return waySensor2;
  }



  public final static void main(String[] args)
  { try
    {
      (new MainEmulation()).execute();
    }
    catch (IOException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }
  
  
  
  /**This is the run method of the execution. */
  private void execute() throws IOException
  {
    short[] targets = { 15000, 17000, 11000};
    int idxTarget = 0;
    
    for(int ii = 0; ii < 2000; ii++)
    { 
      theController.step();  //the main control thread.
      
      //it may be also in another thread: set target values:
      if(idxTarget < targets.length)
      { if(theController.setTarget(targets[idxTarget]))
        { idxTarget +=1; //increment it only if it was successfull.
        }
      }
      
      step();  //the emulation of environment in main control thread.
      writeValues.write();
    }
    
  }
  
  
  /**Support write values in graphical output. */
  private class WriteValues
  {
    final StringBuffer buffer = new StringBuffer(100);
    
    /**Graphical output. */
    final OutputStream grvOut;
    
    StringFormatter out = new StringFormatter(buffer);
    
    /**Helper class to write float values into a byte[]. */
    class GrvLineAccess extends ByteDataAccess
    { /**Constructor creates a byte buffer for 16 float values. */
      GrvLineAccess(){ assignEmpty(new byte[64]);}
      
      /**abstract method of baseclass: set empty data. */
      @Override protected void specifyEmptyDefaultData(){ for(int ii=0; ii < data.length; ii++){ data[ii]= 0;} }
      
      
      @Override protected int specifyLengthElement() throws AccessException{ return 64;}
      @Override public int specifyLengthElementHead(){ return 64;}
      public void setValue(int idx, float val){ setFloat(0, idx, 16, val);}
    };
    
    final GrvLineAccess grvLineAccess;
    
    WriteValues() throws IOException
    { grvOut = new FileOutputStream(new File("out/out.grv"));

      grvLineAccess = new GrvLineAccess();
    }
    
    public void write() 
    throws IOException
    {
      out.reset()
         .addint(time, "22221")
         .add("   w=").addint(theController.getWaySetvalue(),"1.1111 m ")
         .add(" x").addint((int)way1,"+1.111'111 m ")
         .add("  d=").addint(theController.getWayOffset(),"+22221.111 mm ")
         .add(" V=").addint(voltage1,"-221.111 V ")
         .add("  intg=").addint((int)(theController.getWay11Intg()*10),"-22221.1")
         ;
      //System.out.println(out.getContent());
      msg.sendMsg(ProcessMessageIdents.kProcessValueInStep, out.getContent());
      out.reset();
      
      
      
      grvLineAccess.setValue(0,theController.getWaySetvalue());
      grvLineAccess.setValue(1,theController.getWay11Intg());
      grvLineAccess.setValue(2,theController.getWayOffset());
      grvLineAccess.setValue(3,(float)(way1));
      grvLineAccess.setValue(4,voltage1);
      
      grvOut.write(grvLineAccess.getData(),0,64);
      
    }
    
  }
  
  
  WriteValues writeValues;
   
  
  void testQuadrad()
  { float x = -10.0E30F;
    float y = x*x;
    float z = y + 3.0F;
    float a = z /0.0F;
    System.out.println(y);
  }
  
  
  void stop()
  { //debug stop
  }

}




