package org.vishia.exampleJava2C.java4c;

import org.vishia.exampleJava2C.emulationEnv.MainEmulation;
import org.vishia.exampleJava2C.emulationEnv.LogMessageJc;
import org.vishia.exampleJava2C.emulationEnv.WayActuator;
import org.vishia.exampleJava2C.emulationEnv.WaySensor;
import org.vishia.exampleJava2C.emulationEnv.iRequireMainController;

/**This class is the example main class to control a movement.
 * 
 * @author JcHartmut
 * @staticInstance CC: Instances of tis class will be created 
 *                     only in the construction path of the system.
 */ 
public class MainController extends TestSuper
{
  /**The message interface.*/
  private final LogMessageJc msg;
  
  /**Aggregation to a way sensor for input for control. */
  private final WaySensor way1Sensor, way2Sensor;
  
  /**Aggregation to a way actuator, servomotor or such, output for control. */
  private final WayActuator way1Actuator, way2Actuator;
  
  /** The set value for the way in 0.1mm-units from above -3 m to 3 m. */
  private short wWay = 10000;  //1 m as initial.
  
  /**controller offset (deviation). */
  private int dWay;
  
  /**Simple set value generator as first example: add 0.5 mm per step in cycle time. */
  private short addWay = 5;
  
  /**A Set value generator used hence second example using new. */
  private final SetValueGenerator mySetValueGenerator;
  
  /**The actuating variable for the movement. It is a voltage for a motor in range -10000 to 10000, unit milliVolt. */
  private short yOut;

  /**The PID controller. 
   * CC: In C it is an embedded struct because the reference is final (given from construction, fix)
   *     and created here.
   */
  private final PID_controller pidCtrl1 = new PID_controller();
  
  /**A interface reference to test.
   * CC: This reference uses a reference struct with 2 values, the reference to the Object_Jc
   *     and a value which contains the index of jump table of the instance for interface method call.
   */
  private Testifc testifc;
  
  /**A reference to test the garbage collection.
   * CC: This reference uses a reference struct with 2 values, the reference to the instance
   *     and a value which contains beside the index of jump table of the instance for dynamic method call
   *     the index of back reference from the instance block in BlockHeap.
   */
  private TestClass testDynamicReference;
  
  
  /**A reference to test which references need not 2 values.
   * final, it means it is given from startup and it is never destroyed if this instance itself is not destroyed.
   * TestClass, it is a final class so no virtual methods are called.
   */
  final TestClassFinal testStaticReference;
  
  private class TestSubClass
  { int a;
    int getTest_a(){ return a; }
  }  
  

  /**The constructor of this Controller assumes, that the depended instances all built always. 
   * @param broker Interface contains all require methods to depended instances.
   */
  public MainController(iRequireMainController broker, LogMessageJc msg, TestClassFinal testStaticReference)
  { super(5);
    this.msg = msg;
    mySetValueGenerator = new SetValueGenerator(msg);
    this.testStaticReference = testStaticReference;
    way1Sensor = broker.requireWay1Sensor();
    way2Sensor = broker.requireWay2Sensor();
    way1Actuator = broker.requireWay1Actuator();
    way2Actuator = broker.requireWay2Actuator();
  }
  
 
  /**Gets the set value in units mm*/
  public short getWaySetvalue(){ return wWay; }
  
  /**Gets the offset of the controller on input*/
  public int getWayOffset(){ return dWay; }
  
  
  /**This is the cyclical called method to execute the controlling. */
  public void step()
  {
    //reads the inputs from Hardware, it is in units 1 micrometer.
    int xWay = way1Sensor.getWay();

    //first example: way moves up and down between 1.000 m and 2.000 m in 5 mm per step (1m/s if sample time = 5 millisec)
    boolean bFirstExample = false;
    if(bFirstExample)
    { if(wWay >= 20000)
      { 
        addWay = -50; 
        pidCtrl1.setIntg((short)-yOut);
  
      }
      else if(wWay <= 10000)
      { addWay = 50; 
        pidCtrl1.setIntg((short)-yOut);
      }
      wWay += addWay;
    }
    else
    { //for good example:
      wWay = mySetValueGenerator.step();
    }
    /**difference of way in units 1 Micrometer, range should be -32..32 mm. */
    dWay = xWay - 100 * wWay;

    /* If the difference is outside a capturing range, the controller is off 
     * and the output value is set immediately. 
     */
    if(dWay > 30000)
    { //The way is out of range, move down.
      yOut = -20000;
      pidCtrl1.setIntg(yOut);
    }
    else if(dWay < -30000)
    { //The way is out of range, move up.
      yOut = 20000;
      pidCtrl1.setIntg(yOut);
    }
    else
    { /*call of PID-control. */ 
      yOut = pidCtrl1.step((short)(-dWay));
      /*The yOut have an output range from -16768 to 16767, limit the output value: */ 
      if(yOut > 19999)
      { yOut = 19999; 
        /* the next statement is commented, because it may be "forgotten",
         * therefore the integral value runs away and has an overflow than.
         * the overflow is recognized and an exception is thrown.
         * If the statement is not commented, the controller is stable.
         */
        //pidCtrl1.setIntg(yOut);
      }
      else if(yOut < -19999)
      { yOut = -19999; 
        //next statement commented, see above.
        //pidCtrl1.setIntg(yOut);
      }
    }  
    
    way1Actuator.setMotorVoltage((short)(yOut>>1));
      
  }

  /**Get the internal value of the integrator of the first way controller. */
  public float getWay11Intg(){ return pidCtrl1.getIntg(); }

  /**stores a next target point. This method may be called in a slower thread.
   * If there is no space for storing the demand, the value isn't be stored.
   * The method should be called later once more.
   * 
   * @param targetValue the target point value.
   * @return true if it is stored, false if there is no space to storing. 
   */ 
  public boolean setTarget(short targetValue)
  { //it is a delagation pattern. The outside user don't know the SetValueGenerator.
    return mySetValueGenerator.setTarget(targetValue);
  }
  

}


class TestSuper
{ int b;
  TestSuper(int a)
  { b=a;
  }
}
