#include "ExceptionJc.h"
#include "MainEmulation.h"

#include "MainController.h"
#include "iRequireMainController.h"
#include <stdio.h>
#include <string.h>
#include "testSomeDynamicLinkedMethodConcepts.h"
#include "LogMessageJc.h"

#include "Main.h"
#include <stdio.h>

/**static data of BlockHeap concept. */


/**static data in C instead new in Java. */
MainController mainController;

/**static data for the simulation environment in C.
 */ 
MainEmulation mainEmulation;


/**Subroutine for setting new targets to the controller. */
void setTargets_Main();

/**Subroutine to write the state to the logMessage-system. */
void logMsg_Main(int nStep);


/**In this phase of example the main routine simulates the time-behaviour of a real controlling system and the environment.
 * It is all done in one thread.
 * In main a while-loop is executed. Each pass of the while-loop is adequate to one sampling time of the controller.  
 */
void main()
{ STACKTRC_ENTRY("main");
  initBlockHeap();
  //testWaitNotify();
  testSomeDynamicLinkedMethodConcepts();
  
  { int nStep = 0;
    /* The manually written and static instanciated struct should be cleared at least.
    */
    memset(&mainEmulation, 0,sizeof(mainEmulation));
    /* For a generated but manually instanciated struct, call the generated constructor to init it.
     * because the interface concept is not implemented yet and the required instances are singleton,
     * the pointer to iRequireMainController_t is given with null, it is not used.
     */
    ctor_MainController(build_MemC(&mainController, sizeof(mainController)), null, null, null, _threadContext);
    ctor_MainEmulation(build_MemC(&mainEmulation, sizeof(mainEmulation)));       
    //
    test_StacktraceJc(&stacktrace);
    //start of the run mode:
    setRunModeBlockHeap();

    TRY
    { int countGcTime = 1;
      while(true)
      { test_StacktraceJc(&stacktrace);
        if(nStep == 210)
        { /* changing the amplification of actuator causes an exception,
           * because the software MainController.java has a difficult error to demonstrate this:
           * If the output from PID-Controller is greater than 20000, it is limited,
           * because the actuator has a range from -10000 to 10000. But if the limitation occurs,
           * the integral part of PID-ctrl should be fixed. This isn't implemented. So the integral
           * has an overflow and an exception is thrown. 
           * If the amplification is more less, the control deviation is reached a programmed limit,
           * so the PID-controller is switched off and no exception occurs.
           * If the amplification is greater, the actuator value is no limited and no exception occurs.
           * It is only a small range.
           */ 
          mainEmulation.nAmplificationActuator = 0.45F;
        }
        step_MainController(&mainController, _threadContext); //step sample times of the controller.
        step_MainEmulation(&mainEmulation);                   //step sample times of the enviroment simulation.         
        setTargets_Main();                                    //step thread to set target points.
        nStep +=1;

        logMsg_Main(nStep);

        if(nStep == 3000) break;
        if(--countGcTime < 0)
        { countGcTime =100;
          garbageCollection(null);
        }
      }
    }_TRY
    CATCH(RuntimeException, exc)
    { printStackTrace_ExceptionJc(exc, stdout);
    }
    END_TRY
  }
  STACKTRC_LEAVE;
}



/**Subroutine for setting new targets to the controller. */
void setTargets_Main()
{ static short targets[] = { 15000, 17000, 11000};
  static int targets_length = sizeof(targets) / sizeof(targets[0]);
  static int idxTarget = 0;
  STACKTRC_ENTRY("setTargets_Main");
  //it may be also in another thread: set target values:
  if(idxTarget < targets_length)
  { if(setTarget_MainController(&mainController, targets[idxTarget], _threadContext))
    { idxTarget +=1; //increment it only if it was successfull.
    }
  }
  STACKTRC_LEAVE;
}



void logMsg_Main(int nStep)
{ char buffer[200];
  STACKTRC_ENTRY("logMsg_Main");
  sprintf(buffer, "%5.5i x=%1.4f w=%1.4f m, dw=%2.3f mm, intg=0x%8.8X = %i"
         , nStep
         , mainEmulation.hardware.way1/1000000.0F
         , mainController.wWay/10000.0F
         , mainController.dWay/1000.0F
         , mainController.pidCtrl1.intgVal
         , mainController.pidCtrl1.intgVal
         );
  sendMsg_LogMessageJc(null, 1, buffer, _threadContext);
  STACKTRC_LEAVE;
}





MainEmulation* ctor_MainEmulation(MemC mem)
{ MainEmulation* ythis = (MainEmulation*)mem.address;
  STACKTRC_ENTRY("ctor_MainEmulation");
  if(size_MemC(mem) < sizeof(MainEmulation)) THROW(RuntimeException,"fault size", size_MemC(mem));
  init0_MemC(mem);  
  /*For emulation, set the addresses of the hwPorts to the existing emulation variables.*/
  ythis->waySensor1.hwPort = &ythis->hardware.way1;  
  ythis->waySensor2.hwPort = &ythis->hardware.way2;  
  ythis->wayActuator1.hwPort = &ythis->hardware.motor1;  
  ythis->wayActuator2.hwPort = &ythis->hardware.motor2;  
  ythis->hardware.way1 = 1000000L;
  ythis->hardware.way2 = 2000000L;
  ythis->nAmplificationActuator = 1.7F;
  STACKTRC_LEAVE;
  return ythis;
}



void step_MainEmulation(MainEmulation* ythis)
{
  ythis->hardware.way1 += (int32)(ythis->nAmplificationActuator* ythis->hardware.motor1);  //integrate the way because voltage!
  ythis->hardware.way2 += 2* ythis->hardware.motor2;  //integrate the way because voltage!
}


struct WaySensor_t* requireWay1Sensor_iRequireMainController(iRequireMainController* broker, StacktraceThreadContext_s* _stacktraceThreadContext)
{ //ignore the broker, because it is all singleton. Easy in C.
  return &mainEmulation.waySensor1;
}

struct WaySensor_t* requireWay2Sensor_iRequireMainController(iRequireMainController* broker, StacktraceThreadContext_s* _stacktraceThreadContext)
{ //ignore the broker, because it is all singleton. Easy in C.
  return &mainEmulation.waySensor2;
}

struct WayActuator_t* requireWay1Actuator_iRequireMainController(iRequireMainController* broker, StacktraceThreadContext_s* _stacktraceThreadContext)
{ //ignore the broker, because it is all singleton. Easy in C.
  return &mainEmulation.wayActuator1;
}

struct WayActuator_t* requireWay2Actuator_iRequireMainController(iRequireMainController* broker, StacktraceThreadContext_s* _stacktraceThreadContext)
{ //ignore the broker, because it is all singleton. Easy in C.
  return &mainEmulation.wayActuator2;
}

