/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author Jchartmut from www.vishia.org
 * @version 2008-03-23  (year-month-day)
 * list of changes:
 * 2008-03-23: JcHartmut creation. It is the sunday of eastern.
 *
 ****************************************************************************/

/**This source is written to test and show the concept of the virtual method call in C at a simple example.
 * Normally most of the source lines should be generated automatically by Java2C translator from a Java source,
 * especially the reflection and the jump table of virtual methods.
 * But in this case the source lines are manually written to examine the functions.
 */

#include "testSomeDynamicLinkedMethodConcepts.h"
#include "ObjectJc.h"
#include "ExceptionJc.h"
#include "StringJc.h"
#include "ReflectionJc.h"



/**An example struct based on ObjectJc with one additional value. */
typedef struct MyStruct_t
{ ObjectJc object;
  int32 a;
}MyStruct;



/**The method toString_ObjectJc should be overloaded with this method.
 * The argument list have to be the same as MT_toString_ObjectJc.
 * @param mem It is the place to store the string itself.
 */
StringJc toString_MyStruct(ObjectJc* othis)
{ MyStruct* ythis = (MyStruct*)(othis);
  StringBufferJc* buffer;
  STACKTRC_ENTRY("toString_MyStruct");
  
  buffer = new_StringBufferJc(0);
  append_s0_StringBufferJc(buffer, "MyStruct a=");
  append_i_StringBufferJc(buffer, ythis->a);
  STACKTRC_LEAVE;
  return toString_StringBufferJc(buffer);
}


/**To organize dynamic link method call the jump table of virtual methods is neccessary. */
struct Jtbl_MyStruct_t
{ Jtbl_ObjectJc jtblObjectJc;  //same method types as ObjectJc
}jtblMyStruct
=
{ { clone_Object_F
  , equals_Object_F
  , finalize_Object_F
  , hashCode_Object_F
//  , toString_Object_F
  , toString_MyStruct          //but this method is overwritten in opposition to ObjectJc
  }
};


/**To organize dynamic link method call the reflection is neccessary. 
 * But only the pointer to the jtbl is needed. The rest is uninteresting yet.
 */
ClassJc reflection_MyStruct
=
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(ClassJc), &reflection_MyStruct, null)
, "MyStruct"
, sizeof(MyStruct)
, 0
, null  //attributes and associations
, null  //method
, null  //superclass
, null  //interfaces
, 0     //modifier
, &jtblMyStruct.jtblObjectJc.clone  //NOTE: the first virtual method of ObjectJc is the reference base.
};


/**The constructor of MyStruct is necessary to tidily work. Especially the reflection is to be set.
 */
MyStruct* ctor_MyStruct(MemC mem)
{ MyStruct* ythis = (MyStruct*)mem.address;
  ctor_ObjectJc(mem);
  ythis->object.reflectionClass = &reflection_MyStruct;
  return ythis;
}

/**Now the test method for dynamic linked call.
 */ 
void testSomeDynamicLinkedMethodConcepts()
{
  MyStruct myStruct;                                          //struct yet in stack local
  ctor_MyStruct(build_MemC(&myStruct, sizeof(myStruct)));     //to initialize use the constructor.
  { StringJc text = toString_ObjectJc(&myStruct.object); //mbuffer.mem);
    printf(text.ref);
  }

}


