package org.vishia.header2Reflection;

import java.io.File;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.vishia.mainCmd.Report;
import org.vishia.util.FileSystem;


import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;
import java.util.Iterator;

import org.vishia.util.StringPart;
import org.vishia.util.StringPartFromFileLines;
import org.vishia.zbnf.ZbnfParseResultItem;
import org.vishia.zbnf.ZbnfParser;

public class Header2Reflection
{
  
  /**Aggregation to the Console implementation class.*/
  Report console;

  class FileIn
  {
    /**Input files. */
    final List<File> listFileIn = new LinkedList<File>();
    
    /**Position in absolute path of start of relative path, may be 0. */
    int posPath;
  }
  
  
  /**Input files. */
  final List<FileIn> listFileIn = new LinkedList<FileIn>();
  
  /**The parser. */
  ZbnfParser parser;
  
  String sFileZbnf;
  
  /**File-path of the file with the type info for Reflection generating.*/
  String sFileReflectionTypes;
  
  
  /**File-path of the file with the type info to write out from Reflection generating useable for next pass.*/
  String sFileReflectionTypesOut;
  
  /**File-path of the file with the infos for blocked types and files. */
  String sFileReflectionBlockedTypes;
  
  /**True if a new file for types should be created, false if append. */
  boolean bNewFileReflectionTypesOut = false;
  
  String sFileAllC;
  
  /**Cmdline-argument, set on -o option. Outputfile to output something. :TODO: its a example.*/
  String sOutDir = null;

  
  
  private final static String scalarTypes = "0-void:1-bool:1-boolean:4-float:4-float32:8-double:1-char:4-int:2-short:1-byte:4-int32:4-uint32:2-int16:2-uint16:1-int8:2-uint8:";
  
  private final static String sSyntaxReflectionTypes 
    = "$setLinemode. ReflectionTypes::={ <reflectionType> \\n} \\e."
    + "reflectionType::= <!.?sign> <* ,\n?path> [, {<?type> <!.?sign> <$?name> ? , } ].";
    
  
  private static class TypeEntry
  { final String sType;
    final String sPath;
    char sign;
    
    TypeEntry(String sPath, String sType, char sign)
    { this.sPath = sPath; this.sType = sType; this.sign = sign; }
      
  }
  
  TreeMap<String, TypeEntry> blockedFilesAndTypes = new TreeMap<String, TypeEntry>();
  
  TreeMap<String, TypeEntry> types = new TreeMap<String, TypeEntry>();
  
  TreeMap<String, List<TypeEntry>> typesByFilename = new TreeMap<String, List<TypeEntry>>();
  
  
  /**created header and c file. */
  BufferedWriter fileH;
  /**created header and c file. */
  BufferedWriter fileC;
  
  /**If setOutC is called, this writer is present, else it is null. */
  BufferedWriter fileAllC;
  /**If setOutC is called, this writer is present, else it is null. */
  BufferedWriter fileAllH;
  
  void setSyntax(String sFileZbnf)
  { this.sFileZbnf = sFileZbnf;
  }

  void setReflectionTypes(String sFile)
  { this.sFileReflectionTypes = sFile;
  }

  void setReflectionTypesOut(String sFile, boolean bNew)
  { this.sFileReflectionTypesOut = sFile;
    bNewFileReflectionTypesOut = bNew;
  }

  void setReflectionBlockedTypes(String sFile)
  { this.sFileReflectionBlockedTypes = sFile;
  }

  void setOutC(String sFile)
  { this.sFileAllC = sFile;
  }

  void setOutDir(String sFile)
  { this.sOutDir = sFile;
  }

  boolean addInputFilemask(String sMask)
  { boolean bOk = true;
    int posSep = sMask.indexOf(':');
    FileIn fileIn = new FileIn();
    File dir = null;
    if(posSep > 2)
    { String sDir = sMask.substring(0, posSep);
      dir = new File(sDir);
      if(dir.exists())
      { sMask = sMask.substring(posSep+1);
        String sDirAbs = dir.getAbsolutePath();
        fileIn.posPath = sDirAbs.length();
      }
      else
      { bOk = false; 
        console.writeError("input dir not found:" + sDir);
      }
    }
    else
    { fileIn.posPath = -1;
    }
    if(bOk)
    { try{ FileSystem.addFileToList(dir, sMask, fileIn.listFileIn); }
      catch(FileNotFoundException exc)
      { bOk = false;
        if(dir!=null)
        { console.writeError("input files not found:" + sMask + " in dir:" + dir.getAbsolutePath());
        }
        else
        { console.writeError("input files not found:" + sMask);
        }
      }
      listFileIn.add(fileIn);
    }
    return bOk;
  }

  
  
  /**The Constructor is called after parsing CmdLine arguments. */
  Header2Reflection(Report report)
  { this.console = report;
    parser = new ZbnfParser(console, 10);
  }
  
  
  
  public void execute() throws Exception
  { if(readBlockedFilesAndTypes())
    //if(readReflectionTypes())
    { if(init()) //here all action are done, testversion. 
      { 
      
        if(sFileAllC != null)
        { fileAllC = new BufferedWriter(new FileWriter(new File(sFileAllC)));   
          int posDot = sFileAllC.lastIndexOf('.');
          int posPath = sFileAllC.lastIndexOf('/');
          String sPathFileAllH = sFileAllC.substring(0, posDot) + ".h";
          String sFileAllH = sFileAllC.substring(posPath+1, posDot) + ".h";
          String sFileName = sFileAllC.substring(posPath+1, posDot);
          fileAllH = new BufferedWriter(new FileWriter(new File(sPathFileAllH)));
          fileAllC.write("\n/*This file is generated from Header2Reflection. */");
          fileAllC.write("\n#include \"" +sFileAllH+ "\"");
          fileAllH.write("\n#ifndef __" + sFileName + "_h__\n");
          fileAllH.write("\n#define __" + sFileName + "_h__\n");
          fileAllH.write("\n#include \"Reflection_Jc.h\"");
        }
        for(FileIn fileIn: listFileIn)
        { for(File file: fileIn.listFileIn)
          { String fileName = file.getAbsolutePath();
            fileName = fileName.substring(fileIn.posPath+1);
            translate(file, fileName);
          }
        }
        if(fileAllC != null)
        { fileAllC.close();
          fileAllH.write("\n#endif // ___h__\n");
          fileAllH.close();
        }  
        console.writeInfoln("...finished.");
        
      }
      //if(sFileReflectionTypesOut!= null)  writeReflectionTypes();
    }
    else
    { console.writeError("read Reflection types failed.");
    }
  
  }
  

  
  /**Reads all types from the file given with -r: argument in cmdline or setted with setReflectionTypes(filepath);
   * 
   * @return true if success.
   * @throws ParseException
   * @throws IllegalCharsetNameException
   * @throws UnsupportedCharsetException
   * @throws FileNotFoundException
   * @throws IOException
   */
  boolean readBlockedFilesAndTypes() 
  throws ParseException, IllegalCharsetNameException, UnsupportedCharsetException, FileNotFoundException, IOException
  { ZbnfParser parser = new ZbnfParser(console, 10);
    parser.setSyntax(sSyntaxReflectionTypes);
    StringPart spInput = null;
    spInput = new StringPartFromFileLines(new File(sFileReflectionBlockedTypes),-1, null, null);
    boolean bOk = parser.parse(spInput);
    if(!bOk)
    { console.writeError(parser.getSyntaxErrorReport());
      stop();      
    }
    else
    { List<ZbnfParseResultItem> items = parser.getFirstParseResult().listChildren("reflectionType");
      //String sType;
      for(ZbnfParseResultItem item: items)
      { char cSign = item.getChild("sign").getParsedString().charAt(0);
        String sPath = item.getChild("path").getParsedString();
        TypeEntry blockedFileOrType = new TypeEntry(sPath, sPath, cSign); 
        blockedFilesAndTypes.put(sPath, blockedFileOrType);
      }
    }
    return bOk;
  }  
  
  
  
  
  
  /**Reads all types from the file given with -r: argument in cmdline or setted with setReflectionTypes(filepath);
   * 
   * @return true if success.
   * @throws ParseException
   * @throws IllegalCharsetNameException
   * @throws UnsupportedCharsetException
   * @throws FileNotFoundException
   * @throws IOException
   */
  boolean readReflectionTypes() 
  throws ParseException, IllegalCharsetNameException, UnsupportedCharsetException, FileNotFoundException, IOException
  { ZbnfParser parser = new ZbnfParser(console, 10);
    parser.setSyntax(sSyntaxReflectionTypes);
    StringPart spInput = null;
    spInput = new StringPartFromFileLines(new File(sFileReflectionTypes),-1, null, null);
    boolean bOk = parser.parse(spInput);
    if(!bOk)
    { console.writeError(parser.getSyntaxErrorReport());
      stop();      
    }
    else
    { List<ZbnfParseResultItem> items = parser.getFirstParseResult().listChildren("reflectionType");
      String sType;
      BufferedWriter wrTypes = null;
      if(bNewFileReflectionTypesOut)
      { wrTypes = new BufferedWriter(new FileWriter(new File(sFileReflectionTypesOut)));
        wrTypes.write("//********Generated Reflectiontypes*****************************************\n");
      }
      for(ZbnfParseResultItem item: items)
      { char cSign = item.getChild("sign").getParsedString().charAt(0);
        String sPath = item.getChild("path").getParsedString();
        int posSlash = sPath.lastIndexOf("/");  //-1 is admissible
        String sFileName = sPath.substring(posSlash+1);
        LinkedList<TypeEntry> typesPerFile = new LinkedList<TypeEntry>(); 
        typesByFilename.put(sFileName, typesPerFile);
        //if("+!$%".indexOf(cSign) >0)
        if("#.".indexOf(cSign) < 0)
        { //The filename is the same as a type.
          int posDot = sPath.lastIndexOf(".");
          if(posDot < posSlash){ posDot = sPath.length(); }
          sType = sPath.substring(posSlash+1, posDot);
          TypeEntry type = new TypeEntry(sPath, sType, cSign);
          types.put(sType, type);
          typesPerFile.add(type);
        }
        if(cSign == '#')
        { TypeEntry notUsedFile = new TypeEntry(sPath, "###", '#');
          typesPerFile.add(notUsedFile);
        }
        
        List<ZbnfParseResultItem> typeItems = item.listChildren("type");
        if(typeItems!=null)for(ZbnfParseResultItem typeItem: typeItems)
        { sType = typeItem.getChild("name").getParsedString();
          cSign = typeItem.getChild("sign").getParsedString().charAt(0);
          TypeEntry type = new TypeEntry(sPath , sType, cSign);
          types.put(sType, type);
          typesPerFile.add(type);
        }
        if(cSign == '#' && wrTypes != null)
        {
          wrTypes.write("" + cSign + " " + sPath + "\n");  //note: a type is not relevant.
        }
      }
      if(wrTypes != null)
      { wrTypes.flush();
        wrTypes.close();
      }
    }
    return bOk;
  }  
  
  
  
  
  
  void writeReflectionTypes() 
  throws IOException
  { BufferedWriter wrTypes = new BufferedWriter(new FileWriter(new File(sFileReflectionTypesOut), true));
    wrTypes.append("//***********************************************************************************\n");
    wrTypes.append("//from " + sFileAllC + "\n");
    
    
    Set<Map.Entry<String, List<TypeEntry>>> fileSet = typesByFilename.entrySet();
    for(Map.Entry<String, List<TypeEntry>> entry: fileSet)
    { List<TypeEntry> types = entry.getValue();
      if(types.size()>0 && types.get(0).sign != '#')
      { String line = ". " + entry.getKey();
        for(TypeEntry typeEntry: types)
        {
          if(typeEntry.sign != '#')
          { line += ", " + typeEntry.sign + " " + typeEntry.sType;
          }  
        }
        wrTypes.append(line + "\n");
      }      
    }
    
    /*
    Set<Map.Entry<String, TypeEntry>> typeSet = types.entrySet();
    for(Map.Entry<String, TypeEntry> entry: typeSet)
    { TypeEntry typeEntry = entry.getValue();
      if(typeEntry.sign != '#')
      { String line = ". " + typeEntry.sPath + ", " + typeEntry.sign + " " + typeEntry.sType;
        wrTypes.append(line + "\n");
      }  
    }
    */
    wrTypes.close();
  
  }
  
  
  
  
  
  
  boolean init() 
  throws IllegalCharsetNameException, UnsupportedCharsetException, FileNotFoundException, IOException
  { boolean bOk= true;
    File fileSyntax= new File(sFileZbnf);
    String sSyntax = FileSystem.readFile(fileSyntax);
    parser = new ZbnfParser(console, 10);
    try{ parser.setSyntax(sSyntax); }
    catch(ParseException exc)
    { console.writeError("Headersyntax fault in " + sFileZbnf, exc);
      parser.reportSyntax(console);
      bOk = false;
    }
    return bOk;  
  }
  
  
  
  
  void translate(File headerFile, String sFileNameRelative) 
  throws IllegalCharsetNameException, UnsupportedCharsetException, FileNotFoundException, IOException, ParseException
  {  
    console.writeInfoln("generate from " + headerFile.getName()+"...");
    String sFileName = headerFile.getName();
    /*
    String sTypeFromFileName;
    { int posDot = sFileName.indexOf('.');
      if(posDot < 0){ posDot = sFileName.length(); }
      sTypeFromFileName = sFileName.substring(0, posDot); 
    }  
    List<TypeEntry> typesInFile = typesByFilename.get(sFileNameRelative);
    if(typesInFile == null)
    { //The file is unknown yet, it should be parsed:
      typesInFile = new LinkedList<TypeEntry>();
      typesByFilename.put(sFileNameRelative, typesInFile);
    }
    
    if(typesInFile.size() == 0 || typesInFile.get(0).sign != '#')  //if there are no types stored, a # is written at first, do not translate it.
    { 
    */
    TypeEntry typeEntry = blockedFilesAndTypes.get(sFileNameRelative);
    if(typeEntry == null || typeEntry.sign !='#')
    { StringPart spInput = null;
      spInput = new StringPartFromFileLines(headerFile,-1, null, null);
      boolean bOk = parser.parse(spInput);
      if(!bOk)
      { String sError = parser.getFoundedInputOnError();
        //console.writeError(sError);
        sError = parser.getExpectedSyntaxOnError();
        //console.writeError(sError);
        sError = parser.getSyntaxErrorReport();
        console.writeError(sError);
        stop();      
        
      }
      else
      { parser.reportStore(console, Report.fineInfo);
        if(sOutDir != null)
        { fileC = new BufferedWriter(new FileWriter(new File(sOutDir + "/Reflection_" + sFileName + ".c")));
          fileH = new BufferedWriter(new FileWriter(new File(sOutDir + "/Reflection_" + sFileName + ".h")));
        }
        else
        { fileC = fileAllC;
          fileH = null;
        }
        ZbnfParseResultItem resultItem = parser.getFirstParseResult();
        if(resultItem.getSemantic().equals("Cheader"))
        { Iterator<ZbnfParseResultItem> iter= resultItem.iteratorChildren();
        //resultItem = resultItem.nextSkipIntoComponent(null);
          if(fileH != null)
          { fileH.write("\n#ifndef __" + sFileName + "_h__\n");
            fileH.write("\n#define __" + sFileName + "_h__\n");
          }
          fileC.write("//this file is generated by Header2Reflection");
          fileC.write("\n#include \"Reflection_Jc.h\"");
          fileC.write("\n#define protected public  //only active inside this compilation unit");
          fileC.write("\n#define private public    //  to enable the access to all elements.");
          fileC.write("\n#include \"" + sFileNameRelative + "\"\n");
          if(fileH != null)
          { fileH.flush();
          }
          fileC.flush();
          
          //while(resultItem != null)
          while(iter.hasNext())
          { resultItem = iter.next();
            String sSemantic = resultItem.getSemantic();
            if(sSemantic.equals("outside") || sSemantic.equals("CLASS_C"))
            { Iterator<ZbnfParseResultItem> iter1= resultItem.iteratorChildren();
              String sCLASS_C_name = null;
              if(sSemantic.equals("CLASS_C"))
              { sCLASS_C_name = resultItem.getChild("@name").getParsedString();
              }
              while(iter1.hasNext())
              { resultItem = iter1.next();
                sSemantic = resultItem.getSemantic();
                if(sSemantic.equals("classDef") || sSemantic.equals("structDefinition"))
                { //String sClassName = 
                  ZbnfParseResultItem itemName = resultItem.getChild("@name");
                  if(itemName != null)
                  { String sClassName = itemName.getParsedString();
                    if(  sCLASS_C_name == null             //no wrapper CLASS_C exists
                      || sCLASS_C_name.equals(sClassName)  //or it is the struct associated with CLASS_C 
                      )
                    { ConverterClass converterClass = new ConverterClass();
                      converterClass.convertClass(sClassName, null, null, resultItem, sFileNameRelative); //, typesInFile);  //this is a class or struct.      
                    }
                  }
                  else
                    stop();
                }
              }  
        
              
            }
            //resultItem = resultItem.next(null);
          }  
          if(fileH != null)
          { fileH.write("#endif //__" + sFileName + "_h__\n");
            fileH.flush();
          }
          fileC.flush();
        }
      }    
    }
    else
    { console.writeInfo(" ignored.");
    }
      
  }

  
  
  class ConverterClass
  { 
    /**Inheritance situation. */
    boolean bObjectJcpp = false ;
    /**Inheritance situation. */
    boolean bObjectJc = false ;
    /**Inheritance situation. */
    boolean bObjectifcBaseJcpp = false ; 
      
    final TreeMap<String, String> innerTypes =new TreeMap<String, String>();
      
    /**This mask is defined originally in ReflectionJc.h, TODO import org.vishia.byteDataAccess.ReflectionJc
     */
    static final int mObject_Jc_Modifier_reflect_Jc = 0x04000000;

    /**This mask is defined originally in ReflectionJc.h, TODO import org.vishia.byteDataAccess.ReflectionJc
     */
    static final int mObjectifcBase_Jcpp_Modifier_reflect_Jc = 0x08000000;
    
    private void convertClass
    ( String sClassName
    , String sParentCppClassName
    , String sParentReflectionClassName
    , ZbnfParseResultItem classItem
    , String sFilePath
    //, List<TypeEntry> typesInFile
    ) 
    throws IOException
    { //String sClassName = classItem.getChild("@name").getParsedString();
      String sReflectionClassName = sParentReflectionClassName != null ? sParentReflectionClassName + "_" + sClassName : sClassName;
      String sCppClassName = sParentCppClassName != null ? sParentCppClassName + "::" + sClassName : sClassName; 
      
      if(sClassName.startsWith("TestSomething"))
        stop();
      
      /*
      TypeEntry type = types.get(sReflectionClassName);
      if(type == null)
      { //not known file:
        type = new TypeEntry(sFilePath,  sReflectionClassName, '!');
        types.put( sReflectionClassName, type);
        typesInFile.add(type);
      }
      */
      
      int bitModifiers = 0;
      
      console.writeInfo(sClassName + "...");
      
      if(fileAllH != null)
      { fileAllH.write("\nextern const Class_Jc reflection_" + sReflectionClassName + ";");
        fileAllH.write("\n#define REFLECTION_" + sReflectionClassName + " &reflection_" + sReflectionClassName);
      }
      List<ZbnfParseResultItem> visibilityBlocks = classItem.listChildren("classVisibilityBlock");
      
      if(visibilityBlocks!=null)for(ZbnfParseResultItem visibilityBlockItem : visibilityBlocks)
      { convertInnerClassAndStruct(sCppClassName, sReflectionClassName, visibilityBlockItem, sFilePath);
      }
      convertInnerClassAndStruct(sCppClassName, sReflectionClassName, classItem, sFilePath);
      
      StringBuffer sbCfile = new StringBuffer(50000);
      StringBuffer sbForward = new StringBuffer(5000);
      String sSeparator = "\n    ";
      List<ZbnfParseResultItem> superclasses = classItem.listChildren("superclass");
      //determine the heredity from Object_Jc etc. note: only 1 level of heredity is supported here,
      //because the other super-classes are unknown yet.
      if(superclasses!=null)for(ZbnfParseResultItem superclassItem : superclasses)
      { String sNameSuperclass = superclassItem.getChild("name").getParsedString();
        if(sNameSuperclass.equals("Object_Jcpp"))
        { bObjectJcpp = true;
          //type.sign = '$';
        }
        else if(sNameSuperclass.equals("Object_Jc"))
        { bObjectJc = true; 
          //type.sign = '$';
        }
        else if(sNameSuperclass.equals("ObjectifcBase_Jcpp"))
        { bObjectifcBaseJcpp = true; 
          //type.sign = '%';
        }
      }

      int nrofAttributes = 0;
      List<ZbnfParseResultItem> attributes = classItem.listChildren("attribute");  //outside visibility block
      List<ZbnfParseResultItem> listInnerStructAttributes = classItem.listChildren("implicitStructAttribute");  //outside visibility block
      List<ZbnfParseResultItem> listAllAttributes = new LinkedList();
      if(attributes != null){ listAllAttributes.addAll(attributes); }
      if(listInnerStructAttributes != null)
      { listAllAttributes.addAll(listInnerStructAttributes); }
      for(ZbnfParseResultItem attributeItem : listAllAttributes)
      { if(convertAttribute(sSeparator, sCppClassName, sReflectionClassName, attributeItem, sbCfile, sbForward))
        { nrofAttributes +=1;
          sSeparator = "\n  , ";  //next entries.
        }  
      }
      
      if(visibilityBlocks!=null)for(ZbnfParseResultItem visibilityBlockItem : visibilityBlocks)
      {
        attributes = visibilityBlockItem.listChildren("attribute");
        if(attributes!=null)for(ZbnfParseResultItem attributeItem : attributes)
        { if(convertAttribute(sSeparator, sCppClassName, sReflectionClassName, attributeItem, sbCfile, sbForward))
          { nrofAttributes +=1;
            sSeparator = "\n  , ";  //next entries.
          }  
        }
      }
      if(nrofAttributes >0)
      { fileC.append(sbForward); sbForward.setLength(0);
        //fileC.append("\nextern const Class_Jc reflection_" + sReflectionClassName + ";\n");
        fileC.append("\nconst struct Reflection_Fields_" + sReflectionClassName + "_t");
        fileC.append("\n{ ObjectArray_Jc head;");
        fileC.append("\n  Field_Jc data[" + nrofAttributes + "];");
        fileC.append("\n} reflection_Fields_" + sReflectionClassName + " =");
        fileC.append("\n{ CONST_ObjectArray_Jc(Field_Jc, " + nrofAttributes+ ", OBJTYPE_Field_Jc, null, &reflection_Fields_" + sReflectionClassName + ")");
        fileC.append("\n, {");
        fileC.append(sbCfile); sbCfile.setLength(0);
        fileC.append("\n} };\n\n");
      }
      
      
      
      sbCfile.append(  "\nconst Class_Jc reflection_" + sReflectionClassName + " =");
      sbCfile.append(  "\n{ CONST_Object_Jc(OBJTYPE_Class_Jc + sizeof(Class_Jc), &reflection_" + sReflectionClassName + ", null)");
      sbCfile.append(  "\n, \"" + (sCppClassName.length() > 28 ? sCppClassName.substring(0,28)  : sCppClassName) + "\"");
      if(bObjectJcpp)
      { sbCfile.append("\n, (int)(static_cast<Object_Jc*>(static_cast<Object_Jcpp*>((" + sCppClassName + "*)0x1000))) - 0x1000     //posObject_Jc");
        bitModifiers |= mObject_Jc_Modifier_reflect_Jc;
        //NOTE: on evaluating the generated Reflections in C, the Object_Jc is getted with add offset and C-cast to Object_Jc.
      }
      else if(bObjectJc)
      { sbCfile.append("\n, (int)(static_cast<Object_Jc*>((" + sCppClassName + "*)0x1000)) - 0x1000     //posObject_Jc");
        bitModifiers |= mObject_Jc_Modifier_reflect_Jc;
        //NOTE: on evaluating the generated Reflections in C, the Object_Jc is getted with add offset and C-cast to Object_Jc.
      }
      else if(bObjectifcBaseJcpp)
      { sbCfile.append("\n, (int)(static_cast<ObjectifcBase_Jcpp*>((" + sCppClassName + "*)0x1000)) - 0x1000     //posObject_Jc");
        bitModifiers |= mObjectifcBase_Jcpp_Modifier_reflect_Jc;
        //NOTE: on evaluating the generated Reflections in C, the Object_Jc is getted with toObject_Jc.
      }
      else
      { sbCfile.append("\n, 0");
        bitModifiers |= 0;  //set no additional bit.
      }
      sbCfile.append(  "\n, sizeof(" + sCppClassName + ")");
      if(nrofAttributes >0)
      { sbCfile.append("\n, (Field_JcArray const*)&reflection_Fields_" + sReflectionClassName + "  //attributes and associations");
      }
      else
      { sbCfile.append("\n, null  //attributes");
      }
      sbCfile.append(  "\n, null  //method");
      sbCfile.append(  "\n, null  //superclass");
      sbCfile.append(  "\n, null  //interfaces");
      sbCfile.append(  "\n, 0x" + Integer.toHexString(bitModifiers) + "  //modifiers");
      sbCfile.append(  "\n};\n\n\n");
        
      fileC.append(sbCfile.toString());
    }
    
    
    
    
    /**called inside convertClass to detect inner classes. */
    void convertInnerClassAndStruct
    (String sParentCppClassName
    , String sParentReflectionClassName
    , ZbnfParseResultItem zbnfParent
    , String sFilePath
    ) 
    throws IOException
    { List<ZbnfParseResultItem> innerClasses = zbnfParent.listChildren("classDef");
      if(innerClasses!=null)for(ZbnfParseResultItem innerClassItem : innerClasses)
      { String sTypeName = innerClassItem.getChild("@name").getParsedString();
        innerTypes.put(sTypeName, "");
        ConverterClass converterClass = new ConverterClass();
        converterClass.convertClass(sTypeName, sParentCppClassName, sParentReflectionClassName, innerClassItem, sFilePath);//, typesInFile);
      }
      List<ZbnfParseResultItem> zbnfImplicitStruct = zbnfParent.listChildren("implicitStructAttribute");
      if(zbnfImplicitStruct!=null)for(ZbnfParseResultItem zbnfImplicitStructItem : zbnfImplicitStruct)
      { ZbnfParseResultItem zbnfTypeName = zbnfImplicitStructItem.getChild("@tagname");
        if(zbnfTypeName == null)
        { //TODO  problem in convertClass 
          zbnfTypeName = zbnfImplicitStructItem.getChild("@name");
        }
        String sTypeName;
        if(zbnfTypeName != null)
        { sTypeName = zbnfTypeName.getParsedString();
          if(!sParentCppClassName.startsWith("struct "))
          { sParentCppClassName = "struct " + sParentCppClassName;
          }
          innerTypes.put(sTypeName, "");
          ConverterClass converterClass = new ConverterClass();
          converterClass.convertClass(sTypeName, sParentCppClassName, sParentReflectionClassName, zbnfImplicitStructItem, sFilePath);//, typesInFile);
        }
        else
          stop(); //TODO test.
      }
    }    
    
    
    
    
    
    /**converts a attribute from parsed header.
     * @param sSeparator initialy "    " but than "  , " to implement the separator between or the introduce to the first attribute block.
     * @param sCppClassName Name of the class in c++-language, a.e. MyClass::InnerClass
     * @param sReflectionClassName Name of the class for Reflection identifier, a.e. MyClass_InnerClass
     * @param attributeItem The parse result item describing the attribute.
     * @param sbCfile buffer for C-file-content.
     * @param sbForward buffer for forward declarations
     * @throws IOException
     */
    private boolean convertAttribute(String sSeparator, String sCppClassName, String sReflectionClassName, ZbnfParseResultItem attributeItem, StringBuffer sbCfile, StringBuffer sbForward) throws IOException
    { ZbnfParseResultItem itemName = attributeItem.getChild("@name");
      String sAttributeName = itemName.getParsedString();
      
      ZbnfParseResultItem typeItem = attributeItem.getChild("type"); 
      String sType;
      boolean bReference;
      if(typeItem == null) 
      { if(attributeItem.getChild("@implicitStruct")!=null)
        { /*an implicit struct like ... struct Tag_t{ type element,...} name has a tag-name perhaps, but not always,
            therefore the name of the attribute is used as type-name. 
            The same rule is taken by detect types from inner implicit struct
          */
          ZbnfParseResultItem zbnfTagName = attributeItem.getChild("@tagname");
          if(zbnfTagName != null)
          { sType = zbnfTagName.getParsedString();
          }
          else
          { sType = sAttributeName;
          }
          //sType = sReflectionClassName + "_" + sAttributeName;
          bReference = false;  //a implicit struct can not be used outside, because it is unknown.
          //struct{ type element} *name; is syntactical correct in C, but it is not useable. Do not consider here.
        }
        else
        { stop();  //TODO
          return false;
        }  
      }
      else
      { ZbnfParseResultItem itemTypeName = typeItem.getChild("@name");
        if(itemTypeName == null) 
        { stop();
          return false;
        }
        sType = itemTypeName.getParsedString();
        bReference = typeItem.getChild("@pointer") != null;
        boolean bForwardStruct = typeItem.getChild("@forward") != null;
        if(bForwardStruct && sType.endsWith("_t"))
        { //struct Type_t may be mostly of type Type
          sType = sType.substring(0,sType.length()-2);
        }
      }  
      { int posScalarType = scalarTypes.indexOf("-"+sType+":");
        if(sAttributeName.equals("VehHndl"))
          stop();
        
        String sInnerType = innerTypes.get(sType);
        if(sInnerType != null)
        { //the type is a type of the ambient class. Therefore the type should named with this information.
          sType = sReflectionClassName + "_" + sType;
        }
        
        int bytesScalarType;
        
        boolean useReflection;
        
        if(posScalarType > 0)
        { bytesScalarType = scalarTypes.charAt(posScalarType-1)-'0';
          useReflection = true;
        }
        else
        { bytesScalarType = -1;
          TypeEntry typeSign = types.get(sType);
          /*
          if(typeSign == null)
          { //type is unknown
            useReflection = false;
          }
          else
          { useReflection = "!$%".indexOf(typeSign.sign)>0;
          }
          */
          TypeEntry typeEntry = blockedFilesAndTypes.get(sType);
          if(typeEntry != null && typeEntry.sign == '#')
          { useReflection = false;
          }
          else
          { useReflection = true;
          }
          if(useReflection)
          { sbForward.append("\nextern const Class_Jc reflection_" + sType + ";");
          }
        }
        if(bReference)
        { //test wether a Reflection is found.
        }
        sbCfile.append(sSeparator +"{ \"" + (sAttributeName.length() > 28 ? sAttributeName.substring(0,28)  : sAttributeName) + "\"");
        sbCfile.append(  "\n    , 0");
        if(bytesScalarType >=0)
        { sbCfile.append("\n    , REFLECTION_" + sType);
          sbCfile.append("\n    , ("+ bytesScalarType+"<<kBitPrimitiv_Modifier_reflect_Jc)   //Modifier"); 
        }
        else
        { 
          if(useReflection)
          { sbCfile.append("\n    , &reflection_" + sType);
          }
          else
          { sbCfile.append("\n    , REFLECTION_void");
          }
          sbCfile.append("\n    , 0");
          if(bReference){ sbCfile.append("| mReference_Modifier_reflect_Jc"); }
          sbCfile.append(" //Modifier"); 
        }
        sbCfile.append(  "\n    , (int16)((int32)(&((" + sCppClassName + "*)(0x1000))->" + sAttributeName + ") ");
        if(bObjectJcpp)
        { sbCfile.append(          "- (int32)(static_cast<Object_Jc*>(static_cast<Object_Jcpp*>((" + sCppClassName+ "*)0x1000))))");
        }
        else if(bObjectJc)
        { sbCfile.append(          "- (int32)(static_cast<Object_Jc*>((" + sCppClassName+ "*)0x1000)))");
        }
        else
        { sbCfile.append(          "- (int32)(" + sCppClassName+ "*)0x1000)");
        }
        sbCfile.append(  "\n    , 0  //offsetToObjectifcBase");
        sbCfile.append(  "\n    , &reflection_" + sReflectionClassName);
        sbCfile.append(  "\n    }");
        return true;
      }
    }
  }  
  
  void stop()
  { //debug
  }
  
}