/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 2008-04-06  (year-month-day)
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import org.vishia.zbnf.ZbnfParseResultItem;

class GenerateClass
{
  /**String to generate newline and indent. */
  final private String sNewLine = "\n                                                                              ";

  //ZbnfParseResultItem resultItemClass;
  /**A writer to file. */
  protected iWriteContent writeContent;

  /**A TreeMap of all classData sorted by classname to find all other classData.
   * This list is complete if the second path runs.
   */
  protected final TreeMap<String, ClassData> allClassData;

  /**The data of this class for first and second path. A nested class will be recognized as a normal class. */
  protected ClassData classData;

  /**The interface to call a firstpass run of a needed type. */
  final RunRequiredFirstPass_ifc runRequiredFirstPass;


  GenerateClass(iWriteContent writeContent, TreeMap<String, ClassData> allClassData, RunRequiredFirstPass_ifc runRequiredFirstPass)
  { //this.sClassName = sClassName;
    //this.resultItemClass = resultItem;
    this.writeContent = writeContent;
    this.allClassData = allClassData;
    this.runRequiredFirstPass = runRequiredFirstPass;
  }

  /**TODO this method may be assigned to classData or the main type identifiers should be set outside. */
  protected void setInitialTypeIdentifiers()
  { classData.typeIdentifiers.put("int", "%int32");
    classData.typeIdentifiers.put("short", "%int16");
    classData.typeIdentifiers.put("long", "%int64");
    classData.typeIdentifiers.put("float", "%float");
    classData.typeIdentifiers.put("double", "%double");
    classData.typeIdentifiers.put("char", "%char");
    classData.typeIdentifiers.put("byte", "%int8");
    classData.typeIdentifiers.put("LinkedList", "@LinkedListJc");
  }




  /**
   *
   * @param item description::= test_description::=
   * @throws IOException
   */
  protected void write_Description(ZbnfParseResultItem item) throws IOException
  {
    Iterator<ZbnfParseResultItem> iterDescr = item.iteratorChildren();
    if(iterDescr != null)
    { while(iterDescr.hasNext())
      { ZbnfParseResultItem resultItemDesription = iterDescr.next();
        String sSemanticDescription = resultItemDesription.getSemantic();
        if(sSemanticDescription.equals("text"))
        { String sText = resultItemDesription.getParsedString();
          writeContent.writeClassH("\n/**" + sText + "*/\n");
        }
    } }
  }


  /**writes a variable definition with syntax attributeDefinition::=.
   *
   * @param itemVariableDefinition ##a
   * @throws IOException
   * @throws ParseException
   */
  /**
   * @param itemVariableDefinition
   * @param idents
   * @param variablesToInit
   * @param intension intension of calling a=argument list c=class level s=static class level b=inside statement block
   * @return
   * @throws ParseException
   */
  protected String gen_variableDefinition
  ( ZbnfParseResultItem itemVariableDefinition
  , LocalIdents idents
  , List<ClassData.VariablesToInit> variablesToInit
  , char intension
  )
  throws ParseException
  {
    String ret = "";
    boolean bReference = true;
    boolean bEmbeddedStruct = false;
    /**true if the reference is final. */
    boolean bFinalReference = itemVariableDefinition.getChild("final")!= null;
    boolean bStaticField = itemVariableDefinition.getChild("static")!= null;
    List<ZbnfParseResultItem> listModifier = itemVariableDefinition.listChildren("Modifier");
    String sName = itemVariableDefinition.getChild("name").getParsedString();
    if(sName.equals("theInstance"))
      stop();
    ZbnfParseResultItem itemType = itemVariableDefinition.getChild("typeIdent");
    if(itemType == null)
      stop();
    String sType = gen_EnvIdent(itemType, "__");

    /**Check if it is an new Type(param), it may be an embedded struct. */
    if(sType.contains("RefValueGen__Target"))
      stop();
    ZbnfParseResultItem itemNewObject = itemVariableDefinition.getChild("newObject");  //it may be or may null.
    ZbnfParseResultItem assignment = itemVariableDefinition.getChild("value");  //it may be nul
    if(intension=='c')
    { //only at class level, test if it should be an embedded struct in C
      if(itemNewObject != null)
      { String sTypeNewObject = gen_EnvIdent(itemNewObject.getChild("className"), "__");  //itemNewObject.getChild("className").getParsedString();
        if(sTypeNewObject.equals(sType))
        { //it is a form Type final name = new Type(...), implement an embedded struct in C.
          bEmbeddedStruct = true;
          bReference = false;
          bFinalReference = false;
          writeContent.addIncludeH(sType);  //not a reference
        }
      }
    }  
    /* check wether the type is known:
     * scalar types int,short...
     * ...
    */
    String sTypeTranslate = classData.typeIdentifiers.get(sType);
    String sTypeResult;
    char cTypeKind = sTypeTranslate != null ? sTypeTranslate.charAt(0) : 0;     //the kind of the type:
    switch(cTypeKind)
    { case '%': sTypeResult = sTypeTranslate.substring(1); bReference = false; break;   //basic type
      case '@': sTypeResult = sTypeTranslate.substring(1) + "*"; break;  //translated reference type
      case '&': sTypeResult = sType; break; //interface type
      case '!': sTypeResult = sType; break; //final type
      case '$': sTypeResult = sTypeTranslate.substring(1) ; bReference = false; break;  //translated embedded type
      case 0:   sTypeResult = sType; break;
      default: throw new ParseException("unknown typeTranslateKind" + sTypeTranslate,0);
    }

    ClassData classDataType = allClassData.get(sType);
    if(classDataType == null && cTypeKind != '%')
    { classDataType = runRequiredFirstPass.runRequestedFirstPass(sType);  //may be null if it is an external type.
    }
    //NOTE: all unknown types are final.
    boolean bFinalType = classDataType == null || classDataType.isFinal;

    String sModifier = "";

    
    List<ZbnfParseResultItem> listZbnfAssignments;
    String sConstantValue;
    if(  intension == 's'   //static variable
      && bFinalType         //final static TYPE name = assignment
      && assignment!= null
      && (listZbnfAssignments = assignment.listChildren()).size()==1   //a simple constant assignemnt
      && (sConstantValue = genConstantValue(listZbnfAssignments.get(0))) != null
      )
    { //an assignment with one static constant value: implement in C as #define name value.
      //a define is used in C as immediate constant, useable at example in case val:
      //but a const TYPE name = val is not immediate, but indirect.
      //therefore the #define is a important language element in C.
      ret += "#define " + sName  + "_" + classData.sName + " " + sConstantValue;
      sModifier = "d";
      assignment = null;  //donot create entry for variablesToInit
    }
    else 
    { if(intension == 's')   //static variable, it is in headerFile outside a frame
      { ret += "extern ";
        if(bFinalType)
        { ret += "const ";
          sModifier = "S"; //static const element
        }
        else
        {
          sModifier = "s"; //static element
        }
      }
      if(bEmbeddedStruct)
      {
        ret += sTypeResult + " " + sName;
        sModifier += "$";
      }
      else if(bReference)   //not an primitive type
      { //forward declared reference or enhanced reference?
        if(  intension == 'a'        //forward declaration always in argument lists
          || intension == 'b'        //forward declaration always in stack variable of block statement
          || classDataType == null   //to a unknown, not in Java for C, converted type
          || (  bFinalType                      //the type is final, no index of jump table is needed
             && bFinalReference                 //and the reference variable is final
             && classData.isStaticInstanciated  //either isStaticInstanciated and final reference
             )
            
          )
        { //forward declared reference type, no garbage collector is needed, no index of jump table is needed.
          ret += "struct " + sTypeResult + "_t* " + sName;
          sModifier += "*";
        }
        else    
        { //enhanced reference
          if(classDataType != null)
          { classDataType.referenceNeeded();
          }
          ret += sTypeResult + "REF " + sName;
          sModifier += "@";
        }  
      }
      else
      { //primitive type:
        ret += sTypeResult + " " + sName;
        sModifier += "%"; //% for a basic scalar type
      }

      if(bStaticField)
      { ret += "_" + classData.sName;
      }
      if(intension != 'a')
      { ret += ";";
      }
    
    }    

    if(  intension == 's'          //a static declaration in h file needs a definition in c-File.
      && sModifier.indexOf('d')<0  //if a #define is generated, no static definition is need
      )
    { ClassData.VariablesToInit staticVariableInCfile 
      = new ClassData.VariablesToInit
      ( sModifier.charAt(1)  //charAt(0) is 's' or 'S'
      , sType
      , sName
      , assignment  //may be null
      );
      //TODO what is with bFinal? const?
      classData.staticVariables.add(staticVariableInCfile);
    }
    else
    { if(itemNewObject!=null)
      { //char kind = bEmbeddedStruct ? '$' : '@';
        variablesToInit.add(new ClassData.VariablesToInit(sModifier.charAt(0), sType, sName,itemNewObject));
      }
      else
      { if(assignment != null)
        { variablesToInit.add(new ClassData.VariablesToInit(sModifier.charAt(0), sType, sName,assignment));
        }
      }
    }    
    if(idents != null)
    { if("cs".indexOf(intension)>=0)
      { //at class level, class variable or static variable of the class.
        idents.putClassElement(sName, sType, classDataType, sModifier, classData);
      }
      else
      { idents.putLocalElement(sName, sType, classDataType, sModifier);
      }
    }
    String sDescription = get_description(itemVariableDefinition);
    if(sDescription != null)
    { ret += "  /*" + sDescription + "*/";
    }
    return ret;
  }


  
  
  
  
  /**gets the description from any item which contains < description>.
   * The description is shorted to one line, but only to ". ", the first sentence.
   * @param parent The item
   * @return Description guaranteed without newline.
   */
  String get_description(ZbnfParseResultItem parent)
  { String sDescription = null;
    ZbnfParseResultItem itemDescription = parent.getChild("description");
    if(itemDescription != null)
    { sDescription = itemDescription.getChild("text").getParsedString();
      int posNewline = sDescription.indexOf('\n');
      int posDot = sDescription.indexOf(". ");
      int posEnd = posDot > 0 && posDot < posNewline ? posDot : posNewline > 0 ? posNewline : -1;
      if(posEnd > 0){ sDescription = sDescription.substring(0, posEnd); }
    }
    return sDescription;
  }

  
  
  protected String genConstantValue(ZbnfParseResultItem item)
  {
    String simpleValue;
    String sSemantic = item.getSemantic();
    if(sSemantic.equals("simpleStringLiteral"))
    { simpleValue = "\"" + item.getParsedString() + "\"";
    }
    else if(sSemantic.equals("simpleCharLiteral"))
    { simpleValue = "\'" + item.getParsedString() + "\'";
    }
    else if(sSemantic.equals("hexNumber"))
    { simpleValue = "0x" + Long.toHexString(item.getParsedInteger());
    }
    else if(sSemantic.equals("hexNumberNegative"))
    { simpleValue = "-0x" + Long.toHexString(item.getParsedInteger());
    }
    else if(sSemantic.equals("intNumber"))
    { simpleValue = "" + item.getParsedInteger();
    }
    else if(sSemantic.equals("floatNumber"))
    { simpleValue = "" + item.getParsedFloat()+"F";
    }
    else if(sSemantic.equals("doubleNumber"))
    { simpleValue = "" + item.getParsedFloat();
    }
    else
    { simpleValue = null;
    }
    return simpleValue;
  }
  
  
  
  protected String genIndent(int indent)
  { return sNewLine.substring(0, 2*indent +1);
  }



  private String gen_xxvariableDefinition(ZbnfParseResultItem parent, int indent, TreeMap<String, String> idents) throws ParseException
  { String ret = genIndent(indent);
    ZbnfParseResultItem item;
    Iterator<ZbnfParseResultItem> iter = parent.iterChildren("Modifier");
    while(iter != null && iter.hasNext())
    { item = iter.next();
    }
    String sType = gen_EnvIdent(parent.getChild("typeIdent"), "__");
    ret += sType + " ";
    String sName = parent.getChild("attribute").getParsedString();
    ret += sName;
    idents.put(sName, sType);
    if( (item = parent.getChild("value"))!= null)
    {
    }

    { iter = parent.iterChildren("typeArray");
      while(iter != null && iter.hasNext())
      { item = iter.next();
      }
    }
    return ret + ";";
  }




  protected String gen_EnvIdent(ZbnfParseResultItem parent, String sSep)
  { String sRet = "";
    ZbnfParseResultItem itemEnv = parent;
    while( (itemEnv=parent.getChild("envIdent"))!= null)
    { sRet += parent.getChild("name").getParsedString() + sSep;
      parent = itemEnv;
    }
    sRet += parent.getChild("name").getParsedString();
    return sRet;
  }




  void stop()
  { //debug
  }

}
