/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 2008-04-06  (year-month-day)
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.File;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map;

import org.vishia.mainCmd.Report;
import org.vishia.util.FileSystem;


import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;

import org.vishia.util.StringPart;
import org.vishia.util.StringPartFromFileLines;
import org.vishia.zbnf.ZbnfParseResultItem;
import org.vishia.zbnf.ZbnfParser;


/**This class is the main class of translation Java2C. 
 * It is used from the command line call-implementation class {@link Java2C}.  
 * @author JcHartmut
 *
 */
public class Java2C_Main implements RunRequiredFirstPass_ifc
{

  /**Aggregation to the Console implementation class.*/
  Report console;

  /**Input files. */
  final List<File> listFileIn = new LinkedList<File>();

  /**The parser. */
  final ZbnfParser parser;

  /**Writer for a file containing all enhanced references. */
  BufferedWriter fileRef;

  /**List of known types. */
  final TreeMap<String, ClassData> allClassData = new TreeMap<String, ClassData>();
  //final AllData allClassData;

  /**List of all files processed in the first pass to consider in the second pass. */
  final List<GenerateFile> allFileParseResults = new LinkedList<GenerateFile>();

  private String sPathOut;

  public void setPathOut(String path)
  { path = path.replace("\\", "/");
    if(!path.endsWith("/")){ path+= "/"; }
    sPathOut = path;
  }


  private String sSyntaxPath;

  public void setSyntaxPath(String path)
  { sSyntaxPath = path;
  }



  boolean addInputFilemask(String sMask)
  { boolean bOk = true;
    try{ FileSystem.addFileToList(sMask, listFileIn); }
    catch(FileNotFoundException exc)
    { bOk = false;
    }
    return bOk;
  }


  /**The Constructor is called after parsing CmdLine arguments. */
  Java2C_Main(Report report)
  { this.console = report;
    parser = new ZbnfParser(console, 10);
    //allClassData = new AllData();
  }



  public void execute() throws Exception
  { if(init())
    { for(File file: listFileIn)
      { runFirstPass(file, fileRef);
      }
      for(GenerateFile genFile : allFileParseResults)
      { genFile.runSecondPass(allClassData);
      }
      writeFileRef();
      console.writeInfoln("...done.");
    }
  }



  private void writeFileRef()
  throws IOException
  {
    fileRef = new BufferedWriter(new FileWriter(sPathOut + "/allReferences.h"));
    fileRef.write("/* This file, generated by Java2C, contains all enhanced reference definitions.");
    fileRef.write("\n * It needs no further includes, because all user types are forward declared.");
    fileRef.write("\n * This file is included overall to know the requested enhanced reference types.");
    fileRef.write("\n * Enhanced reference types are used for all interfaces and dynamic instances.");
    fileRef.write("\n */");
    fileRef.write("\n#ifndef __allReferences_h__");
    fileRef.write("\n#define __allReferences_h__");
    Set<Map.Entry<String,ClassData>> listAllClassData = allClassData.entrySet();
    for(Map.Entry<String,ClassData> entry: listAllClassData)
    { ClassData classData = entry.getValue();
      if(classData.isReferenceNeeded())
      { String sName = classData.sName;
        fileRef.write("\ntypedef struct " + sName + "Ref_t { ObjectRefValuesJc refbase; struct " + sName + "_t* ref; } " + sName + "REF;");
      }
    }
    fileRef.write("\n#endif  //__allReferences_h__");
    fileRef.close();
    fileRef = null;
  }








  boolean init()
  throws ParseException, IllegalCharsetNameException, UnsupportedCharsetException, FileNotFoundException, IOException
  { boolean bOk = true;
    File fileSyntax = null;
    fileSyntax= new File(sSyntaxPath); //"../../srcJava/org/vishia/Java2C/Java2C.zbnf");
    String sSyntax;
    sSyntax = FileSystem.readFile(fileSyntax);
    if(sSyntax == null)
    { console.writeError("syntaxfile not found, error arg -syntax:" + sSyntaxPath);
      bOk = false;
    }
    else
    { parser.setSyntax(sSyntax);
      parser.reportSyntax(console);
    }
    return bOk;
  }



  /**parses und translates one java file. It is called from {@link execute()} for every file.
   * */
  void runFirstPass(File javaFile, BufferedWriter fileRef)
  throws IllegalCharsetNameException, UnsupportedCharsetException, FileNotFoundException, IOException, ParseException
  {
    String sFileName = javaFile.getName();
    int posDot = sFileName.indexOf('.');
    sFileName = sFileName.substring(0, posDot);
    StringPart spInput = null;
    if(allClassData.get(sFileName) == null)
    { spInput = new StringPartFromFileLines(javaFile,10000, null, null);
      boolean bOk = parser.parse(spInput);
      if(!bOk)
      { String sError = parser.getFoundedInputOnError();
        console.writeError(sError);
        sError = parser.getExpectedSyntaxOnError();
        console.writeError(sError);
        sError = parser.getSyntaxErrorReport();
        console.writeError(sError);
        stop();

      }
      else
      { parser.reportStore(console, Report.fineInfo);
        ZbnfParseResultItem resultItem = parser.getFirstParseResult();
        if(resultItem.getSemantic().equals("JavaSrc"))
        { GenerateFile genFile = new GenerateFile(sFileName, this);
          allFileParseResults.add(genFile);
          //parsing and first pass:
          genFile.runFirstPass(resultItem, allClassData, sPathOut);
        }
      }
    }
    else
    { //the file is translated before because it is needed by a type.
      stop();
    }
  }









  void stop()
  { //debug
  }


  public ClassData runRequestedFirstPass(String sClassName)
  { //search whether there is a file with the required name:
    console.reportln(Report.info, "dependent Type need:" + sClassName + "...");
    File fileRequ = null;
    for(File file: listFileIn)
    { String sName = file.getName();
      int posDot = sName.lastIndexOf('.');
      if(posDot > 0){ sName = sName.substring(0,posDot); }
      if(sName.equals(sClassName))
      { fileRequ= file; break;
      }
    }
    if(fileRequ != null)
    { try
      { runFirstPass(fileRequ, null);
        console.report(Report.info, "translated.");
      }
      catch(ParseException exc)
      { console.report(Report.info, "parse exception.");
      }
      catch(Exception exc)
      { console.report(Report.info, "any exception.");
      }
    }
    else
    { console.report(Report.info, "file not found, it may be external.");
    }
    ClassData classData = allClassData.get(sClassName);  //may be null if error
    return classData;
  }

}