package org.vishia.java2C;

import java.util.TreeMap;


/**This class contains all Idents there are visible at a local position of code.
 * It means, visible at class level by generating the class members,
 * visible at level of a block statement including stack variable etc.
 * @author JcHartmut
 *
 */
public class LocalIdents
{
  public static class FieldTypeInfos
  { 
    /**The Type to the ident. */
    public final String sTypeIdent;
    
    public final ClassData typeClazz;
    
    
    /**Kind of the identifier.
     * <ul><li>@ enhanced reference. 
     *     <li>T Type
     * </ul>    
     * */
    public final String sModifier;
    
    /**level of the affiliation to a class in the inherition level.
     * 0= it is a local visible variable. 1=element of this class, 2=element of super class etc.
     */
    private int nClassLevel;

    /**level of the affiliation to a class in the outer class relation.
     * 0= it is a local visible variable. 1=element of this class, 2=element of outer class etc.
     */
    private int nOuterLevel;
    FieldTypeInfos(String sType, ClassData typeClazz, String sModifier, ClassData clazz)
    { this.sTypeIdent = sType; this.sModifier = sModifier; this.clazz = clazz;
      this.typeClazz = typeClazz;
    }
    
    /**The class where the ident is member of, null if local ident. */
    public final ClassData clazz;
    
    int getClassLevel(){ return nClassLevel; }

    int getOuterLevel(){ return nOuterLevel; }
    
    public String toString()
    { String sRet = sTypeIdent + "(" + sModifier + ")" + nClassLevel + nOuterLevel
             + (clazz != null ? clazz.sName : " -stacklocal");
      return sRet;
    }
  }
  
  /**List of known fields of this class. */
  private final TreeMap<String, FieldTypeInfos> fieldIdentifiers = new TreeMap<String, FieldTypeInfos>();

  
  
  LocalIdents()
  {
  }

  
  /**Copies all parent idents as base. */
  LocalIdents(LocalIdents parent)
  { fieldIdentifiers.putAll(parent.fieldIdentifiers);
  }

  FieldTypeInfos get(String sIdent)
  { return fieldIdentifiers.get(sIdent);
  }
  
  void putClassElement(String sIdent, FieldTypeInfos value)
  { fieldIdentifiers.put(sIdent, value);
  }
  
  void putClassElement(String sIdent, String sType, ClassData typeClazz, String sModifier, ClassData clazz)
  { FieldTypeInfos value = new FieldTypeInfos(sType, typeClazz, sModifier, clazz);
    value.nClassLevel = 1;
    value.nOuterLevel = 1;
    fieldIdentifiers.put(sIdent, value);
  }
  
  void putLocalElement(String sIdent, String sType, ClassData typeClazz, String sModifier)
  { FieldTypeInfos value = new FieldTypeInfos(sType, typeClazz, sModifier, null);
    value.nClassLevel = 0;
    value.nOuterLevel = 0;
    fieldIdentifiers.put(sIdent, value);
  }
  
  void putLocalElement(String sIdent, FieldTypeInfos value)
  { fieldIdentifiers.put(sIdent, value);
  }
  
}
