/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 2008-04-06  (year-month-day)
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

import org.vishia.zbnf.ZbnfParseResultItem;

public class SecondPass extends GenerateClass
{

  /**This class helps to return some informations. */
  private static class RetGenStatement
  { }


  /**This class holds the common values of a block statement.
   */
  private class GenBlockStatement
  {
    /**The number of new calles in this blockStatment. */
    int nrofNew = 0; 
  
    boolean lastWasReturn = false;
  
    
    private String gen_statementBlock(ZbnfParseResultItem parent, int indent, LocalIdents localIdentsP, RetGenStatement retInfo)
    throws ParseException
    { String ret = genIndent(indent) + "{ ";
      LocalIdents localIdents = localIdentsP;
  
      List<ClassData.VariablesToInit> variablesToInit = new LinkedList<ClassData.VariablesToInit>();
  
      Iterator<ZbnfParseResultItem> iter = parent.iterChildren("variableDefinition");
      if(iter != null)
      { //if at least one variableDefinition exists, the localIdents should be local:
        localIdents = new LocalIdents(localIdentsP);
        while(iter !=null && iter.hasNext())
        { ZbnfParseResultItem item = iter.next();
          ret += genIndent(indent+1) + gen_variableDefinition(item, localIdents, variablesToInit, 'b');
        }
      }
      String content = "";
      { //Statements of the block. They may contain several new(..) operations.
        //now take all variableDefinitions with intial value and produce statements.
        iter = parent.iterChildren("variableDefinition");
        while(iter !=null && iter.hasNext())
        { ZbnfParseResultItem item = iter.next();
          content += gen_VariableInitAssignment1(item, indent+1, localIdents);
        }
    
        iter = parent.iterChildren("statement");
        while(iter !=null && iter.hasNext())
        { ZbnfParseResultItem item = iter.next();
          content += gen_statement(item, indent+1, localIdents, retInfo);
        }
      }  
      if(nrofNew >0)
      { ret += genIndent(indent+1) + "MemC ";
        int ctNrofNew = 0;
        while(++ctNrofNew < nrofNew){ 
          ret += "mem" + ctNrofNew + ", ";}
        ret += "mem" + ctNrofNew + ";";
      }
      ret += content; 
      if(!lastWasReturn)
      { ret += gen_ActivateGarbageCollection(indent);
      }
      ret += genIndent(indent) + "}";
      return ret;
    }

    
    
    
    private String gen_ActivateGarbageCollection(int indent)
    { String ret = "";
      int ctNrofNew = 0;
      while(++ctNrofNew <= nrofNew)
      { ret += genIndent(indent+1) + "activateGarbageCollectorAccess_BlockHeapJc(mem" + ctNrofNew + ");";
      }
      return ret;
    }
    
    
    
    
  
    private String gen_statement(ZbnfParseResultItem parent, int indent, LocalIdents localIdents, RetGenStatement retInfo)
    throws ParseException
    { String ret = genIndent(indent);
      Iterator<ZbnfParseResultItem> iterStatement= parent.iterChildren();
      //NOTE: a <statement> has only one child because a statement syntax component is a alternative without loop.
      while(iterStatement.hasNext())
      { ZbnfParseResultItem itemStatement = iterStatement.next();
        lastWasReturn = false;  //default, set to true if it is a return.
        String sDescription = get_description(itemStatement);
        if(sDescription != null)
        { ret += "/*" + sDescription + "*/" + genIndent(indent);
        }
        String semanticStatement = itemStatement.getSemantic();
        if(semanticStatement.equals("assignment"))
        { ret += gen_assignment(itemStatement, indent, localIdents);
        }
        else if(semanticStatement.equals("return"))
        { ret += "{ _threadContext->stacktrace = stacktrace.previous;";
          ret += gen_ActivateGarbageCollection(indent+1) + genIndent(indent+1);
          
          ZbnfParseResultItem itemExpr = itemStatement.firstChild();
          if(itemExpr != null && itemExpr.getSemantic().equals("value"))
          { ret += "return " + gen_value(itemExpr, localIdents, 'e') + ";";
          }
          else
          { ret += "return;";
          }
          ret += genIndent(indent) + "}";
          //to indicate that no additional '_threadContext->stacktrace = stacktrace.previous' should be generate:
          lastWasReturn = true;  
        }
        else if(semanticStatement.equals("statementBlock"))
        { GenBlockStatement genBlockStatment = new GenBlockStatement();
          ret += genBlockStatment.gen_statementBlock(itemStatement, indent, localIdents, retInfo);
        }
        else if(semanticStatement.equals("if_statement"))
        { ZbnfParseResultItem itemCondition = itemStatement.getChild("condition");
          String sCondition = gen_value(itemCondition, localIdents, 'e');
          ret += "if(" + sCondition + ")";
          ZbnfParseResultItem ifStatement = itemStatement.getChild("statement");
          String sIfStatement = ifStatement != null ? gen_statement(ifStatement, indent+1, localIdents, retInfo) : ";";
          ret += sIfStatement;
          ZbnfParseResultItem elseStatement = itemStatement.getChild("elseStatement");
          if(elseStatement != null)
          { ret += genIndent(indent) + "else";
            String sElseStatement = gen_statement(elseStatement, indent+1, localIdents, retInfo);
            ret += sElseStatement;
          }
        }
        else if(semanticStatement.equals("switch_statement"))
        { ZbnfParseResultItem itemSwitchValue = itemStatement.getChild("switchValue");
          String sSwitchValue = gen_value(itemSwitchValue, localIdents, 'e');
          ret += "switch(" + sSwitchValue + "){";
          List<ZbnfParseResultItem> listCases = itemStatement.listChildren("case");
          if(listCases != null)for(ZbnfParseResultItem itemCase: listCases)
          { ZbnfParseResultItem zbnfCaseValue = itemCase.getChild("value");
            String sCaseValue = gen_value(zbnfCaseValue, localIdents, 'e');  //should be a constant
            ret += genIndent(indent+1) + "case " + sCaseValue + ":";
            List<ZbnfParseResultItem> zbnfCaseStatements = itemCase.listChildren("statement");
            if(zbnfCaseStatements != null)for(ZbnfParseResultItem zbnfCaseStatement: zbnfCaseStatements)
            { String sCaseStatement = gen_statement(zbnfCaseStatement, indent+1, localIdents, retInfo);
              ret += sCaseStatement;
            }
          }
          ZbnfParseResultItem zbnfDefault = itemStatement.getChild("default");
          if(zbnfDefault != null)
          { String sDefaultStatement = gen_statement(zbnfDefault, indent+1, localIdents, null);
            ret += "default:" + sDefaultStatement;
          }  
          ret += genIndent(indent) + "}/*switch*/;";
        }
        else if(semanticStatement.equals("while_statement"))
        { ZbnfParseResultItem itemCondition = itemStatement.getChild("condition");
          String sCondition = gen_value(itemCondition, localIdents, 'e');
          ret += "while(" + sCondition + ")";
          ZbnfParseResultItem itemWhileStatement = itemStatement.getChild("statement");
          String sWhileStatement = itemWhileStatement != null ? gen_statement(itemWhileStatement, indent+1, localIdents, retInfo) : ";";
          ret += sWhileStatement;
        }
        else if(semanticStatement.equals("dowhile_statement"))
        { ZbnfParseResultItem itemCondition = itemStatement.getChild("condition");
          String sCondition = gen_value(itemCondition, localIdents, 'e');
          ZbnfParseResultItem itemWhileStatement = itemStatement.getChild("statement");
          ret += "do ";
          String sWhileStatement = itemWhileStatement != null ? gen_statement(itemWhileStatement, indent+1, localIdents, retInfo) : ";";
          ret += sWhileStatement;
          ret += "while(" + sCondition + ");";
                    
        }
        else if(semanticStatement.equals("comment"))
        { String sComment = itemStatement.getParsedString();
          ret += "/*" + sComment + "*/" + genIndent(indent);
        }
        else if(semanticStatement.equals("methodCall"))
        { String sMethodCall = gen_simpleValue(itemStatement, localIdents, 'm');
          ret += sMethodCall + ";";
        }
        else if(semanticStatement.equals("throwNew"))
        { String sMethodCall = gen_throwNew(itemStatement, localIdents);
          ret += sMethodCall + ";";
        }
        else if(semanticStatement.equals("break_statement"))
        { ret += "break;";
        }
        else
        { ret += "unknownStatement(); /*unknown statement with semantic: " + semanticStatement + "*/";
          //throw new ParseException("Java2C-ERROR:unknown statement semantic: " + semanticStatement, 0);
        }
      }
      return ret;
    }
  
  
  
  
  
    private String gen_assignment(ZbnfParseResultItem parent, int indent, LocalIdents localIdents) throws ParseException
    { String ret = genIndent(indent);
  
      LocalIdents.FieldTypeInfos typeLeftValue[] = new LocalIdents.FieldTypeInfos[1];
    
      ZbnfParseResultItem item = parent.getChild("leftValue");  //##a
      if(item == null)
      { throw new ParseException("leftValue expected",0);
      }
      { //test
        String sName = item.getChild("variableName").getParsedString();
        if(sName.equals("testStaticReference"))
          stop();
      }
      
      String sVariableC = gen_variable(item, localIdents, '=', typeLeftValue); 
      item = parent.getChild("assignOperator");;
      String sAssignOperatorC = item.getParsedText();
      item = parent.getChild("value");
      String sValueC = gen_value(item, localIdents, 'e');
      
      if(typeLeftValue[0] != null && typeLeftValue[0].sModifier.indexOf('@')>=0)
      { //it is an enhanced reference:
        //ret += "/*" + sVariableC + " =  ...for an enhanced reference*/" + genIndent(indent);
        ret += "if(" + sVariableC + ".ref != null) clearBackRefJc(&(" + sVariableC + ".refbase));" + genIndent(indent);
        ret += sVariableC + ".ref = " + sValueC + ";";
        if(!sValueC.equals("null"))
        { ret += genIndent(indent) + "setBackRefJc(&(";
          //ret += sVariableC + ".refbase), (ObjectJc*)(" + sVariableC + ".ref));";
          ret += sVariableC + ".refbase)/*addr of ref*/, " + sVariableC + ".ref /*The refed Object as void* */);";
        }
        ret += genIndent(indent);
      }
      else
      { //normal assignment, at ex. a numerical value to a int variable
        ret += sVariableC + sAssignOperatorC + sValueC + ";"; 
      }
      return ret;
  
    }
  
  
  
    /**writes a variable definition with syntax attributeDefinition::=.
     *
     * @param parent
     * @throws IOException
     * @throws ParseException
     */
    private String gen_VariableInitAssignment1(ZbnfParseResultItem parent, int indent, LocalIdents idents)
    throws ParseException
    {
      String ret;
  
      //Iterator<ZbnfParseResultItem> iterAttrib = parent.iterChildren("attribute");
      //while(iterAttrib.hasNext())
      ZbnfParseResultItem itemAttrib = parent;
      { //ZbnfParseResultItem itemAttrib = iterAttrib.next();
        String sName = itemAttrib.getChild("name").getParsedString();
        ZbnfParseResultItem assignment = itemAttrib.getChild("value");
        if(assignment != null)
        { String value = gen_value(assignment, idents, 'e');
          ret = genIndent(indent) + sName + " = " + value + ";";
        }
        else if( (assignment = itemAttrib.getChild("newObject")) != null)
        { //an assignment with new...
          ret = genIndent(indent) + sName + " = " + gen_newObject(assignment, idents);
        }
        else
        {
          ret = genIndent(indent) + sName + " = 0; /*assignment not found*/";
        }
      }
      return ret;
    }
  
  
  
    /**generates a variable in C. A Variable may have a reference. If no reference is found,
     * either it is a local variable or it is a variable of the class. The argument localIdents
     * contains the necessary informations to detect the variable identifier in its context.
     * If it is a class variable, <code>ythis-></code> is written before.
     * If it is a referenced variable, the identifier info of the referencing object is used,
     * the reference is build correctly using it (it may be a embedded struct or such).
     * The localIdents of the referencing object are returned in retIdentInfo. 
     * <br>
     * A variable may have a pre- or post- increment or -decrement like --x or y++.
     * 
     * @param itemVariable The ZBNF parse result item of the < ?variable>-semantic.
     *                     It is a part of a simpleValue-syntax-prescript.
     * @param localIdents The Identifier info of the environment.
     * @param intension Info about the location respectively cause to call this method.
     *                  e-expression R-first reference r-nested reference =:leftvalue ...
     * @param retIdentInfo
     * @return generated String of variable access for the C-code
     * @throws ParseException
     */
    private String gen_variable(ZbnfParseResultItem itemVariable, LocalIdents localIdents
        , char intension, LocalIdents.FieldTypeInfos[] retIdentInfo) 
    throws ParseException
    { String ret = "";
      boolean bReferenced;
      if(itemVariable.getChild("preDecrement")!= null)
      { ret +="--"; 
      }
      else if(itemVariable.getChild("preIncrement")!= null)
      { ret +="++";
      }
      ZbnfParseResultItem itemReference = itemVariable.getChild("reference");
      if(itemReference != null)
      { ret += gen_reference(itemReference, localIdents, intension, retIdentInfo);
        localIdents = retIdentInfo[0].typeClazz.classLevelIdents;
        bReferenced = true;
      }
      else
      { bReferenced = intension == 'r';  //true if it is called inside a reference
      }
      String sIdent = itemVariable.getChild("variableName").getParsedString();
      if(sIdent.equals("dvalue"))
        stop();
      //Info about the identifier of the variable in local context:
      LocalIdents.FieldTypeInfos identInfo;  
      int nClassLevel;
      retIdentInfo[0] = identInfo = localIdents.get(sIdent);  //search it.
      //the ident Info should be exists, otherwise an exception is thrown here.
      if(identInfo == null)
      { //it is a referenced variable.
        ret+= sIdent/*unknown ident*/;  
        nClassLevel = 0;
        //throw new ParseException("unknown ident: " + sIdent,0);
      }
      else
      { nClassLevel = identInfo.getClassLevel();
        char cModifier = identInfo.sModifier.charAt(0);
        if(identInfo.clazz != null && "Ssd".indexOf(cModifier)>=0)
        { //the identifier is static or defined with #define
          ret += sIdent + "_" + identInfo.clazz.sName;
        }
        else
        { if(nClassLevel > 0 && !bReferenced)
          { //a variable of the own class should be referenced with 'ythis'.
            ret += "ythis->";
          }
          while(--nClassLevel > 0)
          { ret += "super.";  //member of a super class. It is an embedded struct named 'super'
          }
          int nOuterLevel = retIdentInfo[0].getOuterLevel();
          while(--nOuterLevel > 0)
          { ret += "outer->";  //member of a outer class. It is referenced with 'outer'
          }
          ret += sIdent;
          //TODO test wether it is a local overwritten variable against a class element.
        }
        if("Rrmex".indexOf(intension)>=0 && retIdentInfo[0].sModifier.indexOf('@')>=0)
        { //the element is a enhanced reference, but only the value of reference is needed.
          ret += ".ref";
        }
      }  
      { List<ZbnfParseResultItem> listArrayIndices = itemVariable.listChildren("arrayIndex");
        if(listArrayIndices != null) for(ZbnfParseResultItem item: listArrayIndices)
        {
          ret += "[" + gen_value(item, localIdents, 'e') + "]";
      } }
      if(itemVariable.getChild("postDecrement")!= null)
      { ret +="--"; 
      }
      else if(itemVariable.getChild("postIncrement")!= null)
      { ret +="++";
      }
      return ret;
  
    }
  
  
  
    String gen_reference
    ( ZbnfParseResultItem itemReferences
    , LocalIdents localIdents
    , char intension
    , LocalIdents.FieldTypeInfos[] retIdentInfo
    ) 
    throws ParseException
    { String ret = ""; 
      int countNestedRef = 0; //to test
      Iterator<ZbnfParseResultItem> iterReferences = itemReferences.iteratorChildren();
      intension = 'R';  //the left element of a reference.
      while(iterReferences.hasNext())
      { if(++countNestedRef >= 2)
          stop();
        ZbnfParseResultItem itemRef = iterReferences.next();
        String semantic = itemRef.getSemantic();
        if(semantic.equals("this"))
        { ret += "ythis->";
          retIdentInfo[0] = classData.classTypeInfo;
        }
        else if(semantic.equals("super"))
        { ClassData superClazz = classData.superClazz;
          if(superClazz != null)
          { retIdentInfo[0] = superClazz.classTypeInfo;
            ret += "ythis->super.";
          }
          else
          { ret += "/*super?*/";
          }
        }
        else if(semantic.equals("referenceMethod"))
        { //TODO it is not tested
          assert false;
          ret = gen_simpleMethodCall(itemRef, ret, null /*retIdentInfo*/, null, localIdents);
        }
        else if(semantic.equals("referenceAssociation"))
        { String sReference = gen_variable(itemRef, localIdents, intension, retIdentInfo); //itemRef.getParsedString();
          //NOTE: retIdentInfo[0] = setted in gen_variable with the type info of the detected variable.
          if(sReference.equals("newTarget"))
            stop();
          ret += sReference;
          if(retIdentInfo[0].sModifier.indexOf('$')>=0)
          { //the identifier of this scope is an embedded struct.
            ret += ".";
          }
          else if(retIdentInfo[0].sModifier.indexOf('@')>=0)
          { //the identifier of this scope is an enhanced reference.
            //ret += ".ref->";
            ret += "->";
            stop();
          }
          else
          { assert retIdentInfo[0].sModifier.indexOf('*')>=0;
            //the identifier of this scope is an simple reference.
            ret += "->";
          }
          //The file which contains the definition of the reference type have to be included.
          ClassData typeClass = retIdentInfo[0].typeClazz;
          if(typeClass != null)
          { writeContent.addIncludeC(retIdentInfo[0].typeClazz.sFileName);
            localIdents = typeClass.classLevelIdents;  
          }
          else
          { //a type without typeClass is not recognize in Java2C, it will be an external type.
            localIdents = null;  //there don't have to use in a next nested reference, because the type is unknown. 
            stop();
          }
          intension = 'r'; //for the next nested reference.
        }
      }
      return ret;
    }
  
  
    /**generates the expression to get a value. From syntax item <...?value>.
     * @param parent The parse result item which has the semantic <...?value>.
     * @param localIdents Map of all known identifiers in the localy environment.
     * @param intension intension of generating: 
     *        <ul><li>'e'-value in an expression, enhanced references are taken with .ref
     *            <li>'l'-left value, 
     *            <li>'a'-argument
     *        </ul>    
     * @return
     * @throws ParseException
     */
    private String gen_value(ZbnfParseResultItem parent, LocalIdents localIdents, char intension)
    throws ParseException
    { String expr = ""; //"/*gen_value=" + intension + "*/";
      Iterator<ZbnfParseResultItem> iter= parent.iterChildren();
      String sSemantic;;
      while(iter.hasNext())
      { ZbnfParseResultItem item = iter.next();
        sSemantic = item.getSemantic();
        /**Because the Java.zbnf-script defines a < value> respectively < simpleValue?> in a flatten way,
         * the < simpleValue> is not an extra result item. All items of a value expression are disposed
         * one after another: an optional < unaryOperator> is always followed by a peculiarity of a 
         * < simpleValue>, after them a < binaryOperator> may be followed. 
         * The < conditional> is the last one in this disposal. 
         * But this correct order of items should not be tested here. It is dedicated by the syntax of Java
         * and the syntax of ZBNF script.
         */
        if(sSemantic.equals("unaryOperator"))
        { String operator = item.getParsedText();
          expr += operator;
        }
        else if(sSemantic.equals("binaryOperator"))
        { String operator = item.getParsedText();
          expr += operator;
        }
        else if(sSemantic.equals("conditional"))
        { //it should be the last one in the disposal of parts of value.
          expr += " ? ";
          ZbnfParseResultItem zbnfTrueValue = item.getChild("trueValue");
          String sTrueValue = gen_value(zbnfTrueValue, localIdents, intension);
          expr += sTrueValue;
          ZbnfParseResultItem zbnfFalseValue = item.getChild("falseValue");
          String sFalseValue = gen_value(zbnfFalseValue, localIdents, intension);
          expr += " : ";
          expr += sFalseValue;
        }    
        else //it is any peculiarity of simpleValue, need not: if(semantic.equals("simpleValue"))
        { expr += gen_simpleValue(item, localIdents, intension);
        }
      }
      return expr;
    }
  
  
  
  
  
  
  
  
    /**A simplevalue is a non calculated value, in opposite to a value.
     * But a simpleValue may be an expression accessing a referenced value.
     * Examples are: var, this.var, super.var, ref.var, "xyz", method(a.b),
     *               "xyz".indexof(cc), method().val
     *
     * @param parent simpleValue::=
     * @throws IOException
     * @throws ParseException
     */
    private String gen_simpleValue(ZbnfParseResultItem item, LocalIdents localIdents, char intension)
    throws ParseException
    { String simpleValue = "";
      String extPath = "ythis";
      //ZbnfParseResultItem item = parent.firstChild();
      //Iterator<ZbnfParseResultItem> iter = parent.iterChildren();
      //ZbnfParseResultItem item = iter.next();
  
      LocalIdents.FieldTypeInfos typeLastVariable[] = new LocalIdents.FieldTypeInfos[1];
      String sSemantic = item.getSemantic();
      String sConstantValue = genConstantValue(item);  //simpleStringLiteral, hexValue etc.
      if(sConstantValue != null)
      { simpleValue += sConstantValue;
      }
      else if(sSemantic.equals("casting"))
      { item = item.firstChild();    //<typeIdent>
        String castType = gen_EnvIdent(item, "__");
        simpleValue += "(" + castType + ")";
        item = item.next();                     //<simpleValue>
        simpleValue += gen_value(item, localIdents, 'e');

      }
      else if(sSemantic.equals("parenthesisExpression"))
      {
        simpleValue += "(" + gen_value(item, localIdents, 'e') + ")";

      }
      else if(sSemantic.equals("variable"))
      { simpleValue += gen_variable(item, localIdents, intension, typeLastVariable);
        if(typeLastVariable[0] != null && typeLastVariable[0].clazz != null)
        { //to use for the next, referenced variable.
          localIdents = typeLastVariable[0].clazz.classLevelIdents;
        }
        else
        { //there should be no next referenced variable!
          localIdents = null;
        }
      }
      /*
      else if(sSemantic.equals("simpleVariable"))
      { simpleValue += "" + item.getParsedString();
      }
      else if(sSemantic.equals("superAccess"))
      { simpleValue += "?superAccess";
      }
      */
      else if(sSemantic.equals("newObject"))
      { simpleValue += gen_newObject(item, localIdents);;
      }
      else if(sSemantic.equals("assignment"))
      { simpleValue += "?assignment";
      }
      else if(sSemantic.equals("xxxreferenceOperator"))
      { simpleValue += "->";
      }
      else if(sSemantic.equals("methodCall"))
      { 
        { String sMethodName = item.getChild("methodName").getParsedString();
          if(sMethodName.equals("run"))
            stop();
        }
        ZbnfParseResultItem itemReference = item.getChild("reference");
        String sInstanceRef;
        if(itemReference != null)
        { //LocalIdents.FieldTypeInfos[] retIdentInfo = new LocalIdents.FieldTypeInfos[1];
          sInstanceRef = gen_reference(itemReference, localIdents, 'm', typeLastVariable); // retIdentInfo);
          ClassData typeClazz = typeLastVariable[0].typeClazz;
          //the type of a reference should be known.
          if(typeClazz == null)
          { //localIdents = null;  //unknown type because external reference
            //throw new ParseException("methodCall: unknown typeClazz from referencetype " + retIdentInfo[0].sTypeIdent, 0);
          }
          //else{ localIdents = retIdentInfo[0].typeClazz.classLevelIdents; }
        }
        else
        { //no reference before method call, either it is a static method or this-method.
          sInstanceRef = "ythis";
          localIdents = classData.classLevelIdents;
        }
        //if it is a static method, the sInstance will be ignored.
        simpleValue = gen_simpleMethodCall
                      ( item
                      , sInstanceRef
                      , typeLastVariable[0]
                      , classData       //it should be the instance type class!
                      , localIdents     //used for method arguments. 
                      );
      }
      else if(sSemantic.equals("xxxexternObject"))
      { simpleValue += "?externObject";
      }
      else
      { throw new ParseException("unexpected semantic:" + sSemantic,0);
      }
      return simpleValue;
    }
  
  
  
  
  
    private String gen_simpleMethodCall(ZbnfParseResultItem parent, String sInstanceRef, LocalIdents.FieldTypeInfos methodEnvType, ClassData envClass, LocalIdents localIdents)
    throws ParseException
    { String ret;
      boolean bEmbeddedType = false;
      String sMethodEnvType;
      if(methodEnvType != null) // it is a implicit this access
      { //The reference type is defined in an other class, include it!
        boolean bTypeSign = true;
        switch(methodEnvType.sModifier.charAt(0))
        { case '$': bEmbeddedType = true; break;
          default: bTypeSign = false;
        }
        if(bTypeSign)
        { //sMethodEnvType = sMethodEnvType.substring(1);
        }
        sMethodEnvType = methodEnvType.sTypeIdent;
        /*
        int posEndBaseType = sMethodEnvType.indexOf("__");
        if(posEndBaseType < 0){ posEndBaseType = sMethodEnvType.length(); }
        writeContent.addIncludeC(sMethodEnvType.substring(0, posEndBaseType));
        */
        //writeContent.writeIncludeC("\n#include \"" + sMethodEnvType.substring(0, posEndBaseType) + ".h\"" );
      }
      else
      { sMethodEnvType = classData.sName;
      }
      String sMethodName = parent.getChild("methodName").getParsedString();
      if(sMethodName.equals("run"))
        stop();
      ret = sMethodName + "_" + sMethodEnvType + "(";
      if(sInstanceRef.endsWith("."))
      //if(bEmbeddedType)
      { sInstanceRef = sInstanceRef.substring(0, sInstanceRef.length()-1);
        ret += "&(" + sInstanceRef + ")";   //envValue in Java is envValue.methodName(...), in C it is a value, not reference.
      }
      else
      { sInstanceRef = sInstanceRef.substring(0, sInstanceRef.length()-2);
        ret += sInstanceRef;
      }
      ZbnfParseResultItem args = parent.getChild("actualArguments");
      if(args != null)
      { Iterator<ZbnfParseResultItem> iterArgs = args.iterChildren();
        while(iterArgs.hasNext())
        { ZbnfParseResultItem arg = iterArgs.next();
          String argSemantic = arg.getSemantic();
          if(argSemantic.equals("value"))
          { ret += ", " + gen_value(arg, localIdents, 'a');
          }
          else if(argSemantic.equals("objectAccess"))
          { ret += ", " + "?objectAccess";
          }
          else
          { ret +=", " + "?unknown:" + argSemantic;
          }
        }
      }
      ret +=", _threadContext)";
      return ret;
    }
  
  
  
    
    /**creates the statement for a new Type(...) expression.
     * 
     * @param sName
     * @param assignment
     * @param indent
     * @param idents
     * @return
     * @throws ParseException 
     */
    private String gen_newObject(ZbnfParseResultItem assignment, LocalIdents idents) 
    throws ParseException
    {
      String ret;
      String sTypeNewObject = gen_EnvIdent(assignment.getChild("className"), "__");  //itemNewObject.getChild("className").getParsedString();
      { //induce to generate the necesarry include statement:
        ClassData typeOfNew = allClassData.get(sTypeNewObject);        
        writeContent.addIncludeC(typeOfNew.sFileName);
        
        
      
      }
      nrofNew +=1;
      ret  = "/*new " + sTypeNewObject + "(...)*/";
      ret += "ctor_" + sTypeNewObject;
      ret += "(mem"+nrofNew+" = alloc_s_BlockHeapJc(sizeof(" + sTypeNewObject + "))";
      
      ZbnfParseResultItem itemActArguments = assignment.getChild("actualArguments");
      if(itemActArguments != null)
      { List<ZbnfParseResultItem> listArgs = itemActArguments.listChildren("value");
        for(ZbnfParseResultItem itemArg: listArgs)
        { ret += ", " + gen_value(itemArg, idents, 'e');
        }
        stop();
      }
      
      
      //TODO args of new
      ret += ", _threadContext);";
      return ret;
    }
  
  

    private String gen_throwNew(ZbnfParseResultItem parent, LocalIdents localIdents)
    throws ParseException
    { String expr = "";
  
      String sExceptionClass = parent.getChild("exceptionClass").getParsedString();
      ZbnfParseResultItem itemText = parent.getChild("text");
      if(itemText != null)
      { String sText = itemText.getParsedString();
        expr = "throw_s0Jc(ident_" + sExceptionClass + "Jc, \"" + sText + "\", ";
      }
      else
      { expr = "THROW_s(" + sExceptionClass + ", xxx, "  ;
      }
      ZbnfParseResultItem itemValue = parent.getChild("value");
      String sValue = itemValue != null ? gen_value(itemValue, localIdents, 'a') : "0";
      expr += sValue + ", _threadContext, __LINE__)";
      return expr;
    }
  
  
  
  
  
  
  
  
  }//class GenBlockStatement
  
  
  
  
  

  SecondPass(iWriteContent writeContent, TreeMap<String, ClassData> allClassData, ClassData classData, RunRequiredFirstPass_ifc runRequiredFirstPass)
  { super(writeContent, allClassData, runRequiredFirstPass);
    this.classData = classData;
  }


  /**Generates the content of a class in c- and h-file. */
  void runSecondPass(ZbnfParseResultItem itemClass, String sOuterClassName)
  throws IOException, ParseException
  {
    ZbnfParseResultItem itemDescription = itemClass.getChild("description");
    if(itemDescription != null)
    { write_Description(itemDescription);
    }

    String sClassName = classData.getName();

    for(ClassData.VariablesToInit staticVariable : classData.staticVariables)
    { String sName = staticVariable.sName;
      String sType = staticVariable.sType;
      char kind = staticVariable.kind;
      String sStaticDef = sType + (kind=='@' ? "REF " : " ") + sName + "_" + sClassName;
      if(staticVariable.itemValue == null)
      { //no init value
        if("@".indexOf(kind)>=0)
        { //embedded struct or enhanced reference, null-initializing
          sStaticDef += " = { 0, null };";
        }
        else if("$".indexOf(kind)>=0)
        { //embedded struct or enhanced reference, null-initializing
          sStaticDef += " = { 0 };";
        }
        else if("*".indexOf(kind)>=0)
        { //normal reference, null-initializing
          sStaticDef += " = null;";
        }
        else
        { sStaticDef += "= 0;";
        }
      }
      else
      { if(kind == '$')
        { //embedded struct, call initializer
        }
        else if("@*%".indexOf(kind)>=0)
        { //all others
          //String sValue = genBlockStatement.gen_value(variableToInit.itemValue, classData.classLevelIdents, 'e');
          //writeContent.writeClassC("\n  ythis->" + sName + " = " + sValue + "; /*" + sType + "*/");
        }
        else
        { throw new ParseException("unknown kind of static variable:" + kind, 0);
        }
      }
      writeContent.writeClassC("\n" + sStaticDef);      
    }
    
    
    List<ZbnfParseResultItem> listConstructors = itemClass.listChildren("constructorDefinition");
    if(listConstructors != null)
    { for(ZbnfParseResultItem itemConstructor : listConstructors)
      { write_constructorDefinition(itemConstructor, itemClass, sClassName);
      }
    }
    else
    { //no constructor found, create a default constructor because all variable initializations should be done.
      write_constructorDefinition(null, itemClass, sClassName);
    }

    Iterator<ZbnfParseResultItem> iterMethod = itemClass.iterChildren("methodDefinition");
    if(iterMethod != null)
    { while(iterMethod.hasNext())
      { ZbnfParseResultItem itemMethod = iterMethod.next();
        write_methodDefinition(itemMethod, sClassName);
    } }

    /*NOTE: classDefinition inside a class are translated because runFirstPass dissolve the inner class
     *      and the second pass for the inner class is called independent.
     */
    { List<ZbnfParseResultItem> listSubClasses = itemClass.listChildren("xxxclassDefinition");
      if(listSubClasses != null)
      { for(ZbnfParseResultItem itemSubClasses : listSubClasses)
        { runSecondPass(itemSubClasses, sClassName + "__");
      } }
    }
  }

  /**writes a methodDefinition with syntax methodDefinition::=.
   *
   * @param itemConstructor if null than create an default constructor with all initializations.
   * @throws IOException
   * @throws ParseException
   */
  private void write_constructorDefinition(ZbnfParseResultItem itemConstructor, ZbnfParseResultItem itemClass, String sClassName)
  throws IOException, ParseException
  { LocalIdents localIdents = new LocalIdents(classData.classLevelIdents);
    String sMethodHead = "\n\n" + sClassName + "* ctor_" + sClassName + "(MemC rawMem";

    if(itemConstructor != null)
    { Iterator<ZbnfParseResultItem> iterArguments = itemConstructor.iterChildren("argument");
      if(iterArguments != null)
      { while(iterArguments.hasNext())
        { ZbnfParseResultItem itemArgument = iterArguments.next();
          String sArg = gen_variableDefinition(itemArgument, localIdents, null, 'a');
          sMethodHead += ", " + sArg;
        }
      }
    }
    sMethodHead += ", StacktraceThreadContext_s* _threadContext)";
    writeContent.writeClassC(sMethodHead);

    writeContent.writeClassC("\n{ " + sClassName + "* ythis = (" + sClassName + "*)rawMem.address;");
    writeContent.writeClassC("\n  StacktraceJc stacktrace = {_threadContext->stacktrace, { \"ctor_" + sClassName + "\", __FILE__, __LINE__} , null}; ");
    writeContent.writeClassC("\n  _threadContext->stacktrace = &stacktrace;");
    writeContent.writeClassC("\n  if(size_MemC(rawMem) < sizeof(" + sClassName + "))");
    writeContent.writeClassC("\n  { throw_s0Jc(ident_RuntimeExceptionJc, \"ctor mem-size insufficient\", size_MemC(rawMem), _threadContext, __LINE__);");
    writeContent.writeClassC("\n  }");
    writeContent.writeClassC("\n  memset(ythis, 0, sizeof(*ythis));  //initial all class values are 0 per default.");

    //TODO: super(xyz)
    writeContent.writeClassC("\n  ctor_ObjectJc(rawMem);  ");
    writeContent.writeClassC("\n  //TODO setReflection_ObjectJc(&reflection" + sClassName + ");  ");
    
    GenBlockStatement genBlockStatement = new GenBlockStatement();
    
    for(ClassData.VariablesToInit variableToInit : classData.variablesToInit)
    { String sName = variableToInit.sName;
      String sType = variableToInit.sType;
      char kind = variableToInit.kind;
      if(kind == '$')
      { //embedded struct, call constructor
        writeContent.writeClassC("\n  { MemC mem" + sName + "= { sizeof(" + sType
                                + "), (MemAreaC*)(&ythis->" + sName + ") };");
        writeContent.writeClassC("\n    ctor_" + sType + "(mem" + sName);
        //TODO args for constructor

        writeContent.writeClassC(", _threadContext);");
        writeContent.writeClassC("\n  }");
      }
      else if(kind == 's')
      { //initialize static variable not here.
        stop();
      }
      else if(kind == 'S')
      { //final static ... do not initialize here, but at .c-level.
        stop();
      }
      else if("@*%".indexOf(kind)>=0)
      { //all others
        String sValue = genBlockStatement.gen_value(variableToInit.itemValue, classData.classLevelIdents, 'e');
        writeContent.writeClassC("\n  ythis->" + sName + " = " + sValue + "; /*" + sType + "*/");
      }
      else
      { throw new ParseException("unknown kind of variable to init:" + kind, 0);
      }
    }


    if(itemConstructor != null)
    { GenBlockStatement genBlockStatment = new GenBlockStatement();
      String sBody = genBlockStatment.gen_statementBlock(itemConstructor, 1, localIdents, new RetGenStatement());  //it is a statementBlock
      writeContent.writeClassC(sBody);
    }
    writeContent.writeClassC("\n _threadContext->stacktrace = stacktrace.previous;");
    writeContent.writeClassC("\n  return ythis;\n}\n\n");
  }


  /**writes a methodDefinition with syntax methodDefinition::=.
   *
   * @param parent
   * @throws IOException
   * @throws ParseException
   */
  private void write_methodDefinition(ZbnfParseResultItem parent, String sClassName)
  throws IOException, ParseException
  { LocalIdents localIdents = new LocalIdents(classData.classLevelIdents);
    ZbnfParseResultItem itemType = parent.getChild("typeIdent");
    String sType = gen_EnvIdent(itemType, "__");
    String sName = parent.getChild("name").getParsedString();
    String sMethodHead = "\n\n" + sType + " " + sName + "_" + sClassName + "(" + sClassName + "* ythis";
    //
    Iterator<ZbnfParseResultItem> iterArguments = parent.iterChildren("argument");
    if(iterArguments != null)
    { while(iterArguments.hasNext())
      { ZbnfParseResultItem itemArgument = iterArguments.next();
        String sArg = gen_variableDefinition(itemArgument, localIdents, null, 'a');
        sMethodHead += ", " + sArg;

        /*ZbnfParseResultItem itemArgument = iterArguments.next();
        String sArgType = gen_EnvIdent(itemArgument.getChild("typeIdent"), "__");
        String sArgName = itemArgument.getChild("name").getParsedString();
        sMethodHead += ", " + sArgType + " " +sArgName;
        localIdents.put(sArgName, sArgType);
        */
      }
    }
    sMethodHead += ", StacktraceThreadContext_s* _threadContext)";
    writeContent.writeClassC(sMethodHead);
    writeContent.writeClassC("\n{  StacktraceJc stacktrace = {_threadContext->stacktrace, { \"" + sName + "_" + sClassName + "\", __FILE__, __LINE__} , null}; ");
    writeContent.writeClassC("\n  _threadContext->stacktrace = &stacktrace;");

    ZbnfParseResultItem itemBody = parent.getChild("methodbody");
    if(itemBody != null)
    { RetGenStatement retInfo = new RetGenStatement();
      GenBlockStatement genBlockStatment = new GenBlockStatement();
      String sBody = genBlockStatment.gen_statementBlock(itemBody, 1, localIdents, retInfo);  //it is a statementBlock
      writeContent.writeClassC(sBody);
      if(!genBlockStatment.lastWasReturn)
      { writeContent.writeClassC("\n  _threadContext->stacktrace = stacktrace.previous;");
      }
    }
    writeContent.writeClassC("\n}\n\n");
  }








}
